block ControlLogic_fmu "Satellite controller"
  parameter String instanceName_ = getInstanceName() "Name of instance used for logging" annotation(Dialog(tab = "FMU"));
  parameter Boolean debugLogging_ = false "Enable FMU debug logging (aka \"loggingOn\")" annotation(Dialog(tab = "FMU"));
  parameter Integer fmilLogLevel_ = 3 "FMI Library debug logging level" annotation(Dialog(tab = "FMU"), choices(choice = 0 "Nothing", choice = 1 "Fatal", choice = 2 "Error", choice = 3 "Warning", choice = 4 "Info", choice = 5 "Verbose", choice = 6 "Debug", choice = 7 "All"));
  parameter Boolean complianceChecking_ = true "Enable FMU wrapper self-testing of FMI compliance" annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  parameter Boolean visible_ = false "Show FMU simulator application window" annotation(Dialog(tab = "FMU"));
  constant Boolean useTriggerInput_ = true "Determine communication steps using external trigger signal" annotation(Dialog(tab = "Co-Simulation"));
  Interfaces_.Boolean_DiscreteInput stepTrigger_ if useTriggerInput_ annotation(Placement(visible = true, transformation(origin = {-20, 100}, extent = {{-20, -10}, {0, 10}}, rotation = -90), iconTransformation(origin = {-50, 100}, extent = {{-40, -20}, {0, 20}}, rotation = -90)));
  parameter Boolean logEvents_ = false "Log all events (\"logEvents\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  parameter Boolean logDynamicStateSelection_ = false "Log the dynamic selection of states (\"logDynamicStateSelection\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  parameter Boolean logDebug_ = false "Log debug messages (\"logDebug\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  parameter Boolean logStatusWarning_ = false "Log when functions return fmi2Warning (\"logStatusWarning\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  parameter Boolean logStatusDiscard_ = false "Log when functions return fmi2Discard (\"logStatusDiscard\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  parameter Boolean logStatusError_ = true "Log when functions return fmi2Error (\"logStatusError\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  parameter Boolean logStatusFatal_ = true "Log when functions return fmi2Fatal (\"logStatusFatal\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  parameter Boolean logLevelVerbose_ = false "Enable verbose logging for selected log categories (\"logLevelVerbose\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  parameter Boolean logLevelDebug_ = false "Enable debug logging for selected log categories (\"logLevelDebug\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  parameter Boolean logAll_ = false "Log all messages (\"logAll\")" annotation(Dialog(tab = "FMU", group = "Logging categories"));
  final parameter String[:] categories_ = {if logEvents_ then "logEvents" else "", if logDynamicStateSelection_ then "logDynamicStateSelection" else "", if logDebug_ then "logDebug" else "", if logStatusWarning_ then "logStatusWarning" else "", if logStatusDiscard_ then "logStatusDiscard" else "", if logStatusError_ then "logStatusError" else "", if logStatusFatal_ then "logStatusFatal" else "", if logLevelVerbose_ then "logLevelVerbose" else "", if logLevelDebug_ then "logLevelDebug" else "", if logAll_ then "logAll" else ""} annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  parameter Modelica.Units.SI.Time step_(min = 0.0, start = -1.0) "Communication step size, required unless useTriggerInput_" annotation(Dialog(enable = not useTriggerInput_, tab = "Co-Simulation"));
  parameter Real tolerance_ = -1.0 "Solver tolerance (negative means master solver tolerance)" annotation(Dialog(tab = "Co-Simulation"));
  fmu_record_0_ target_force(k = 0.0);
  fmu_record_0_ target_height(k = 7500000.0);
  fmu_record_11_ controller(D(y_start = 0.0), Izero(k = 0.0), Nd = 10.0, Ni = 0.90000000000000002, P(k = 1.0), Td = 2000.0, Ti = 0.5, addD(k2 = -1.0), addP(k2 = -1.0), addPID(k1 = 1.0, k2 = 1.0, k3 = 1.0), k = 10.0, wd = 0.0, wp = 1.0, xd_start = 0.0, xi_start = 0.0, yMax = 1000000.0, y_start = 0.0);
  fmu_record_11_ forward_controller(D(y_start = 0.0), Izero(k = 0.0), Nd = 10.0, Ni = 0.90000000000000002, P(k = 1.0), Td = 1000.0, Ti = 0.5, addD(k2 = -1.0), addP(k2 = -1.0), addPID(k1 = 1.0, k2 = 1.0, k3 = 1.0), k = 0.10000000000000001, wd = 0.0, wp = 1.0, xd_start = 0.0, xi_start = 0.0, yMax = 1000000.0, y_start = 0.0);
  fmu_record_12_ launchSwitch(startTime = 1700.0, startValue = false);
  fmu_record_15_ satelliteLaunchControl1(add1(k1 = 1.0, k2 = 1.0), add2(k1 = 1.0, k2 = 1.0), add3(k1 = 1.0, k2 = 1.0), launch_force_x(offset = -2319344.3389588799, startTime = 500.0), launch_force_x1(offset = -1348580.9065405, startTime = 1700.0), launch_force_y(offset = -726055.60277378, startTime = 500.0), launch_force_y1(offset = 1966024.5383319899, startTime = 1700.0), launch_force_z(offset = 5485752.9199755397, startTime = 500.0), launch_force_z1(offset = -309962.98680795199, startTime = 1700.0));
  fmu_record_5_ altitudeConverter1(const(k = 0.0), directionConverter1(satelliteNormalize1(add31(k1 = 1.0, k2 = 1.0, k3 = 1.0))));
  fmu_record_5_ altitudeConverter2(const(k = 0.0), directionConverter1(satelliteNormalize1(add31(k1 = 1.0, k2 = 1.0, k3 = 1.0))));
  fmu_record_7_ gain1(k = -1.0);
  Interfaces_.Real_ContinuousInput[3] direction(each start = 0.0) annotation(Placement(visible = true, transformation(origin = {-145, 60}, extent = {{-20, -10}, {0, 10}}, rotation = 0), iconTransformation(origin = {-100, 50}, extent = {{-40, -20}, {0, 20}}, rotation = 0)));
  Interfaces_.Real_ContinuousInput[3] position(each start = 0.0) annotation(Placement(visible = true, transformation(origin = {-145, 0}, extent = {{-20, -10}, {0, 10}}, rotation = 0), iconTransformation(origin = {-100, 0}, extent = {{-40, -20}, {0, 20}}, rotation = 0)));
  Interfaces_.Real_ContinuousOutput[3] y annotation(Placement(visible = true, transformation(origin = {145, 0}, extent = {{0, -5}, {10, 5}}, rotation = 0), iconTransformation(origin = {100, 0}, extent = {{0, -10}, {20, 10}}, rotation = 0)));
  Interfaces_.Real_ContinuousInput height(start = 0.0) annotation(Placement(visible = true, transformation(origin = {-145, -60}, extent = {{-20, -10}, {0, 10}}, rotation = 0), iconTransformation(origin = {-100, -50}, extent = {{-40, -20}, {0, 20}}, rotation = 0)));
protected
  package Interfaces_
    connector Real_ContinuousInput = input Real "'input (continuous) Real' as connector" annotation(defaultComponentName = "u", Icon(graphics = {Polygon(visible = true, lineColor = {1, 37, 163}, fillColor = {1, 37, 163}, fillPattern = FillPattern.Solid, points = {{0, 20}, {40, 0}, {0, -20}})}, coordinateSystem(extent = {{0, -20}, {40, 20}}, preserveAspectRatio = true, initialScale = 1, grid = {2, 2})), Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 1, extent = {{-20, -10}, {0, 10}}, grid = {2, 2}), graphics = {Polygon(visible = true, lineColor = {1, 37, 163}, fillColor = {1, 37, 163}, fillPattern = FillPattern.Solid, points = {{-20, 10}, {0, 0}, {-20, -10}}), Text(visible = true, textColor = {0, 0, 0}, extent = {{-10, 11}, {-10, 16}}, horizontalAlignment = TextAlignment.Right, textString = "%name")}));
    connector Real_ContinuousOutput = output Real "'output (continuous) Real' as connector" annotation(defaultComponentName = "u", Icon(graphics = {Polygon(visible = true, lineColor = {1, 37, 163}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 0.5, points = {{-20, 10}, {0, 0}, {-20, -10}})}, coordinateSystem(extent = {{-20, -10}, {0, 10}}, preserveAspectRatio = true, initialScale = 1, grid = {1, 1})), Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 1, extent = {{0, -5}, {10, 5}}, grid = {1, 1}), graphics = {Polygon(visible = true, origin = {10, 0}, lineColor = {1, 37, 163}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-10, 5}, {0, 0}, {-10, -5}}), Text(visible = true, extent = {{5, 6}, {5, 11}}, horizontalAlignment = TextAlignment.Left, textString = "%name")}));
    connector Boolean_DiscreteInput = input Boolean "'input Boolean' as connector" annotation(defaultComponentName = "u", Icon(graphics = {Polygon(visible = true, lineColor = {190, 52, 178}, fillColor = {190, 52, 178}, fillPattern = FillPattern.Solid, points = {{0, 20}, {40, 0}, {0, -20}})}, coordinateSystem(extent = {{0, -20}, {40, 20}}, preserveAspectRatio = true, initialScale = 1, grid = {2, 2})), Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 1, extent = {{-20, -10}, {0, 10}}, grid = {2, 2}), graphics = {Polygon(visible = true, lineColor = {190, 52, 178}, fillColor = {190, 52, 178}, fillPattern = FillPattern.Solid, points = {{-20, 10}, {0, 0}, {-20, -10}}), Text(visible = true, textColor = {0, 0, 0}, extent = {{-10, 11}, {-10, 16}}, horizontalAlignment = TextAlignment.Right, textString = "%name")}));
    connector Boolean_DiscreteOutput = output Boolean "'output Boolean' as connector" annotation(defaultComponentName = "u", Icon(graphics = {Polygon(visible = true, lineColor = {190, 52, 178}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 0.5, points = {{-20, 10}, {0, 0}, {-20, -10}})}, coordinateSystem(extent = {{-20, -10}, {0, 10}}, preserveAspectRatio = true, initialScale = 1, grid = {1, 1})), Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 1, extent = {{0, -5}, {10, 5}}, grid = {1, 1}), graphics = {Polygon(visible = true, origin = {10, 0}, lineColor = {190, 52, 178}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-10, 5}, {0, 0}, {-10, -5}}), Text(visible = true, extent = {{5, 6}, {5, 11}}, horizontalAlignment = TextAlignment.Left, textString = "%name")}));
  end Interfaces_;

  package Terminals_
    block Boolean_DiscreteInputTerminal "Block with single input to be used in the equations of the containing model"
      extends Icons_.InputTerminal;
      Interfaces_.Boolean_DiscreteInput u annotation(Placement(visible = true, transformation(origin = {0, 0}, extent = {{-20, -10}, {0, 10}}, rotation = 0), iconTransformation(origin = {-3, 0}, extent = {{-4, -2}, {0, 2}}, rotation = 0)));
      annotation(Icon(coordinateSystem(extent = {{-5, -5}, {5, 5}}, preserveAspectRatio = true, initialScale = 1, grid = {1, 1})), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, grid = {5, 5})), preferredView = "icon");
    end Boolean_DiscreteInputTerminal;

    block Boolean_DiscreteOutputTerminal "Block with single output for exposing a variable of the containing model"
      extends Icons_.OutputTerminal;
      Interfaces_.Boolean_DiscreteOutput y annotation(Placement(visible = true, transformation(origin = {0, 0}, extent = {{0, -5}, {10, 5}}, rotation = 0), iconTransformation(origin = {3, 0}, extent = {{0, -1}, {2, 1}}, rotation = 0)));
      annotation(Icon(coordinateSystem(extent = {{-5, -5}, {5, 5}}, preserveAspectRatio = true, initialScale = 1, grid = {1, 1})), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, grid = {5, 5})), preferredView = "icon");
    end Boolean_DiscreteOutputTerminal;
  end Terminals_;

  package Types_
    type Modelica_Blocks_Types_Init = enumeration(NoInit, SteadyState, InitialState, InitialOutput);
    type Modelica_Blocks_Types_InitPID = enumeration(NoInit, SteadyState, InitialState, InitialOutput, DoNotUse_InitialIntegratorState);
    type Modelica_Blocks_Types_SimpleController = enumeration(P, PI, PD, PID);
  end Types_;

  package Icons_
    partial block BooleanDiscreteBlock "Basic graphical layout of Boolean block"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-10, -10}, {10, 10}}, initialScale = 1, grid = {1, 1}), graphics = {Rectangle(extent = {{-10, -10}, {10, 10}}, lineColor = {190, 52, 178}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 0.5), Text(extent = {{-15, 11}, {15, 16}}, textString = "%name", lineColor = {0, 0, 0})}), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, grid = {5, 5})), preferredView = "icon");
    end BooleanDiscreteBlock;

    block Terminal "Icon for terminal blocks"
      annotation(Icon(coordinateSystem(extent = {{-5, -5}, {5, 5}}, preserveAspectRatio = true, initialScale = 1, grid = {1, 1}), graphics = {Ellipse(visible = true, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 0.1, extent = {{-3, -3}, {3, 3}}), Line(visible = true, points = {{-2.8532, -0.9271}, {2.5441, 1.5898}}, thickness = 0.05), Line(visible = true, points = {{-2.5441, -1.5898}, {2.8532, 0.9271}}, thickness = 0.05)}), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, grid = {5, 5})), preferredView = "icon");
    end Terminal;

    block InputTerminal "Icon for input terminal blocks"
      annotation(Icon(coordinateSystem(extent = {{-5, -5}, {5, 5}}, preserveAspectRatio = true, initialScale = 1, grid = {1, 1}), graphics = {Ellipse(visible = true, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 0.1, extent = {{-3, -3}, {3, 3}}), Line(visible = true, points = {{-2.8532, -0.9271}, {2.5441, 1.5898}}, thickness = 0.05), Line(visible = true, points = {{-2.5441, -1.5898}, {2.8532, 0.9271}}, thickness = 0.05), Text(visible = true, origin = {-3, 6}, textColor = {0, 0, 0}, extent = {{0, 0}, {0, 5}}, horizontalAlignment = TextAlignment.Left, textString = "%name")}), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, grid = {5, 5})), preferredView = "icon");
    end InputTerminal;

    block OutputTerminal "Icon for out terminal blocks"
      annotation(Icon(coordinateSystem(extent = {{-5, -5}, {5, 5}}, preserveAspectRatio = true, initialScale = 1, grid = {1, 1}), graphics = {Ellipse(visible = true, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 0.1, extent = {{-3, -3}, {3, 3}}), Line(visible = true, points = {{-2.8532, -0.9271}, {2.5441, 1.5898}}, thickness = 0.05), Line(visible = true, points = {{-2.5441, -1.5898}, {2.8532, 0.9271}}, thickness = 0.05), Text(visible = true, origin = {3, 6}, textColor = {0, 0, 0}, extent = {{0, 0}, {0, 5}}, horizontalAlignment = TextAlignment.Right, textString = "%name")}), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, grid = {5, 5})), preferredView = "icon");
    end OutputTerminal;
  end Icons_;
protected
  constant String fmuResource_ = Modelica.Utilities.Files.loadResource("modelica://ControlLogic_fmu/Resources") annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  FMUFunctions_.FMUModel fmu_ = FMUFunctions_.FMUModel(fmuResource_, instanceName_, fmilLogLevel_, debugLogging_, categories_, complianceChecking_, visible_, tolerance_) annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  discrete Real[3] out_c_real_ annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  discrete Real[10] p_real_ annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  discrete Real[10] p_real1_ annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  discrete Real[4] p_real2_ annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  Boolean[10] p_bool_ annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  Boolean[4] p_bool1_ annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  Integer[6] p_enum_(each start = 1, each fixed = true) annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  parameter Modelica.Units.SI.Time startTime_(fixed = false) annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  discrete Modelica.Units.SI.Time currentCommunicationPoint_ annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  Boolean terminateSimulation_(start = false, fixed = true) annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  Boolean stepDone_(start = false, fixed = true) annotation(Dialog(__Wolfram_show = false, enable = false, tab = "Wolfram Internal", group = "Internal"));
  Modelica.Blocks.Sources.SampleTrigger sampleTrigger_(period = step_, startTime = startTime_ + step_) if not useTriggerInput_ annotation(Placement(visible = true, transformation(origin = {-50, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Terminals_.Boolean_DiscreteInputTerminal stepTriggerInternal_ annotation(Placement(visible = true, transformation(origin = {0, 90}, extent = {{-5, -5}, {5, 5}}, rotation = 0), iconTransformation(origin = {6.558, -7.403}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
initial algorithm
  assert(useTriggerInput_ or step_ > 0, "Communication step size must be positive.");
  FMUFunctions_.wsmSetReal(fmu_, {225, 229, 230, 231, 232, 236, 237, 238, 243, 245, 259, 260, 246, 261, 262, 248, 250, 251, 252, 253, 265, 268, 271, 272, 273, 274, 276, 281, 283, 297, 298, 284, 299, 300, 286, 288, 289, 290, 291, 303, 306, 309, 310, 311, 312, 314, 315, 316, 318, 319, 320, 321, 322, 323, 325, 326, 328, 329, 331, 332, 334, 335, 337, 338, 340, 341, 342, 343}, {altitudeConverter1.const.k, altitudeConverter1.directionConverter1.satelliteNormalize1.add31.k1, altitudeConverter1.directionConverter1.satelliteNormalize1.add31.k2, altitudeConverter1.directionConverter1.satelliteNormalize1.add31.k3, altitudeConverter2.const.k, altitudeConverter2.directionConverter1.satelliteNormalize1.add31.k1, altitudeConverter2.directionConverter1.satelliteNormalize1.add31.k2, altitudeConverter2.directionConverter1.satelliteNormalize1.add31.k3, controller.D.y_start, controller.Izero.k, controller.Nd, controller.Ni, controller.P.k, controller.Td, controller.Ti, controller.addD.k2, controller.addP.k2, controller.addPID.k1, controller.addPID.k2, controller.addPID.k3, controller.k, controller.wd, controller.wp, controller.xd_start, controller.xi_start, controller.yMax, controller.y_start, forward_controller.D.y_start, forward_controller.Izero.k, forward_controller.Nd, forward_controller.Ni, forward_controller.P.k, forward_controller.Td, forward_controller.Ti, forward_controller.addD.k2, forward_controller.addP.k2, forward_controller.addPID.k1, forward_controller.addPID.k2, forward_controller.addPID.k3, forward_controller.k, forward_controller.wd, forward_controller.wp, forward_controller.xd_start, forward_controller.xi_start, forward_controller.yMax, forward_controller.y_start, gain1.k, launchSwitch.startTime, satelliteLaunchControl1.add1.k1, satelliteLaunchControl1.add1.k2, satelliteLaunchControl1.add2.k1, satelliteLaunchControl1.add2.k2, satelliteLaunchControl1.add3.k1, satelliteLaunchControl1.add3.k2, satelliteLaunchControl1.launch_force_x.offset, satelliteLaunchControl1.launch_force_x.startTime, satelliteLaunchControl1.launch_force_x1.offset, satelliteLaunchControl1.launch_force_x1.startTime, satelliteLaunchControl1.launch_force_y.offset, satelliteLaunchControl1.launch_force_y.startTime, satelliteLaunchControl1.launch_force_y1.offset, satelliteLaunchControl1.launch_force_y1.startTime, satelliteLaunchControl1.launch_force_z.offset, satelliteLaunchControl1.launch_force_z.startTime, satelliteLaunchControl1.launch_force_z1.offset, satelliteLaunchControl1.launch_force_z1.startTime, target_force.k, target_height.k}, instantiated = true);
  FMUFunctions_.wsmSetBoolean(fmu_, {317}, {launchSwitch.startValue}, instantiated = true);
  FMUFunctions_.ensureInitializationMode(fmu_);
  FMUFunctions_.wsmSetReal(fmu_, {218, 219, 220, 221, 222, 223, 224}, {direction[1], direction[2], direction[3], height, position[1], position[2], position[3]});
  out_c_real_ := FMUFunctions_.wsmGetReal(fmu_, {60, 61, 62});
initial equation
  startTime_ = time;
  currentCommunicationPoint_ = time;
algorithm
  when FMUFunctions_.initialized(fmu_) then
    p_real_ := FMUFunctions_.wsmGetReal(fmu_, {239, 241, 242, 247, 249, 254, 257, 258, 275, 277}, afterInitialization = true);
    p_real1_ := FMUFunctions_.wsmGetReal(fmu_, {279, 280, 285, 287, 292, 295, 296, 313, 324, 327}, afterInitialization = true);
    p_real2_ := FMUFunctions_.wsmGetReal(fmu_, {330, 333, 336, 339}, afterInitialization = true);
    p_bool_ := FMUFunctions_.wsmGetBoolean(fmu_, {226, 227, 228, 233, 234, 235, 255, 256, 266, 267}, afterInitialization = true);
    p_bool1_ := FMUFunctions_.wsmGetBoolean(fmu_, {293, 294, 304, 305}, afterInitialization = true);
    p_enum_ := FMUFunctions_.wsmGetInteger(fmu_, {240, 263, 264, 278, 301, 302}, afterInitialization = true);
  end when;
equation
  {controller.D.T, controller.D.k, controller.D.x_start, controller.addD.k1, controller.addP.k1, controller.gainPID.k, controller.limiter.uMax, controller.limiter.uMin, controller.yMin, forward_controller.D.T} = p_real_;
  {forward_controller.D.k, forward_controller.D.x_start, forward_controller.addD.k1, forward_controller.addP.k1, forward_controller.gainPID.k, forward_controller.limiter.uMax, forward_controller.limiter.uMin, forward_controller.yMin, satelliteLaunchControl1.launch_force_x.height, satelliteLaunchControl1.launch_force_x1.height} = p_real1_;
  {satelliteLaunchControl1.launch_force_y.height, satelliteLaunchControl1.launch_force_y1.height, satelliteLaunchControl1.launch_force_z.height, satelliteLaunchControl1.launch_force_z1.height} = p_real2_;
  {altitudeConverter1.directionConverter1.satelliteNormalize1.abs1.generateEvent, altitudeConverter1.directionConverter1.satelliteNormalize1.abs11.generateEvent, altitudeConverter1.directionConverter1.satelliteNormalize1.abs12.generateEvent, altitudeConverter2.directionConverter1.satelliteNormalize1.abs1.generateEvent, altitudeConverter2.directionConverter1.satelliteNormalize1.abs11.generateEvent, altitudeConverter2.directionConverter1.satelliteNormalize1.abs12.generateEvent, controller.limiter.limitsAtInit, controller.limiter.strict, controller.limitsAtInit, controller.strict} = p_bool_;
  {forward_controller.limiter.limitsAtInit, forward_controller.limiter.strict, forward_controller.limitsAtInit, forward_controller.strict} = p_bool1_;
  controller.D.initType = Modelica_Blocks_Types_Init_fromInteger_(p_enum_[1]);
  controller.controllerType = Modelica_Blocks_Types_SimpleController_fromInteger_(p_enum_[2]);
  controller.initType = Modelica_Blocks_Types_InitPID_fromInteger_(p_enum_[3]);
  forward_controller.D.initType = Modelica_Blocks_Types_Init_fromInteger_(p_enum_[4]);
  forward_controller.controllerType = Modelica_Blocks_Types_SimpleController_fromInteger_(p_enum_[5]);
  forward_controller.initType = Modelica_Blocks_Types_InitPID_fromInteger_(p_enum_[6]);
equation
  assert(Modelica.Utilities.Strings.isEqual(ModelicaServices.target, "i686-pc-win32"), "Imported FMU does not support the current platform (as determined by ModelicaServices.target=" + ModelicaServices.target + "), supported platforms: i686-pc-win32");
  {y[1], y[2], y[3]} = out_c_real_;
  connect(stepTrigger_, stepTriggerInternal_.u) annotation(Line(visible = true, origin = {-20, 90}, points = {{0, 10}, {-0, -0}, {13, 0}}, color = {190, 52, 178}));
  connect(sampleTrigger_.y, stepTriggerInternal_.u) annotation(Line(visible = true, origin = {-20, 90}, points = {{-19, 0}, {13, 0}}, color = {190, 52, 178}));
algorithm
  when stepTriggerInternal_.u then
    terminateSimulation_ := FMUFunctions_.wsmDoStep(fmu_, currentCommunicationPoint_, time - currentCommunicationPoint_, false);
    currentCommunicationPoint_ := time;
    out_c_real_ := FMUFunctions_.wsmGetReal(fmu_, {60, 61, 62});
    if terminateSimulation_ then
      terminate("Successful termination of simulation triggered by imported FMU: " + getInstanceName());
    end if;
  end when;
  stepDone_ := true;
algorithm
  assert(stepDone_, "Internal error:  Wrong ordering of algorithm sections.", AssertionLevel.warning);
  when stepTriggerInternal_.u then
    FMUFunctions_.wsmSetReal(fmu_, {218, 219, 220, 221, 222, 223, 224}, {direction[1], direction[2], direction[3], height, position[1], position[2], position[3]});
  end when;
protected
  record fmu_record_1_
    Boolean generateEvent "Choose whether events shall be generated";
  end fmu_record_1_;

  record fmu_record_0_
    parameter Real k = 0.0 "Constant output value";
  end fmu_record_0_;

  record fmu_record_7_
    parameter Real k(unit = "1") = 1.0 "Gain value multiplied with input signal";
  end fmu_record_7_;

  record fmu_record_9_
    discrete Real k(unit = "1") "Gain value multiplied with input signal";
  end fmu_record_9_;

  record fmu_record_4_
    fmu_record_3_ satelliteNormalize1;
  end fmu_record_4_;

  record fmu_record_5_
    fmu_record_0_ const;
    fmu_record_4_ directionConverter1;
  end fmu_record_5_;

  record fmu_record_13_
    parameter Real k1 = 1.0 "Gain of upper input";
    parameter Real k2 = 1.0 "Gain of lower input";
  end fmu_record_13_;

  record fmu_record_8_
    discrete Real k1 "Gain of upper input";
    parameter Real k2 = -1.0 "Gain of lower input";
  end fmu_record_8_;

  record fmu_record_12_
    parameter Boolean startValue = false "Output before startTime";
    parameter Real startTime(unit = "s") = 1700.0 "Time instant of step start";
  end fmu_record_12_;

  record fmu_record_14_
    discrete Real height "Height of step";
    parameter Real offset = -2319344.3389588799 "Offset of output signal y";
    parameter Real startTime(unit = "s") = 500.0 "Output y = offset for time < startTime";
  end fmu_record_14_;

  record fmu_record_2_
    parameter Real k1 = 1.0 "Gain of upper input";
    parameter Real k2 = 1.0 "Gain of middle input";
    parameter Real k3 = 1.0 "Gain of lower input";
  end fmu_record_2_;

  record fmu_record_3_
    fmu_record_1_ abs1;
    fmu_record_1_ abs11;
    fmu_record_1_ abs12;
    fmu_record_2_ add31;
  end fmu_record_3_;

  record fmu_record_10_
    Boolean limitsAtInit "Has no longer an effect and is only kept for backwards compatibility (the implementation uses now the homotopy operator)";
    Boolean strict "= true, if strict limits with noEvent(..)";
    discrete Real uMax "Upper limits of input signals";
    discrete Real uMin "Lower limits of input signals";
  end fmu_record_10_;

  record fmu_record_6_
    Types_.Modelica_Blocks_Types_Init initType "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)";
    discrete Real T(unit = "s", final min = 2.2250738585072014e-308) "Time constants (T>0 required; T=0 is ideal derivative block)";
    discrete Real k(unit = "1") "Gains";
    discrete Real x_start "Initial or guess value of state";
    parameter Real y_start = 0.0 "Initial value of output (= state)";
  end fmu_record_6_;

  record fmu_record_15_
    fmu_record_13_ add1;
    fmu_record_13_ add2;
    fmu_record_13_ add3;
    fmu_record_14_ launch_force_x;
    fmu_record_14_ launch_force_x1;
    fmu_record_14_ launch_force_y;
    fmu_record_14_ launch_force_y1;
    fmu_record_14_ launch_force_z;
    fmu_record_14_ launch_force_z1;
  end fmu_record_15_;

  record fmu_record_11_
    fmu_record_0_ Izero;
    fmu_record_10_ limiter;
    fmu_record_2_ addPID;
    fmu_record_6_ D;
    fmu_record_7_ P;
    fmu_record_8_ addD;
    fmu_record_8_ addP;
    fmu_record_9_ gainPID;
    Boolean limitsAtInit "= false, if limits are ignored during initialization";
    Boolean strict "= true, if strict limits with noEvent(..)";
    Types_.Modelica_Blocks_Types_InitPID initType "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)";
    Types_.Modelica_Blocks_Types_SimpleController controllerType "Type of controller";
    parameter Real Nd(final min = 1.110223024625157e-14) = 10.0 "The higher Nd, the more ideal the derivative block";
    parameter Real Ni(final min = 1.110223024625157e-14) = 0.90000000000000002 "Ni*Ti is time constant of anti-windup compensation";
    parameter Real Td(unit = "s", final min = 0.0) = 2000.0 "Time constant of Derivative block";
    parameter Real Ti(unit = "s", final min = 2.2250738585072014e-308) = 0.5 "Time constant of Integrator block";
    parameter Real k(unit = "1", final min = 0.0) = 10.0 "Gain of controller";
    parameter Real wd(final min = 0.0) = 0.0 "Set-point weight for Derivative block (0..1)";
    parameter Real wp(final min = 0.0) = 1.0 "Set-point weight for Proportional block (0..1)";
    parameter Real xd_start = 0.0 "Initial or guess value for state of derivative block";
    parameter Real xi_start = 0.0 "Initial or guess value value for integrator output (= integrator state)";
    parameter Real yMax = 1000000.0 "Upper limit of output";
    discrete Real yMin "Lower limit of output";
    parameter Real y_start = 0.0 "Initial value of output";
  end fmu_record_11_;

  package FMUFunctions_
    class FMUModel
      extends ExternalObject;

      function constructor
        input String fmuDirectory;
        input String instanceName;
        input Integer logLevel;
        input Boolean debugLogging;
        input String[:] categories;
        input Boolean complicanceChecking;
        input Boolean visible;
        input Real tolerance;
        output FMUModel fmu;

        external "WSM" fmu = wsm_fmi2cs_constructor(fmuDirectory, instanceName, logLevel, debugLogging, size(categories, 1), categories, complicanceChecking, visible, tolerance);
        annotation(__Wolfram(FMI(version = "2.0", kind = "CS", runtimeVersion = "2")), Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
      end constructor;

      function destructor
        input FMUModel fmu;

        external "WSM" wsm_fmi2cs_destructor(fmu);
        annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
      end destructor;
    end FMUModel;

    /* FMI 2.0 CS wrappers */

    function wsmDoStep
      input FMUModel fmu;
      input Real currentCommunicationPoint;
      input Real communicationStepSize;
      input Boolean noSetFMUStatePriorToCurrentPoint;
      output Boolean terminateSimulation_;

      external "WSM" wsm_fmi2cs_DoStep(fmu, currentCommunicationPoint, communicationStepSize, noSetFMUStatePriorToCurrentPoint, terminateSimulation_);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end wsmDoStep;

    /* FMI 2.0 Common wrappers */

    function wsmSetReal "Enhanced wrapper for fmi2SetReal"
      input FMUModel fmu;
      input Integer[:] valueReferences;
      input Real[:] values;
      input Boolean instantiated = false "If true, ignore call if the FMU has left the instantiated state";
      input Boolean initialized = false "If true, ignore call if the FMU has left initialized state";

      external "WSM" wsm_fmi2cs_SetReal(fmu, valueReferences, size(valueReferences, 1), values, instantiated, initialized);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end wsmSetReal;

    function wsmSetInteger "Enhanced wrapper for fmi2SetInteger"
      input FMUModel fmu;
      input Integer[:] valueReferences;
      input Integer[:] values;
      input Boolean instantiated = false "If true, ignore call if the FMU has left the instantiated state";
      input Boolean initialized = false "If true, ignore call if the FMU has left initialized state";

      external "WSM" wsm_fmi2cs_SetInteger(fmu, valueReferences, size(valueReferences, 1), values, instantiated, initialized);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end wsmSetInteger;

    function wsmSetBoolean "Enhanced wrapper for fmi2SetBoolean"
      input FMUModel fmu;
      input Integer[:] valueReferences;
      input Boolean[:] values;
      input Boolean instantiated = false "If true, ignore call if the FMU has left the instantiated state";
      input Boolean initialized = false "If true, ignore call if the FMU has left initialized state";

      external "WSM" wsm_fmi2cs_SetBoolean(fmu, valueReferences, size(valueReferences, 1), values, instantiated, initialized);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end wsmSetBoolean;

    function wsmSetString "Enhanced wrapper for fmi2SetString"
      input FMUModel fmu;
      input Integer[:] valueReferences;
      input String[:] values;
      input Boolean instantiated = false "If true, ignore call if the FMU has left the instantiated state";
      input Boolean initialized = false "If true, ignore call if the FMU has left initialized state";

      external "WSM" wsm_fmi2cs_SetString(fmu, valueReferences, size(valueReferences, 1), values, instantiated, initialized);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end wsmSetString;

    function wsmGetReal "Wrapper for fmi2GetReal"
      input FMUModel fmu;
      input Integer[:] valueReferences;
      input Boolean afterInitialization = false "If true, only allow call if FMU has left initialization state";
      output Real[size(valueReferences, 1)] values;

      external "WSM" wsm_fmi2cs_GetReal(fmu, valueReferences, size(valueReferences, 1), values, afterInitialization);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end wsmGetReal;

    function wsmGetInteger "Wrapper for fmi2GetInteger"
      input FMUModel fmu;
      input Integer[:] valueReferences;
      input Boolean afterInitialization = false "If true, only allow call if FMU has left initialization state";
      output Integer[size(valueReferences, 1)] values;

      external "WSM" wsm_fmi2cs_GetInteger(fmu, valueReferences, size(valueReferences, 1), values, afterInitialization);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end wsmGetInteger;

    function wsmGetBoolean "Wrapper for fmi2GetBoolean"
      input FMUModel fmu;
      input Integer[:] valueReferences;
      input Boolean afterInitialization = false "If true, only allow call if FMU has left initialization state";
      output Boolean[size(valueReferences, 1)] values;

      external "WSM" wsm_fmi2cs_GetBoolean(fmu, valueReferences, size(valueReferences, 1), values, afterInitialization);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end wsmGetBoolean;

    function wsmGetString "Wrapper for fmi2GetString"
      input FMUModel fmu;
      input Integer[:] valueReferences;
      input Boolean afterInitialization = false "If true, only allow call if FMU has left initialization state";
      output String[size(valueReferences, 1)] values;

      external "WSM" wsm_fmi2cs_GetString(fmu, valueReferences, size(valueReferences, 1), values, afterInitialization);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end wsmGetString;

    /* Functions specific to the Wolfram System Modeler FMI import */

    function ensureInitializationMode
      input FMUModel fmu;

      external "WSM" wsm_fmi2cs_ensureInitializationMode(fmu);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end ensureInitializationMode;

    function initialized
      input FMUModel fmu;
      output Boolean initialized;

      external "WSM" wsm_fmi2cs_initialized(fmu, initialized);
      annotation(Library = {"fmi2_import_runtime", "fmilib"}, Include = "#include \"fmi2cs_ext_obj.h\"");
    end initialized;
  end FMUFunctions_;

  /* Converters from enumeration types to Integer */
  /* Converters from Integer to enumeration types */

  function Modelica_Blocks_Types_Init_fromInteger_
    input Integer i;
    input Boolean inputIsUndefined = false;
    output Types_.Modelica_Blocks_Types_Init e;
  algorithm
    if inputIsUndefined then
      e := Types_.Modelica_Blocks_Types_Init.NoInit;
      return;
    end if;
    assert(1 <= i and i <= 4, "Integer to be converted to enumeration type is out of range.");
    e := if i == 1 then Types_.Modelica_Blocks_Types_Init.NoInit elseif i == 2 then Types_.Modelica_Blocks_Types_Init.SteadyState
     elseif i == 3 then Types_.Modelica_Blocks_Types_Init.InitialState else Types_.Modelica_Blocks_Types_Init.InitialOutput;
  end Modelica_Blocks_Types_Init_fromInteger_;

  function Modelica_Blocks_Types_InitPID_fromInteger_
    input Integer i;
    input Boolean inputIsUndefined = false;
    output Types_.Modelica_Blocks_Types_InitPID e;
  algorithm
    if inputIsUndefined then
      e := Types_.Modelica_Blocks_Types_InitPID.NoInit;
      return;
    end if;
    assert(1 <= i and i <= 5, "Integer to be converted to enumeration type is out of range.");
    e := if i == 1 then Types_.Modelica_Blocks_Types_InitPID.NoInit elseif i == 2 then Types_.Modelica_Blocks_Types_InitPID.SteadyState
     elseif i == 3 then Types_.Modelica_Blocks_Types_InitPID.InitialState
     elseif i == 4 then Types_.Modelica_Blocks_Types_InitPID.InitialOutput else Types_.Modelica_Blocks_Types_InitPID.DoNotUse_InitialIntegratorState;
  end Modelica_Blocks_Types_InitPID_fromInteger_;

  function Modelica_Blocks_Types_SimpleController_fromInteger_
    input Integer i;
    input Boolean inputIsUndefined = false;
    output Types_.Modelica_Blocks_Types_SimpleController e;
  algorithm
    if inputIsUndefined then
      e := Types_.Modelica_Blocks_Types_SimpleController.P;
      return;
    end if;
    assert(1 <= i and i <= 4, "Integer to be converted to enumeration type is out of range.");
    e := if i == 1 then Types_.Modelica_Blocks_Types_SimpleController.P elseif i == 2 then Types_.Modelica_Blocks_Types_SimpleController.PI
     elseif i == 3 then Types_.Modelica_Blocks_Types_SimpleController.PD else Types_.Modelica_Blocks_Types_SimpleController.PID;
  end Modelica_Blocks_Types_SimpleController_fromInteger_;
  annotation(Protection(access=Access.packageDuplicate), uses(Modelica(version = "4.0.0"), ModelicaServices(version = "4.0.0")), preferredView = "icon", Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Text(visible = true, origin = {0, -260}, extent = {{-150, 110}, {150, 150}}, textString = "%name"), Bitmap(visible = true, fileName = "Resources/model.png", imageSource = "", extent = {{-100, -100}, {100, 100}}), Polygon(visible = true, origin = {50, 145.6}, lineColor = {64, 64, 64}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid, points = {{-40, -13.6}, {-40, -13.6}, {-40, -1.6}, {-40, 14.4}, {-24, 14.4}, {24, 14.4}, {40, 14.4}, {40, -1.6}, {40, -13.6}, {40, -13.6}}, smooth = Smooth.Bezier), Polygon(visible = true, origin = {50, 104}, lineColor = {64, 64, 64}, fillColor = {217, 217, 217}, fillPattern = FillPattern.Solid, points = {{-40, 28}, {-40, 28}, {-40, -8}, {-40, -24}, {-24, -24}, {24, -24}, {40, -24}, {40, -8}, {40, 28}, {40, 28}}, smooth = Smooth.Bezier), Text(visible = true, origin = {50, 146}, textColor = {255, 255, 255}, extent = {{-40, -14}, {40, 14}}, textString = "FMU"), Text(visible = true, origin = {50, 106}, textColor = {102, 102, 102}, extent = {{-40, -14}, {40, 14}}, textString = "CS")}), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Text(visible = true, origin = {10, 70}, extent = {{-110, 0}, {110, 15}}, textString = "Imported FMU for co-simulation", fontSize = 36, horizontalAlignment = TextAlignment.Left), Text(visible = true, origin = {-90, 40}, extent = {{0, 0}, {190, 7.5}}, textString = "FMI version: 2.0", horizontalAlignment = TextAlignment.Left), Text(visible = true, origin = {-90, 0}, extent = {{0, 0}, {190, 7.5}}, textString = "FMU model name: EducationExamples.AerospaceEngineering.Satellite.Components.ControlLogic", horizontalAlignment = TextAlignment.Left), Text(visible = true, origin = {-90, -20}, extent = {{0, 0}, {190, 7.5}}, textString = "Description: Satellite controller", horizontalAlignment = TextAlignment.Left), Text(visible = true, origin = {-90, -40}, extent = {{0, 0}, {190, 7.5}}, textString = "Copyright: ", horizontalAlignment = TextAlignment.Left), Text(visible = true, origin = {-90, -60}, extent = {{0, 0}, {190, 7.5}}, textString = "Generation tool: Wolfram SystemModeler 12.0.0.10", horizontalAlignment = TextAlignment.Left), Text(visible = true, origin = {-90, -80}, extent = {{0, 0}, {190, 7.5}}, textString = "Targets: i686-pc-win32", horizontalAlignment = TextAlignment.Left)}), Documentation(info = "Imported FMU does not contain any documentation."), __Wolfram(ImportedFMICS(filename = "C:/Users/ankitn/Desktop/FMU cosimulation import/SatelliteControlLogic.fmu", options(structuredNames = true, allowQuoting = true, includeDerivatives = false, includeLocal = false, includeMangled = true, manglingMode = "warning", useNominals = true, protectedSubPackages = true, complianceChecking = true, wrapperTerminals = false, filterInputs = false, filterOutputs = false, delayBlockType = "automatic", wrapperFilterT = 0.0001))));
end ControlLogic_fmu;
