package SatelliteControllerDeploy
  package Components "Package containing components used by the example"
    extends Modelica.Icons.Package;

    model Earth "Models the mass of the earth"
      Modelica.Mechanics.Rotational.Sources.ConstantSpeed rotationSpeed(w_fixed = 2 * Modelica.Constants.pi / 86400) annotation(Placement(visible = true, transformation(origin = {-30.0, -40.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -720)));
      Modelica.Mechanics.MultiBody.Joints.Revolute earthRotation(useAxisFlange = true) annotation(Placement(visible = true, transformation(origin = {0.0, 0.0}, extent = {{10.0, -10.0}, {-10.0, 10.0}}, rotation = -180)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = {1, 1, 1}) annotation(Placement(visible = true, transformation(origin = {-50.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -360)));
      Modelica.Mechanics.MultiBody.Parts.Body earth(m = 5.9736 * 10 ^ 24, r_0.start = {1, 1, 1}, r_CM = {0, 0, 0}) annotation(Placement(visible = true, transformation(origin = {50.0, -30.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_earth annotation(Placement(visible = true, transformation(origin = {90.0, -0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0), iconTransformation(origin = {100.0, 0.0}, extent = {{-16.0, -16.0}, {16.0, 16.0}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_world annotation(Placement(visible = true, transformation(origin = {-90.0, 0.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0), iconTransformation(origin = {-97.8899, 0.0}, extent = {{-16.0, -16.0}, {16.0, 16.0}}, rotation = 0)));
    equation
      connect(rotationSpeed.flange, earthRotation.axis) annotation(Line(visible = true, origin = {-6.667, -30}, points = {{-13.333, -10}, {6.667, -10}, {6.667, 20}}, color = {64, 64, 64}));
      connect(earthRotation.frame_b, earth.frame_a) annotation(Line(visible = true, origin = {36.667, -6.667}, points = {{-26.667, 6.667}, {13.333, 6.667}, {13.333, -13.333}}, color = {95, 95, 95}));
      connect(frame_earth, earth.frame_a) annotation(Line(visible = true, origin = {63.333, -6.667}, points = {{26.667, 6.667}, {-13.333, 6.667}, {-13.333, -13.333}}, color = {95, 95, 95}));
      connect(fixedTranslation1.frame_b, earthRotation.frame_a) annotation(Line(visible = true, origin = {-25, 0}, points = {{-15, 0}, {15, 0}}, color = {95, 95, 95}));
      connect(frame_world, fixedTranslation1.frame_a) annotation(Line(visible = true, origin = {-75, 0}, points = {{-15, 0}, {15, 0}}, color = {95, 95, 95}));
      annotation(Documentation(info = "<html>
<h3>Earth</h3>
    <h4>Description</h4>
    <p>
This component models the Earth, rotating around a fixed frame.
</p><p>
View the <a href=\"modelica://SatelliteControllerDeploy.Components.Earth#diagram\">model diagram</a> for this model.
</p></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {0, 0, 128}, fillColor = {255, 255, 255}, lineThickness = 2, extent = {{-100, -100}, {100, 100}}), Text(visible = true, origin = {0.735, -0.42}, extent = {{-54.282, -33.598}, {54.282, 33.598}}, textString = "[Earth]", fontName = "Arial"), Bitmap(visible = true, fileName = "", imageSource = "iVBORw0KGgoAAAANSUhEUgAAAWgAAAFoCAIAAAD1h/aCAAAAIHRFWHRTb2Z0d2FyZQBXb2xm





cmFtIE1hdGhlbWF0aWNhIDguMEgHxzcAAAAhdEVYdENyZWF0aW9uIFRpbWUAMjAxMToxMTox





NCAxMjo1MjoxMUfR8EIAACAASURBVHic7L13uCVXdeC71tqp0gk3306SuqVWaiUQEhISJkgg





hIi2CAM4B57H7+PNc8AJxvbY4MeM/TzPnvecsfFnGIwJJhqDRRASAgQCCQWE1K3Ot28+94RK





O70/Tvft27e7BRdaEsy3f5++o111qu6pU2fXr9dee+8q9N5DIBAIbAR6qg8gEAj88BHEEQgE





NkwQRyAQ2DBBHIFAYMMEcQQCgQ0TxBEIBDZMEEcgENgwQRyBQGDDBHEEAoENE8QRCAQ2TBBH





IBDYMEEcgUBgwwRxBAKBDRPEEQgENkwQRyAQ2DBBHIFAYMMEcQQCgQ0TxBEIBDZMEEcgENgw





QRyBQGDDBHEEAoENE8QRCAQ2TBBHIBDYMEEcgUBgwwRxBAKBDRPEEQgENkwQRyAQ2DBBHIFA





YMMEcQQCgQ0TxBEIBDZMEEcgENgwQRyBQGDDBHEEAoENE8QRCAQ2TBBHIBDYMEEcgUBgwwRx





BAKBDRPEEQgENkwQRyAQ2DBBHIFAYMMEcQQCgQ0TxBEIBDZMEEcgENgwQRyBQGDDBHEEAoEN





E8QRCAQ2TBBHIBDYMEEcgUBgwwRxBAKBDRPEEQgENkwQRyAQ2DBBHIFAYMMEcQQCgQ0TxBEI





BDZMEEcgENgwQRyBQGDDBHEEAoENE8QRCAQ2TBBHIBDYMEEcgUBgwwRxBAKBDRPEEQgENkwQ





RyAQ2DBBHIFAYMMEcQQCgQ0TxBEIBDZMEEcgENgwQRyBQGDDBHEEAoENE8QRCAQ2TBBHIBDY





MEEcgUBgw/Cn+gACP1g867qxe+/eHI+zLdOcu/7YqOLM94voiw/q3/3psf/74c3usUemt+Bs





hxcP5RGxV9xQ//0Hv/5UH3XgyQa990/1MQSeJLZcJDv7LkRIX/2axl0P7mlG2f7drtcBBHGq





zXHtwkVPxxtvePHNz7v+b//yD9/6h3//vnf9t353/gsP7p2bMTc/a/Lw7Ozn/p2/7JZ0sVMo





6f/nB7/05HyjwFNFEMf/UiTxxQCIOLzm8eh/CED8Na8970//5G+FEHgM7z0RIdiHH3zgDT//





899+wP37p9763nf/z396395+mYOPH+eDENEjv+vud7/tbb86NXHOC65K5uaOfOATD7/hDZe9





5+N7bvvw7NQ0XPCMvF0qA3Zpqecxue32B5+ckxB4Egji+F+BJL4YAAARj8oCVpXx8lvP/Ys/





+3vvfRRF3ntE5JwPtx1y8l+bmV9aXFn572/75S/ee9/sgYbOE2TmhC3WVBqHHsA4A7vObyZZ





esPzL3nNrdc/89lvv/7qHTsu7N57MLvvtgOsctVY7Ivetc+aj23MhP/QRx9+Ak9H4IkniOOH





mCS+aFUQJyqD/vKdP3f3N3rv/IuPgikB0Bjzm3/wxjf/7z8jhAAAIjqlMh6fdNPVYgCg2J++





+/dfef3zz9pxtekjEQKArlHbpbn5h7/whX++4QU3VoPDH/vE/v/81nccWe7c/pk/+8LH//ix





2f3ve5cqjP+xH6s/9tnkyst7qWQf/PA3zvAZCTxZBHH88JHEFwHAScpAgONBB+LRNQ4pVvx1





P/fc3/qVX54aH9daSynPyGF8+s4vffvhe/7+nZ969P4jlz1tx3v+6c/K5W/+zh/90y3XbP35





N/3jpbt2/ftdf3XW+C0f/dhbxqauuPoZP+qtu/KK+CUvmB7YiSMHvoyeza8saZ3OLsx98a7D





Z+SQAk8aQRw/NCTxhcdjiqOCgFVlDE1xzBdr2yAICCiTn3nTi9/x278u2Zk8pOff8NK7vzJH





QD/5C0/fsbN9xTnn/+X73jkRbTP1kdHNW9/7/iNRZPY9thCxbJD3JiYnfDU/21MSnHPumqfv





2nn27LZx960jas/cXu56n//8/jN5cIEnkjCO44eAJL4wiS8CIAAEJEQ6FmUQIiESEUNkwzIi





4VoIifH/+se/+o7ferOp6rf80Tue+fzrdx/Y94FP/1u/NN/5s0/PoUOHPnPbRxc7X37165/1





e2/5g5953a8no1dMtp7z7T3Rc5/3U2/5z+/8D6+6dOYxQ47e9l+efuNNVy4tdHtF62d/8npt





3S/+7BUXX3Dw3R8ffOhzAyEPnz05ec/dWzZt2vHCF5530aVbz9R5CzxxhIjjB5c4vmBNM+SE





VgkirIk7jnejEAOdm50XDPbsbSECEQfwAAjgPvDJ//Gy57/pDa9P28QuffH/WQ8W9x/a94IX





vXTHWRdONcX3kPLQWltnLjz/pj/9s9956Uuff/DgwaIooiidnZ09Z9vO63/kNf/HL7/hrb/1





P5xzSExJXlX1G3/x1Y/tOxLJaH65s3zgS4wn9357ZaShbr2Z5YNqdCz7f/62eP0rBsCbd37j





yP3f2H2mz2jgjBHE8QPHMV/AMV8Mr+k1LZE17ZSjBgEANLuelm+epM/emft88/p3jy2sLdzw





IzDgrT/5kz+fmjirqVADfOxTn7n0ol3nbZv6jgf5W7/5//3Gb//0udtvAg9IibMdpZI//+vf





fetv/dn+fQcZJ8bBmaMfNKxjqzXNH1tmXhj0gAY9GKqYl85FAlC7AfDudZebvYeKxlT1lTsW





v78zGjjzBHH8YHGqRMbq9X88lzHcYHW9ovj6qw/ddPNPfe1rH/nmsv/2lwgAwbOTZXFyYcd5





m849d/Tfb78byvbNt0y/5CUvnu3ve+2t/2kkTU5ux/7ze//tVa+9CQCMMffd9+gtt/w8eAYe





PHjvPRH3Dp2vV//4WlmsLZz8un6R0Hq962nzscPalCWZe+5YOgPnN3CGCOL4QWFtw+S4HdYm





Qdf0ua62TYaFcy9YvOfuPW947Yt1feiRuXzv/WcRK2D9xusLj2MTYviSl7zo9s997nU//rKf





+smXbt265Yt33f+0Ky6JYkCA279wl5TRy176S5wpRBjWoJMdscqatwDg1OLw3p/SI0B+y7mL





MRatWG7dMvkP7737+zzPgTNCEMdTzwm5jPV2WB9cwInWQCSgSLuZyy+Ntu3c+rmPGWMHx7wA





q9uvNcXpPQJrh5kffZdiW68cPnL7/v0zVz/9J2SMwyt/uIFzDsA7t1qFvPenEAccfxtWxbGq





CziNO9a+jk6XV16F+w4c2joxcWh+6Su3z36vJztwZgjieIpZ2zY5PlR87YCuE3IccMwFdEwH





OCzDiaZY+4oEhNx7D+BPNsjjxB3ryutYFyA451bXf8dKdbIpTm+QYzLy/jk3dWZmOlEsFjr9





sbHWHf924Duc3MATRhDHU8aaQOOEJsl3zGgc72g9Pmx8rQhg1UHH3sbVHOvjv8I6U5x+AU6h





DO/90bL3brVard1tuAsiPo441hZOikr8s67tFaY30mr3+v3lqnzJzT/+u7/+376Xsx/4/gji





eAqI4wtWgwiANdY4TUZjNehYVcZwy2P6gHWBxgnWOL0pTlp5/GNWD3WdLU7Oeq5btNYOX4cG





GW5PROt2OaGHxZ+ki1NLxCOitfYlt+QLi8Vi1QXtmYcjRs8/sD3Pw+j1J5UgjiebddY4MV44





fUYDEWHNyC5QgGatGmCNS+Aka5xCE0ejkO/dHY+zBgCUUnmeO+eMOT7M7GR3nMIZJy2euKWz





Xr7ieQtzPbdsO0VeeA+XXPfKj/71A73+VzfyOwS+L4I4njzi6Py1CQtYlcVpMhqr649f+YhE





YI3nXBwNPU4fYpy8eLrC6qL3lnmKNpu7P/eJT3/xc7/xK79br4zAGjWc0hFrVzLGVl8BwBhj





jLHWnsYC6wVxupXr3nLavepWnw/04aV57epuWX7s/Q/uuuAV3vv+4Gtn9lcLnJIgjieDONoJ





Jw7lOi6Lx+1GGcqCER8G6mtccOK48tPL4uTg4pQFBAQFD977ob95/9+osv7xN/zH0sNzX/8z





X3rPOy/Z9WqpqKpKRgLg1MHIsDGyumZ1cagMY8zRC965VQUAwDAY8d771aEg3wUAAOBvuDEb





DPY3Uj8/ny/5HEHF0t1759bgjieHII4nnMexxvEmCQICeo/Ea+/UakaDcW9tzSgdti6I6PiV





DjicHf8dxfEdlIGIiMIyYGDJ57TSpJH//tc/9cY3/sWf/79vf+41u37pl/7rnXd8bXWvVY4t





HvtixxaHE+2HrMrCeW+P6WNdDAEAWmt/PAO6ziCrtjlxLfqrn9nPMqu4LXN7YGXFOi+1e+C+





Hd770Gx5ognieAKJo/OOeeHkLtWjBQ80fbY9tI8/+7mDfQtu5sEGAjJMgLlnXz9P1j1wWC4f





mDrvioO7790Exy97JEJEWnUHfHfieJxFIjLGytT/4wf+4XUv+RnOea1LQCnYemXAqeMORIR1





Ww5rl3PuqD6OFU7XBoETJTLcZbW/5iR5qOc8ez5tekQscr2vt5JymQ+K+cXpucdEcMcTShDH





E0UcnXtsPusxa5xQQACw4J72ohWPzllrTM2Z2PP1s7PRR644e3IuH9z/5SnBIwAcjr9APJ7v





GIYbQ06lgnWCgBP7bk8tEU8qVqC1Hn4anNgLs8rJi977U2455OSwwTl3ujhirTKGBlm3/uTd





rr6mnphwZVEfXupNTDbnZxf6VdVot+/5TNuHZssTRhDHmSeKdqxOe38cazAGHqMLr14xarFY





bKaTBVR1mkWLB5qPPTChRH2yLNZc7pFntSDunRDKe+ePCQKJAXo1vrlYnEmJWecNYxyBfce4





4+S34NSRBSDS6vhR59xqRuOUrI8SjjZc1krkhIW1uwzLdV0fW/TDwWDr9AFGv/QVKyu5Kn3H





O3VgdjEmGm207/x8y4dmyxNDEMcZZo01js9SW5UFrqY/ERCI2ku6O5KNF9jZdMnzZi3k9942





fYIscLWPFQFQikzjvs3nDCoNjJpx3C+WdqTtxUIz73hjzMSSrE76K2S5bTaqfp9tmsqOHOqC





j6vOhK5RRp7wFBI5jTpO6ExBRC74O/7w1yzqN//Kn2hTMkJExnniXAmnma7y+PHCyfpYW4Y1





Eonw6yvlxY+z70tf0l+qynvviXddVvTzQhi7UuaP3HOW8Ty444wTxHEmiaIdAOtksd4axHDH





5XWv69ubu1x5jhYN81Xjwa+0Tg4ujhYIEGBs6/zopCNkt9z0ykcfvK+rl/cdKoXsZUKMNsY3





75g6eHimqWQzbW/eet5X7/laI2sY118pC+HSvJYLM310o8XAGZ1wYsO/vza9us4mq+VjRw7O





AaK7/IpLz7/grDtuv+uNv/SG/XtnP/2p2w8fmh9+/XXi8CcmL06pjLVX/jp9HBuEegKPPXbn





pk1XntY7NVW8vu7ZK3nVZcjAu6rSxscPfXXch2bLGSWI44xxzBoAgLB6k64TxmIQoAdeTV6Y





b92sFSTAPLn49ttA+AatJjLWRhlI6OWWCx7zRBefez6XedGra6Obmc+tZYzPzfayVpT3DIvA





OWqmXJc0NpkVvQq4a6cZY+RLW8e+3Zj41r7azk4tLxsC93hhxqnWHPuWRATeqdotN5vtfrf7





lt950zve9lerJ+FodRr2r27EGv6k1OlqPvXkXfbu/eL09NNP93dM1nneM+03H3KXXZnOHNhb





1G5qdHLP/KEj910cQo8zRRDHmSFS24+HGGvCDTw2xGvzjt7+bysuuXN05Y2zilKJUqj25/4V





2NG5ZyeFG54j4c6nP7JtclutC8nVwcVZJuT4aNMXg9oJoWQnL6x2UcTQEefMWkD0xAAROaeI





y2bWMEzr3I5HDTHS2LtHH3q4TZJ5f7Ij1hjrVPoYflNEJIeejo04OREk4+xw9JdfnSl7Omuc





0g5r9XFyX8zaLffu/SIAXLHrabNzVh/rdnHOIZnWVvaCa+sPfImec3F3fnYZVFsI/dDdeqUz





FdxxRgjiOAOc0hoONEMFAIDYaOudl+b33it8r6XGyqddMzhwQGBpDz86zhgNt8FTtVNGt85k





jRq9NRZIeWtFmolIUCrkoKoGBtNEAhBn5C1EklmyHAkROUMgDh4kCmt8YUZGVd9JuWXL9Bdv





i5xzw0zHd4w4TnoX4ISp/adgWKNGRtvO6Xe96z+//GVvXpe2OGVG41jBOXeCTdZ25a4bgbpW





H5smLtUehxvWFsdGy+VF5pzTZ5fXndfBvhYT6VfvTEba6uCjzodmy/dNEMf3y4nWAEDaeXln





z31bPRQAAF4K5a68brlk5t7PNqzxnCcy67hi0oGjU/WYACBigmzgTeJpkGQmbi5kE6WgZqWL





WDDyDhl3QgJDxQzzbZUObJkefnTMJwfP3YnWxowD85xB6e1IksLA6bEocYyvLEwe2S/wpIGn





6wQxHCRyyqADGYI7hThk5Ory+GQ2InzfP/8BALz8Zb8+HBI2vP7hxCzp2oLzbtjfslYZ68on





jwQZLu7bdxcAjI9fpi33Rv/Ii3rbxtrvem+OgN7WV12/P3NSt/3tH9nWnlpcPtJEoBB6fD8E





cXxfDK1xPLUBePEzOsvd8sijW4YqyTaZyy8rK22Ik6mrRtaQzJeAVeXjNOr3yt2PLUxMjOz5





8nYpa87IowGGl9/4SLE09diDrbonOBcA/qzL9lUlKeEcAQESucU9lxFJIgcMBMaEhMx6z3Ze





+aizzjmIIqm4J8Hz3ArWItXh2Npz7zYgR8hP0yg5BcMEqnDMMOcQZByj1oqpvC5Od1qGlerc





i8962+/8LAC84pVvZcgQ9ckX/OlaJafzxbAwDD3W9uxaqI0Vh/d/8ed+ehfnvux3en4s5fkH





Phbd8tLyX/+FVztytVDqpVFCs/m8+YPfHgeA4I7vmSCO751j1lidbIJXXpVXVt//9RFC9ACb





zvfn7tDdXq/daGvrYsEbSh7pLRPKftmXkbDeIsckiw89TAv7doxtPXL2+TmSr+tC8NFH7svy





TjQ6OTu2bbCwb3vWPkzMe4KFfSMcNjHGEGH7lQ9GUeK89Q6TZKTqDySX2tpKQ6SQyYgx245a





S53+xHhjvC2+/WB79nCChGsGj50WxpgHR8jOelr6jl//teG33jOz8q5//uTvvek1APC6n/gD





5o42XU6uSN5WBVYvefbUT//ir/zDu9/1iY/sWdfKONkdzjnnwVsCrI2x3jvnrTV+nThOjj60





S259+TxzqjLOajcoeorH6PoFb73//QvklffeP12xrw2MJwAwtESmDcEd3ytBHN8jUbRjWBha





wwC/5QY22Wp9cffuPfeNIMhrn7dMEGEC3X7ezpoIrjblcllHBEmURlL0er04TYzTnlnPIqQK





vNr/sGm20j33Z7GMtPbbL3u0cjQ5SXlXEAgZ6WLAFnbvJCKg1o4r7mMIRGhBcK77C6PtTX00





nHPOsLG8UnpHaOOJTRqdkjKfHmn3dfLwN8acc6uhxOn0wRiTTf2zP/nc66+9YfhN//RvPtSf





Ly68aHunWx/Y+8ievQt09KZfR1su6+rS8JJGzoAXf/9Xv/v1+/f98f/1d87iGnEcDRnWJ0Gd





1xqImPPa8X6WmqVZcbIy1rqjjuQv/Gg8c3gfkdq6adugqsv88G1f3mtN89GHFTNkmIE1PcRE





zFq7mrcNKY+NEsTxvRCp7Uf/nQUEgLSlL73Q7jxny965pds/q869/EjSRA/eM0ij1GoviG2a





Hp1dmJ/rlYrhWKs9t7jQajSEEIRY20rgqMryh74u2q2RiW0LjLO9D6i4PQ8AUokkjrUpwDSQ





5Xpl09JcfNbl+wlE0XdxJovcx3GStozVGoFxzHRVN1vcVC1j9ZE9W3Y+46GReMIjJpRUrvXQ





PS0gvTrP5WR9WIfbL9Zv/t9ePTp1ybceuecf3vvx/Y8aSSkya41nFAMYXNPJAmvEYa0dzqmP





IpnnJQB674wFIPaqV1z0wQ/f4yw7OTm6ThyI8Lyb3eKhrNmc+dSnkmuuEJ+/OwdXD+8PdHKb





xVpbAr/u+s4dd2QvuOpwuzEuhC+0acdquTt4z/uqyhsAsBYANQIbHiqQ8rYc1v8Qd2yUII4N





E6ntAEf7QQjp5hcotDWP5b27D0ICEuKJ8WhpsJLXVaOR1pVhQkYyIuacc6Yw1juRKHLAiBiD





ZtrK84HzXsZ+zwPJ0sFzksbhS67vLnU6ZS9lBA7r5cd2eteSApyzIps//0rvtO2ugK1lknrr





kAtIUqUE6dq3mqouUcWes0Rx88DdO885f397zHCM0jRRDXnnJ0eRWUZ8qIt14ohTeOUtjWY2





Ot/LP/wvc4wiBhpIHQ2tYJ0xThjlwTgY7RGPxhSc84ufduHy/MLuR+dfdNOum54/ds837vnn





D5TdlVnGYqRTWGO1TMBeclP107/4R2/48d+Yn81XNTH8/7DnxTptnUHPjTHGsGuv7W4aKX1p





IWqlkT2y3M1LHydcMPjQv7RWunPHZMUAjkYfL39t9sF39xCDOzZMEMfGiNQ5ww5XALCWbn1h





wsjNl8W21tiS7e1dnh3NGp26W2loN1PrKu1AMVk7LUgiQrVYNCdGumUhuU+Tcev7HIYPWwMW





q96ye/SeLWCZlFJkM9mmGd/btnBQjG+q884YERETcmTvWTsdMuh3DLpRjwPBufOOM1XlSWuk





amQkeVSRoUF79sAUeOasuOQZj0aNJOZKm8F9X75QSJSRNUagd0TksGZslGOFyEa3+OdeI6DQ





//JJEpIdG5W2dvYKECaMaXvs7sSrG2zZNjVzeM5py1AYMADw4pvPeca1Nz7y0Fcb8oGluTpK





YuYau66+4oEH7v6Ldz5s63Rd3GGtO57rsJaLenLa7HkkcVA657QFdOggtwacc8bhBeeZgSn3





POrPmu4+68qsqrFXDOJkjLuiMt3Smbp2i4uLrYn2v3202ekswXDyi64El4DeO6hNX7Bk2ILp





9UOD5bsliGMDrLUGAt76smmne62Ud/tufCr53H27kxaTinVWesZCmsScWK8skFy5CFt3tAYr





BRM8ymJdu0HRnZqcNMag9x4orwaRai4ciuZ2TwhVoU89GGLooUBIOUNAM73riO20Z2eazbHu





9DkdwZKi0HkvjVJNIK2m5piRnAPYVkNpYxf2nteYODxYONd7yQi3bi+Kupw/NBaLxPr6WS/s





mjL7+peZatbXX9e6/d8k45qInHW3vrrJktEPvmeOeMVQrREHSJKl9k+7Mrr/m4X3nnGw4Mmx





Z//Is+74wl2AZHQ5MZWcvVW86MbzlheX636/318cGR1rtxqPHXhkcnRqYuv5992/e/t5537j





nrs+8vHqWP/IqROfQxoZXHWN/sRHHBCkqpRpeWSGV1pJvnzTC/smF6XIH/omv2hTPXC20sCF





bbZUXdXWM83M8vyyJSKpYlZ+4D0KVpMvBM565zWAAH90eHtIdnyXBHFsgGFqwxgruPixW9Ja





613bz987MzfVVl/41rdKR81G2s9XKm3HGtmRvOs1CsWTiJaWcHzCkyUVSWehX9eMy1Zb1LlB





L4QkC86C5YimkA9/bUSwaNh68J47dKNbVnpzo5c975G8iqhKZx44F8hkU/vbk9J7D1RWhWo1





OYJFBrbahsCmNh/ae+/lOy47cmTPhNXp9ivu5/UooX3s0U2CsiuuXZjRZnZ35Hubxye7K7Mx





k8NnLDDGkDGc3CzmZmrOxJpnLyAAOAuvurXaO4PfuGsMRZFmqswr55EYWu+ThDdaxfOuS3S3





iyCIYLw9wdD0+3lZQyRVOjHWXenOzs04IiLzoY9E3pfHww3PvOt5F3nvjKmPN14cr6x91nUr





286mvY/si7Op+dklz7S1lDAbc7nSH/QK0FBkXOpaW28QMU3TypcLncXxsbGi1tqY7lL+rW8m





Mwf9aooUjiZKI2uL1cXgju+GII7vlkidgyA9aq3tzp31hVuj2suskQ605U4c7B3eOrnttk/v





e+7zWwdmZ3mUNaJWmdcD3RMZWutbWQzOWXBOO0csy9KFpfksySSX4B0wVxuvSDg/iJPkwTu2





e6glj3Y++4iI+o985tKzr9mtIvLg69wWK6Pd2Smw8dmXP2J90jnc1rmYPm8hSmpnfdmb2nrO





wNURlyAUc7X04BkzyKghVD7I0kbfqvYjB9m5OzAqkiiSVcGFB2Kb9z52GFwTEcamKzRuZaV5





vJfFM6PrN7w+I21qY7700PKhg03tM+mw1ZJnbe1ffmmjm7Pl5cVmWo1EsbPswvMvWF5c6NY2





z/OJ8YmP33HnFeds40wNqkKXnS8fjp57Ab3n/Q5Z6WzkrGmOzm3ZzB95TLeacHCfsLXUNSJp





a633eN1zZlXEu72aAeauSokBWdCm6tqVcpAX9aAccCWtd9w5z2xCsRVo0dS6TKKkMLUt/L9+





IlGAzrlutwtrptJMbuaLs15rDcEd3x1BHN8VSm694bodDzyw+5k/Mvnhj/VvuMadvWXq019Z





OHJIGlcytEnLFUXktbn2OV5XODC90pii5wDsSKNZqKqdpZHH0joU3juXJPHCQnd6elQqQGuj





OOr1ernhY2PR3Z/OmuOwbdcK5xQruXikjhtknY1Eo7sQGU3ZWJ40yVu2dIiirL+0+zIk2Lpr





n5G6njvHOzjnrL72mmdccHIWkqThdJVFgkA45xF5GqU+GrbzoZGN9Aa97lwxOZ71B7jv29sA





LcfWVdflX/2KIpIEBhFv+lGWqpUYxpmDmdn+w7vFznOjB1fmtqvmww/al7+66Xt5WcdcGOMh





YWiMVTJxlgCt1np8fOIrD3zrnPFWrQ1nUGkowS0X3S981kxMMCY7ux9rQc1fdGPeK8m6cqnT





v/9B0oPWar/JZdeUEvsINbia0JUFjLYi5vHbB2bzuu4PdBxRlBBq6teF8KyVqpW+oRQZs5XR





pjZVVc4fGn34fjZMs/Z6vbWhxw03b//0x3cPf+7gju9IEMd3Rsmtb/2FK48c6UKU/90H57UW





L3/h9Oe/OtfrGkSJ3IL1AAQed5yv02bFFGuNxLX2C0f6jVitDPJB3R/ZEifUXlg5Mjkywhip





RGqtBecMHRcsjaNeMSAe5dUKgkqaijxxDgtzfSkjKWWtTSR5VUqPNWMcDPKk5j6e3zdp6zbV





I+O7vqJiuL9oGgAAIABJREFUceDenZsvX4oxV1HiycYq5uTR81gSguUoIpE5Yy16TIEzxjlJ





xpuNhvDegidUXMYP7+9Pp5M7zva1Nfffk3a7BJ5d/6MRKwcjXEbED8z1Jsd4Y2Qbcd2vClYV





//IBeevrStISHPPgS18xw5y1aDGWUe1NLJXkAr3uluVsp5970+A0EWVd3e91TXs0HVTdXq0a





XFhTziwtxkodXuh+9SvtKOkWXl94bqXdciKbXJSKi27eQQe9ZZfE/PDiMgFZWxWlm5pq9Tv9





wtQpV1a70bFWp1wutUklgKduR+dUffmTZ1nfHzaC1rrDe++8GXbWQnDHd4I/1QfwQ8CtL5j2





ue5XvV6eewDk7qO3HUIyiOKS7fzBA96DQwAitfdRNJA2xjpXXV0uzRfamEFFrXbLdgwWhrfM





pomJWNKgMjOHlscnM4ZIjEkhnHNxIno90+/apAVVnjNSVSHTtMEFAUAjSj04ZA6QewcksNZO





JjC9fQC06KxBSc7S+JYyliVYpV2FHhkaa1BKZDLSpS+qPibSCWKcwIIFzSG2nIw1DMnWjGWA





Nr9oc6ZEntc8TqILL+9+5Y4WuFTUOWfExkdXDi32+iZrEhZd0RBlpZXlr35N2smnGvzIWGts





vrvApSQOvkameA1oHXa5h7JXaS0489y3GMuyZFBrEccN7kpn+qWRylmqloueyMRUNjI9Mtow





nc/c03jxc1dIpoeWNGPEWTMVPlbTZVmDzTvdfjNOuv2SSDbathyULOJyYGuwnLOq1LoCJVTR





tyqDVpRS7jRb4i4a/rKNRmPVHTi8m7xbfdzcU1jjfgh4vJu+BQBAya2+NI8uLliytmqDj8h7





RgSee+8f2k9O2+HgUY81wxZ35cS2LnlsjZBq+tpUM7Mz1ptBTgLZWKvhUSglJ6YTcFhb1+0N





wEG/qHrLRUPi1Hir7pXcpcVAZA1V1bUxNs/rsjDeCmSANbbbo/v2ABKhj4gcI8VkSqCEjJKJ





WYmCIs+lVLGy3iPjK4O6MlVeFTLOLPdIHhhKKa3jjqH32BvogbY1amMcoVApB6LIkTUmjaMb





b/bE8+5CKg3OHzhkZDE6ulUkWVH0badsqJiUhJRmZvdb72ZXFhkSgSDOBVeOsRVTVt76vMqN





WcnLxV4JkrZMjY1kzXPP2tLpFnHC68KDEpWpR7NmZc0VO87JxmKSfPqc6BU3L1c+mi/qiWZz





ut2eaDcFqSovyqJO4kzFXMZR1hLNuBFHCXABHlWmiAjAlVBHXMY84bHKB7bCGmJ1/fM1raHR





aKRpenQoih8+qWYIZumVT1m1+4EniOPxUHLrf3rVrmwku/H6q84aG7vg/GY7cgDoHHDOEcm7





AunoBLesXXDVu+6mqKUyDXDPl1OgCAUJBY1WOtZSjSzV2vXywdxiryzr0fFMa91uNrWpszhW





KhpUaIwfbY9KJSOJVanTVOnag1PWOibAFr72sDC/vP38iFHqwVtfOTRKikjIoi+TiHOREhEQ





MmIjsSCvCJnVSTIaR5nkUlAcEWeRpGYzIwJELxmr0DoBiGgB67L26LX0kgkpGi2RXHVd6+GH





deUtEu35lmpv6htjU9UiQLRuYAZ7juyPI8GTxBB44uTIO9Yx1VKRd/OitpAzx2WEgtWivu7i





SwH4YFAeOLSoUoks5plAi9bLPUvzTKb3PLp7fmZx3+HDK/UgTjY10qgpYxZFvbIsB7ou+qOt





kWaWlroky7Q2QiQ+0kiSJ4yYQGCxEo6hBGQkPQcL1UCbytna93pVvumcYu2tnonouDuA1g5x





C+44HUEcp0XJrb//+69d6HUn2g2cne/k9bgaufaayKMGNMYYBPR+OJeeSaGaTdSmKuvlbdtG





v/Tp1q5nzOuqH6d227aR6bFo06YpYmyxszTo10JAFGe9XhcsRZKVhV1eLpxjRrNmKwH0SMQ5





DfqOcZ41edZCIbEY1J4EAFoDRjtGIBhKKTkRBxTE2yOEJCvMiUghqzQsakTllGCANQenq9oj





KmCCU+W9dY5zzpQEYOQYOGbB19YYtEmcxiqO0yY5t1hUI63etVfnirMsaQhB99zhd9+fejfg





gikhx1V7pDXSmoSlbo8Ae2VV+BqRxtNmW6lURQacMdb6ejRLL5zctvvQ4bpyUsbOObLUL4va





eBBcpDFSFAsuMrmoqzRKq8rVXucD00iFEqysbb/Kedy0XhrtpYhlIlvtLJWRwJgI0bLaWcNY





5TxLZb/0jSQi4JJLmZJXzDoe08jcgZaSKRHh8XvFD90B4I/eBmlYBzC0WE5DyHGcGs4n3/6b





L+g/9OCzL94BzgHRVLP52MJ+wspX3EszfKwZICDyZ11X7dldH94XXfUjUjDzhc8uWZd15hvN





yZI8K7QlEfmiBGdkJCqt0yyzzkieyKju5oVMIvBEXKMxVU1RHNdGG28YN1JFVWWjiBuGxcAy





5jhjqBiRF1Iyrqpq0G41ykEdp9zrmjhnwJCIxRhZZpxLOSxbxrj3KKViihhw570nIsEZ40w7





dKDbcfuczVvzQTdpNbmDmQMzLOKDQd2MG1KQcSUjboFSpVfm05tenFeF75VjvloeGJ0mPDY2





J1660oCeHG0LY22JI1nMvUVnLbI0jibaaXtsdPbg3KAs20mr78osa3Cjlwc9sFoIboxVjBuL





ZIFzU+iqMqZwyAVFKu72lmIhgJCUkEJMiO35YO6QLr13EiOUMdheLRiLRbkyMAKsdnFDLax0





MpVwGbXRfuGr5bOf2ZzzPYRxbXIi8t6v/bczTbN+v+8dxoko8nqY7Ghkzwij0U8mRByn4Fff





/HNv+5Xny56eGM06hZaKiyyaHkmfd+klkyPj23bWnBTzR1vFE+M2a8CRI9yBBpixBgfLE2mz





TsfyWh99tKGrjEBUEWecIiWN1rWukBli0Gq3yrxCAA48yeK6cnVZoQWwODU5qiQmqXLkiFGa





SSKIUx/H2GgqIbx35dTmKS5peZHK0rfbDcYYF8yBc5YRMqXIcjU2HoHkHoA4gBo+mYEhotHe





Gcc850JxxvfOHCqNm52dne0sDcDlBQ76elBWvW7emStlAxKZPPT1igusqqhyHMWMQ0DrHTIn





IwJ04LemKfPQt6ZTFstlJbmKRSqBOBKTwmnrwCNgjT4GJEQAGBFxmqbcesm45KIZJRFXiExI





5UiXeW2knpi4oNUajRLFlOz1eov93nJ+oD2+ZdvYdBZlVtVO9drjSYQsdYIpniZNQax2msWR





llWK6s7PqWuulp//QhRlyLjn/NjzrE4A0zSN47jIa+fkarjRyJ7x1FXGH1BCxHEKJso+tVrM





dOOGhFr3q5rKKk5j523fmIVZaxE9OgSU0l91kfv4v3JEQM37pXdd8t5eff3InoMLY5tGvbGx





lAkpksKAiXkSNePZpcVGa8yDE4KbWkdRlBcFQ+TKKaWIGJAfbaa6MopJ9CbiqnbeWJukERfE





GKtLiySSRDJvAez42BYZLyz1+owYAzr6zGf05Ln3tkTgEiVjw2dUA0ClnRCCCZXEqtuvtmyZ





XFle4YxqXzEBxnjkqhkrwRh412xmVdMc2YO+X8k4u+7pFr3N815l2fR46hhqVwOzUZQMymLJ





8bzbIZJENChr5lXaaIg8V0otdpaXoQ/OK6U6vRUtPBATzjl0iJTESVlXjtA5IMkiVLWzk2Pj





RV0yD1/9xl2bxjIHJDihTL2HgdYrvXkHSlGaxKNLS/MG/NT42YOVBUCTGyOlZMZqb5O47WtN





SF/65JRXvbrwyVThBmXeTdf97t774SNikiTplyvoEYAfizuuDDNZ1hIijvW8/ddupbGsIfJk





TEij41SmjawmbxxRqi7YuunKqwoyFTFCoLKmj9/OEBEQr76m1UjSbz8cWVvXdYcL6g8GjEgy





5QgrXQOCQeO8761UUhZSCsbRWhs3ZZSKuBFZawiBS4xi5gxKya33Rrt8YIx2RlvnwBowxjLC





KJKApijKQW5GNnW5wFaWMo5CHo0phBDAWOmRC0LnTa21Y8Z5B0wIAZ4YQVnViDi/sOSoNhVy





LgG4SLjWrnaWSWKSMc6SWE5t59OXwK5LGSdqt0Zt5DZvG10adBG9I0Zs+Jhbs/v+ldZYum10





vJWlUSuuyfQHA57EeVWCl7k23aLM6wo5CcuE8Q69kCKVkgCEEJIYIAgSxBnn3DAfcQmctcYT





rUUj4c4Zkago5ipW3Xzg7ACw8lBkjSg35lBnX2XzqfZkK2pIyWMlHeh+vyNVNDJqGCdWR9+6





r3XVFTpuCUbxuluQnNDbErd3nn/RmnoRkh0nEMRxAn/42/+Bc0m90qwMrPOIAN4BmGazSREt





zCxF5MfSKU8FkEXHjt2gHBAgN53JNCGirdvJAgNERQzsMDIB7YvllU5ZFb2ir+JEUOatYYKp





VPb7vTiRiJBEidGecyFYzIXzSEIoKURd1ZzxyemJOFaMC63t1PiE04YRV0o1W4lgkgF5ZyTj





Qogk4gAA5BC9FOC98wSCK/LgLCEwyQXnXJeVtd5ZAHDkBRO6rq1DVle+2ZBRgyGnNItsbQVI





MIyAHAlTU12bpoqds40sG+jaIRQlLfVyT2zHFcRIal6X1LVkMOJRI62dVbE0tSFGMpMqElJK





pVQcx41GI45jznmWZXEcqyhSkRCCc05cMBb5BKSFignuI2hEI6PNVFsr43g8badKxalsNFJi





1rBapaydjnGZDMqVXt0ba40M3CASqXZmsb9w/sWESN5x250uzSAvNfHqdLc+I6IoiiozgDX5





0UYWeliOE8RxnN9/0yux7DEDQhXRVEMw4VsKPXPEZSNN2q3dMwtxJvqluWRX1owF8wgI3vvh





aCHVHBAwD47JYjAovWORiJNIIhVFXXXzajhQc3m+bmbCmNqBA2+1rlqtltYmjdOVZZuNCBlp





rWvviBAH3T6BlIyIodGGEzRSHkuhXYmAutStZqPIa2ut4NIaQgBBjHGUwiF5xkh4ROKCCXJA





yId35qvKEpxRkSpL4JxJKRhjWZZxjtCT1hqBBDl6A0DkkZU6F8SUjKtOJRUtdysQ5IXPdeGB





DFjgFUVeSqlr2y+7tasQUcXSem/JWrK1t3GaMIYWwBFwyVUWG2MQ0aCtwRkEYgAIUgohBRKR





pIhHWRzF1PSAMuEH5xea8eh4s+msYQzTJOIiSmLRajcaKlYqtqg1s55JRjS7sthW44JMK0vi





JG63MooGiAhY97y8ZNfg8ueAx5KIBJPHumaJCAkTAKiqauZwR/C1gUYIOo4TxHGU337ji1B5





zseYr1UWUyyBWbRggXiiavCURk+/9LwkyS6cnr5h19alFVZjAd4PBxnqmgnrP/Sx0js2ucVW





FcaJcGi1MzPzeZm7RtZUMjoys2TBTU+NR2kiUEgmOVfWmGazVVVm8/ZYqYh4GiWSRxFXot1O





siyempocaWRZIuNEcIbtkZSQe88AaHm+K7mAWpsiZozAOUAnpWykSUPwZqy8xFjwlEunOOOQ





RCKOYymlJ3AEDIXKGCMiosppbZwRmqcSYkcCTelX5gf5QOuKdO3zbgEIhfbznaVIRHVlpYys





M5wjCMEdJalyNkbPvHeI2C/7FBkUPs4iYOS5l5GKoihOYuDknFNJDIiCyUiJLIo45yyi2hrr





rSBBnCPD0leRYLr2TkAykh5eXB5ptlvNRCRRI21qbRbyYq7TbabNCBiClwQMbDuK0Xrtyrz2





dU2bWtMI7OIrCu89Ee6+c6pO8kH82ObLFIA1TksZwbF7u3PVy5qMEAUh51wptRp0hGEdq4Tk





6FGyTW0/N1jRvemLJsl60NZa4OAokzB5tkRZd/fPH1nikR9pjRwYLDGsHNQj47rXyQBUe6vd





Prr564JVrtTGJA3R6eWSZ53SjqbJoSOL02c1EGIh1fYtkx4gQu4iVuuKCwLCoqi7/TxptrUF





RK14whAVG3YEIyKBB4YCmB4+4bkqq7o2cUTtkayqi0gqDwMulUiVN+itQUZCqUGhEYVGY9Cl





SoHRjMiBYQIdkCeMM/TWeYmMeZIctI5iqgqiTHHJ4yheWeoDMQBvrWWoeBwV5WC0PWqdj1R0





6MDiyGTGJYJlCY/KquKcjbRScAYRgIPzvva6KArBJXLPUHLE0mpGxEg4bTwCGQcIxngVxaiZ





l2SEg7qEuuaARcNzwyex2esNUIFqicro7sBWdVUVxnpXeVuDP7jSYRoEyumJTc2s1VlZ1Acf





MwiNRqO/0p/vLkvKHDzKcJuHAZLhPHr6tou+VH9j9Bz85oenGFPDaoCIzqjKcCSH3gPy4a06





Vt99amrnDx4h4gAAePvb38gGpRQwefGkP9z1ncr0KwHk05gmm0DW9+esddNTY1NTk+/4u4P/





+J4jROqmG7dKFjsH1tWXXmgcRtquZO28yhG8zWJlyW4ZHwPw7fFWNXCCWLvZ9t4yB9o6aywh





95Z5wxERXOSAGCNbY13X3oMHRSgJOCPGmUBEDhEjbjRFUTw2ETEOy8sdZ0FFSWX9ykALJjaN





N1MVpUmaJEmrlQjuGpkUDNvN2BF3kjNEZCyTShgC45y0xLlBL4ExLkiCEMp5V/bK+b3LppYO





vWJMCC6VzYvcOamdK6tqpVOPjrUwBUKurCjzwpfek6116b2PI8W5AME8Ix5JEQlgVLq6RpMm





UglZ2krGEhk4QSh5BbX2xqGFiLTWjhARgQEnh14zxtqtMewDB6rQeAJMwKiSNTxLXNKQKkOv





iLN4ZrE3s7B4cGYJPO+Vptvtlr6oKpzvzSmVXHnDvuGDeOtu/I393zIee91aUKMsDOcCjz3n





xvt6bcojBB0nE8QBAEALh22uWauJhzrL3aLX7fIkhnaGkgwBI3LOAJG01X98y0e96SMwb+1E





c2V6W8+DP/+KhTs+Cw6bzqfnnt9PE7TOgGeRaCRp0mxmLcWttXmvt2mipUg456Ug7RwSB88Z





Z8TYwqyLIsEYR0TBIw/cO0dE6BEcOeetMXmeO49xrBChtuAAwEOSJI5Vm7eOtiNZ9yqrfWV1





p5eXZZkqOdZqE2ArjouyUix2A4aceeass4ILKwCAA0IiVMoTCQnTKuJoay7TONsS6wqX5+r5





OZMPHJDkseJx6hE9klKKK+96YErDGEPBORecHA7TItY6B946DRYZK4024NM0JaK8KlFBmkUa





aktGY1X7mikCCZ6DBx0nwnrNRewJCcRY1hDTuAILraxl0Mx3lygGD7XjpsYKOKtsUUKtsSh4





6Xg505vrOV1FGryzkgOLNHbuu2v84XvOWziUISIie/Rro86aTaxxUWtHwXqMA4Bnxx9ut/5h





VFLKYVUJQceQMK0e/uD3Xi/3zLDzt4my9rnp9szUZEOMR5RKcpYnkecN6fLX/8QHR7dH3cNm





eJti4+ofe9HUJz7XG5moqpWsN/AezMtu5d/aszA1ni10O1EaxxGkUcYZoudp1vTOoUcLGsk5





4gYceCLvGUNkBpFJmTpmmWfIyFoXKa6Yss4KKZEQwQPY+aXOpunNKlIEaJwuyxIRRsbH67Lf





iBuIiAxW+hVjnLgAB0lKMUWV1d5G3nHLCqESXdSK8xpJRIwzAo8xMHCWVSptpJ7qSnBlaKSp





Zg+AVHEzJkCjpEhbPK/qSGUeDcQoMuYqL5ARYlFXXpNXhiFHBoxQSQnotDVgiTOOgEnEy7KW





qeLIvHdFWXqP1moA0NpRpeMo6uuqsnVhaqt15Sx5b7UhS8yS1y5KxUI9cJWpfMlKKF2lwSni





K/2+tUYaBdYi8k53xVBBTs0uzXImO7ONg49wXTGtrXWF0eCgf8kNC8bbXqfa1Gh96ba4zPm6





WxZaa4e3R1599GRd1xBm3ANAiDgAgOUVbZ0mY9F6SFiaCPSVdxbLkpzxlWad7j+854sksX/k





6C5IfrydfPS2blVVs0egVxli9pwLFge5237WtCeMZNLr9bQnzyyg5BRHXCrBc10yHqdyDFxC





ngmUjJxHIGjrGr035LgxxpZeCkbEPAEXoq5r8ICco+ebpqbBe1NXta4YY1mcjbbHDh08nMmx





qoAkBiklY8g4ZlK001bVxaJrq4qLuDJUs8hVVSkUN9xK5QmBAzIHhlyWZWlTcu6VU21gyJPu





wLTHWKwqzpmKWKcw/cH/z96bxdq2Xnde/zHG18xuzbXW7s7e+7S3973uuzROiiRKglMkkFSC





ChsFAQqFqlTwEPGQAiFUoCqQKBBCKh4IEgEViCpCVSXECXZCqknjpOLYznXqxva1b3/PuafZ





Zzerm83XDB72PcfH1zcODzQln4yHrbnm0l7SmvNbv/kf7ZdUaT2sXGVpVElMAEhVcukqZyj3





EogSNDPHGAEYFmP8MEYAQ47iXUoJQsRsrQXAzBkKo1rwJkVmzqRRUzJMUIEUZDx5b41Y7rpg





MkjhxBambKqaQTFFo+x9udJVzAixdwW81Ju8pHTpq89evPXinLRkZmNIxIpY0voLv3rIBvvV





5IWbN8q9nu9NV/0m9ubd/9Pmtz8Njv7Hf/Vf98dLXN3H6SokLiaV37OMkDUjJYXTIZHwr/6D





G8JFzuF8o5CU5HSRmMmIEy5yHkh4uuVE5MatW/N2sl6tL1/aPVmuN8xtQ57N3PHzp4sbd5Yv





f97ktK7c7D0fKvYu2aQha5Vyb6SKIRsXBUSing2xSCZi1L7ImimDnCFioixiNdOwGRTaDd3u





9varLw6XrxR37i7qhpqiCWOct81rN2/WTeus3SzXZTUt67zcwBZMhML4EKMzhsUoq3Mm2oRA





jk0yOXa12+4RnRRUieuWC9PU85m3Xj1XKUvse0c+byIFsBM2CC5FIedsQGYRVVW2RkUFICUW





K2bMg7PWiMk5IyszMyNnMQzNiBmLUxRFqehK9knjQMIkYAXFGDQzBo2k8IUJEckQgjprur5L





wpRyad3IQ9ePM1utdVPYyVdfmEgkSFSVc9XAWUXS+cu8cv1k9GOxt3X2uVdaZgJYFUT6FmSc





C3Pn3Lno+FN72BWHsyK7MxmGxXJMY8giVPueclicUMicRBF+7dnrMatqOt9bhImZce45AEh5





IJK9S50oH52t24K7fq2qEXnSVhhTIaO3/mf/1u1Pf6J59bO7FqZwNqXw2NNlU5lhXYeNW9zy





TrgwYliIUBceAGc63/ckQ5113lgjwoZFvBAbIWctJVS+XBzF7YOgPhRFKWKcM+207kPcms0n





dVmWtmlc34cxKAmrwrCwsHPGOSHKbChqMkl8ZcYIKILvkE3mRJFO14O0XryIoRypD2PukqUi





SdasIWhYd+NyiAgqnFwgopyIiMYhDinnDKgwE4l664g1pJg0gynmJNawIWMkDtE72d5zZQ3n





nLeOlSgpNCmImZPRkKInFkAJBCTtI3MUtcayBRklK4X3200boLUpLWR6gUSEWZhZ7hkzi7CI





fOm39jX2V5qDo24tZXce2njbne3uF3Gcq6Q/FR0PNTh+8kc/RJsle3t2GuIYF0PozxbD0cL7





xm7t62Q7LNbGlM/+9otG3pRm5zN76M3jN880U7TT7ulHH7v5kn/293d2po8cXimQAwObNf3a





L5Q//3dOPSbCKlIBBrmY7Qy/9LdPTdJyEqs2Xb4C0RARmbiu/DmgRESIvbWlLUtbWCuldV6s





M9awceK88bs7u0XFly7OZ21txThioyzC5w/GN38oRlISpMxkCdaJRc6UUlPUTDCGRURYYIgN





GUe2MEVtDLGDzTkX1rExq6ErmCxEEhnPGx1DJlP4onU6L2laUjDWiWhBAgSKEca6YUxhzDnj





vH1GRIzIeW23iDjniCillIGqqohoDOuuX8WYYkreFE0zEe+iELNx4kwpasDekjBbw8ZkiYUY





I8aLKayPMY/j2KUx5RApz+vJ7s6msPQAL+6z47x4xT73a0/2fNbYyfu+b5FRfBM35f7ROTvw





cGdYHmpwvPM7PkimISKTqGjqceQ7y87PjS47hYmLjTg7nq2v7RuJqvfD6ff3mQaIYH2++ujG





S/6FXzk6OdYY6Prrr7Vm/s5L7zi6u3zhC5fEWO8cMZgFGokI1H/PR+f/wo/LS8d3jkOXjUal





XmEkG0k5RYI6VxJgWGpXeTI5B8qp8I6DndpZ67fGTdisJA8T7VtQQgZFiDhhM46xrCsmBjDG





4cbNY0KsJo3qmMJAOVflpCjdGLvC29KauihImQwxMyx1ogyOGTkENSYRNOXC+D6yq4l6ZO6s





G9lvAo/JRFDOJrdJKGdSOGtUSKxV0qSUySjFPlBQRuYYkgjjXnoipWStJRBSTkKuqqrabrUt





gJiyKjnyhdgs1KUQNJOVRBpiHHNKUCde1Ex8ZY2FsrWFdaVmdc6pA6DQlGDuI+O8QvRB6UEw





z37qatlKpylr+BMDHPftG888VPbwguOjP/gMjWs2SUc9Wp5Ehpl6yZyXK0gyJ51kqGVrzetv





rJR6vje0556RKonwM+9ep+HQyYWh0zBiNjfjaJrSbsLqlWcPlRiqRPeK0wHiuHdQ9eNmEdOX





X2n+8JP7v/HJ09/4zeFWdyMYFyEhJmsLoeys9a6oXdG2TVM3bTVd3c6Xdi9OCzctzCMHB49d





mbWToW1YWLphLIrC+5LZiYooa9Zs6WwVMlCUPqUxqTXqy6rOpiuboqo8nff9p1wVTETDMJjS





eMseKMja0hciFKFBSaEax1HdrrF9aaR0tjHqNFrqNXU5GhaIF2tV5+pSTNaR85IRh3B/rJY6





40SJQVYMshoWTdkaU9YVQ0RkzLRe9wIJnDbj0HUDZfFs1DBD8vlQUIIFeRLWHKHZUGk9iBJl





w+xNmQ1EBVbi2ELLe8iQb5QeIgzQH/7KpbrqvvsnXmd6+y2438IOY95UoA+t6Hh4wfFnvue7





2FqKOXTDpct77XzaiIvdhjKnVQ/D+WTFEDJ8eO0QXL3l3889liuPLcdlFl1BKyT1Zv6h79Lt





PffJT5nf+70zI5lpZHlT6uLNx5QsF+Pv/Hp45VXsuaMhRtftHzxNTz7+ns04Eli8ZM0ipnJ1





yNLgNsdzAAAgAElEQVSPoxOXTv12ufPE1UuWYllIpjBSH2Kw1hqr7aTZmkytMZZgmVjAzAPC





2ekmBBVy/RInZ2PY0N5eXRQoSkMU69o5Z421Y+h9YVSVKxsoGEODg+bRpJyhprSFN8wMUBo5





B8ZUciZOmkMKXtQa0Rw1IymnLMlSYxhOgnFenOMIPp+aEzXnHMVb59ybWoPIWptzXnfdkELf





9zkjMxFL16lmSZph2JCTLI4dMotI5QvjLBtR1cLYLgb4HFN2IGOMUkLGmGKM5uSOGL+4Rwr+





46SHiPnCJ56OvJ5d3nwTajz40hjzMIuOhxQc7/zQnjk6Qz8iJTupwmDOFhtXuU2ycTOqEnIi





77JQ0vSp33jtGz8h56zQebP70ivl819efeazZzHER5/sT45H68ZHntjcuaXX3rXM0WhWwJ47





yQBy1mHj+nX9+rPNadiZtsP2+27vzrW7s5jZgtkaW4c0nhyPd04284lbbTYQO5uqZyNIfdgs





YtelNTNZJ7Bc+7YwZV2UlVSFKytXtlWz7vqzs01OUnvTOHdwWFw72N2eGeSoOZRsCjKhSymq





Kpw3xFJW0jZlWXp1xEZlXg5GIGwd1AgsfGOTGQOnIYZkKIQoJPNlMlmdcyLijXVdjIKYElRD





JlUuPLcF8khGXFJJgn4MIcaUUghBFQomY2GJsihxWVpjZIyJsxo2lSk9JGoQo4nUuuJ8q11S





qKqxNmYIMKzZgCtbqKoRy4arqnQ2hmSF2gdJ8SAyHmQHEf3TT7zr4vtvKOz/TW/lPHDzcIqO





hzQd+0PvejeYMAatqnET2GJufWQREfSUdJnK2hqTg54iZVZOb/0EEQHHzz97kmO1NeOwWuxe





kN2ZaRoak6QyfeA73HqQ9sLZ0Js09MAEBCJiEjGkqim5iemu/tAK8Xwzt2RdVfhCc3QTZ1uZ





T9q+W4Yhx/UwM2boFs2kjdluun7ZBzQGRhvxMQ2sybkGWQupN2HZxy5pRObSpqcuX1usR8sp





J8ynkzH0vvRMFlBnTMpRjFhrjIUxjsWMyE4sgJxDMbEaNGskUzjDKSXnnKo6b1JKzlWhH1B4





AkSIkKCUal+CouHW2JOuawNnEQKLEABrbc6BhUMmZkqqwiblfM4C60zXp9XQezgV46sSIU3L





qq6K20e3XeFjzHmMSZmTijExRcRsiVNyVlxZ+IRgR4qab6fEMfqy9VxHm1VZRFT13GF8c9N6





uZeg5TffjTF++f94Vzldd6d/rLdyvg3L/WVAhIezgvIhBcf2oxfyaRRrCXxy825RVDLTrzz/





+s5sSwsS21BGbrx1lE5iypngCBGA3s/LUcixIIrG8NkqQcudScnuJKuLvc4mFiTZ2PnWEZVU





T8q4Ci8+t3U+EV2z5ujYbI5v1ruP3q0ry2pFRMhCETXWnGrZil3k4B65Mpv4uQn9/uHe0AVv





HfXrkLXwVtVU3uTMYpxzTsAxjl4olkV3Gq/tbG1NZ5xp4tl5GwmKUPkWLGIMUspQV1ZQFeVM





2USbQGKlKJxqELGUjUbuV4PJNIbMQqpqhVQjG7uJoSjcOAbvfSIlskqkisiwjk9X67Ky3QaG





Ru8KESZNKSVAYsrWMREz8ZBCStlbEpIMzRnOlinkmCj1ufJ+MwQNUQpbMUcTVI1VXscNRvVs





jWinyds8rim4IazTOMayKCc5jtDSR2VmlCJLVSXOZX20Pt26T4r7B/ePU+q9oVHM/XYVEN6C





jgf760VMjLGpP/iw1ZI+jOD4+L/8kXfu7XYxt7N6XK6RVSla5/f2Zq0v0+auVEbnuxxSVt1v





nM0OFADKWZjy+bLJ2TSNLjYg3QiVJNTuq3P24t5lorzcrAs2QzG2RXH39NjEau9JH+LNmy/t





IhUAxEaGX53h7p3MW6um3c6co3YUHaCN2fXkGGQ912stHS4dXOlj76RYD2eWbVtUYRjJIA7J





OksiYdi4ai5C6rONejCZXdo7DCF2aagKH2MsrM2pUCIS64kHHZmVAnxhicioiQwjPMiIrCVV





WQN7bELfbpd5nTwopkzKo5IvxQjFCPBoXBFJmVU1M7ExRIYypaYpiagueLUgMRJjFjJVYYeQ





lBLBhDGoImcyxgBKYEGyYodhSBHIqSodQUvnQk5CggSbWZzYbCgnZt6MnVqWrDGiqEgHL0Io





eQx54useZrFix83Wo6/feGFbzYposlkYonTOChE+LwN7EB8529WJz3oEmHv+yTfzVgCISErf





oEi/1e1hBMfju9M+xWpaKWXvyt2Lrl92MIQxJ17YpoFJRzdvVa5od1vyCTQCtO26XstNACCA





gszO1rhc+GyImdrJsN0WQ8p5vSlnTemKvelkf3/7zvHpC6+Z7d3ZS6+tLl+sb7260DRTDAQo





KbOsVnnSiMvRETJTKe1usxPTajrdMllLY9Xq7vRCCmPhiyEky9w29e07HYmmENna0nowj4KU





RlD2xjpvJrNGyJra57UaFl8UDJ84l9534+CdF7EgiAExKZECYnOIsSJnjGFl5ypYKvYsMm8M





G2I/AFkjKI4qniwJiHzBi1VXVY5ZsiqZTMwpZ0WMIdbTajKVGNI45NmWH4ZBSWJStvC+TCmF





0DlXaBpz5pwTgKIous1ojCmc6YaNgscYch6SYKYtEWkxetQsOaWw6jvDBoazEoSSSUUuI4YY





8xCDTa2a7vi4hAxPfmD1/BdSifrq+9/44m/t3VMcX+ewnKNB7LrbGEP8jfWjb8uO82TZwyY6





Hrrg6A//0DOWuN5pjRcw6WYzLNft1hyNHUKspv7WncFM6t2L+4lJps04aTL0yqH9j/7y9//k





D+1lso4tgDSEJ95xlSUIibU2BTo+Odou9rPXMUTjS2berRpK8sjBfm39hW0Wofd8sCIaCecP





KBXk0k261aC6VBnBIXOE6nZ9gUFFUTZl25az467fns6CDv24acqqINnfmu6Ue5OqctYJcx5D





YSxRsiKGmJUmxSSEQWOqC+/ZWSUjrnJFt1pP6ybn0VnylgtbGLBj8WINXF1U3pQCIyKGnTPW





UWHITGfONaCSslMhqpzXpGKwtVWT2K3tdm/ettOCyITR5wQh48RYclGNtUIplpUVfnOKshGb





kjp2UDHGxZjOf7fOeWaLoIdbW/OKq8Ls1zNr/XY7r305py0iCl2gRP0wrIfgrJ1Op+IkcCLS





rDosu8XZadQAJhJe8cnVZ+7mXIBF3Y0n3396+T0v1o197N3Lt8RKv84VyXLxqjmfWnyfFA8e





vGVFEdH5kJSHyh66L/zMhUMydL4ScOeUpr7aanXKw43+0kU/DEN7UCZxEFSFU2MRJ6rhr/7M





j8yfvvxn/ux32jz2uc+qwvyZ33uNVYqKm2nqoj1bKiFU/sIbR4sUuroq8zIdHMxL6zQm791B





NYnpttqjnIvz8TAJnDbGkSf1kkCj3XJzzyKaSzFNUW1PJgeVY4STxZnCllXlOBdOJvXUW38w





PwCcplxa78UYMCuYubDF4mxZ+kKIjRSlryfVrK2qwrjSl+txJSI5ZxLepH7MwYhUdlJLPXV1





Y31Z+MY3oIwMx6byDau3XLrKzbe37MSyAwquZ36Iiakbx5CQDaSesLHxvMZhWjTO2/UyF+Qy





+9JJzlmJxFhiIkaETWMufBahlJTZDP1AOTdVhRwLW3l2tqomri6KglnURFWt67op7M6kFSud





ajcORFQZo4DxJEXpyiIn7mNUsCUzaeWxD7z4yAeOmQohMfXk1ktDu5eE26/tiXA+MvDNv8xM





x2+8vdagb0jKvq3z8jDYQweOs+WSvINlTVm2WkkRjSJLGu6OTKb01byhSKmHFHVWZqP/3c/+





GxxitiaVdZKSlQmOLM5OnLJu1sOtmymu8+WrB9cOZl/46h926zhmCaGfXNDdLRGmqqoqV3Q5





uaK+8iSUe9VCmBnjzRtFWfHq9bspsTJDgBjiMNTOpXH56ed/539/9jd/8wufXZhsY5631e7B





gahlmHbS1M20cnY22zIikuHECHNhLQK8schUFg3BbjUzb+rS+9IXk9lsXu/szneNsyEnYruO





MTNlGqxnA2rL2mfPpMymttXMzA3tEDuwJqSgna9ZvMwmVYzRObdT7/naXr91erJatzzdb9q2





8LPKjxiGOLDpBGLEzk1dsJnY5ry8XUSGccWerSlVtSiKlFLd1N47Q2gqV5iysfXRYgHV5XLJ





sCFEW1IXlpswJgOGMsNam5my4WwzDPlayrqqWyGjxYT7rBOQsZLllrUFOdtaTC7xSIur733t





bXtS7vsroHtjqL+BF29ZUff+kR+qvOzDBY6/+LGPlK6E8RqTrjoKYyqEIOn4ji3IX9j1u1Mt





PQy/9pWbwUMzs7KlArVJd05zTNABAFEgFnAAzptKwGS+8kenP/d3/6jvp8b2+85eOdjZrS+8





/urZtG4aZ+4cnVxoJ17lcNdsXzqGGUPMQtbk6vZqce3ae5bXFyloP24Cxf3DA1viKzdOPvHp





O//kueVzL6W/8bd+5efe+Pz/+OXf++rrL9eWDi80h/s7OQyzqkqrtWEGkSFOKeWgjS8mVe2s





T/14MJsXhXdWSucp68QWeRgReWJmE1/vl7OZK3POq826EFNJgSEU4pw1pXWlkW7sMo7LDA9b





mtJROXS6U2wTqXNOlAdsapG6Loj4laM3RsDBNKhyVMpZxkLBhSeF7zCs0SkSAGa2lp3lXiMR





jSFoVkJOSKVz63XVd3o0Lg6mrYrut/PWele6xbjg0kQKXdgkKM5zOEQ455xX7SZ1ZarJbDJp





lstAInd1WVCztT2P0KEfa1flqM9/+olia11OjxT+vty4z4xzGXIfFN+gMN5GgzyE9nCBo3QO





SJyVu0FNGSxTRuoHMPzli8QKMHqQcXeWq8JXUIUOauqcSu4za4r5zfh5TvHepxIQsowvv1If





L6oXX4yP7O+nEC607Uunt964eRQ4j4gfePqprXryxP7F3bZ9/CL2Hz0hQwPn7cNh6tsvvfSS





svRpOQ69s+K8/Je/9Pd+/pdej9f3Fy8/6i50Ox/ef+6mfCFs/do6/+JLf9BUzkie1v7i3tbF





C/uejCeZTiZ70wuc/LSeVt4Xzl2Yb1e+iOOIpP1mmDVNCmFrMpcgk6JoTTOGYeInBZlZ0xZi





nTdV5aaT0oBrKTyVtasRVQkCi5yHYUxDXqZVQYURu1wPgrJFWTcVsxEuQsgNCoNyaua13Vfj





h/PCfWSCzYmZtBSnqpvT+fLM1OQBIbYxqrPGWuMrfPFzxZX9a07MGMOYh7X2vaTMuWxKcsIE





tuJ8yVKQcem8ADynbuQhNknt9dvH69XgbOFErTeaFEq+FCmKgcattqLsrr8y33935+RrzSkP





6g8RqVt3nwx/IiDuS5KHR3Q8RFmVH/3oO99z7cogY+4GLnzkXHjPQ9CUDQMpUzdqlOHu3RWC





szk5saqqgrhiJp1WcVwqv80VYwiyhXQa25RXY3TbM3e83Hz2cy/fDcPZl1+bTMvZTr0hYWIz





picu7pSTzby9vokYeDnm1jZktY4n9IYeKee7X1qIn+ahHQWU9Iznt5ZVvffEzpW9V/vh254y





tq1m9bz/0stQamopi8PlalVU1dHxYntSeWf7YdyZ7YhQt94UReVKHUOvKVVlQWTbptl0i0Sx





rUtiF3UMFLrUp2hK52B54usQICiB1Jr5gE0GmLn0drM+Xq16mYiwTpvy5vHJ4dZODT0bNvOt





eRwG4IBhIm5swrrr1Jf1iDGLENy4Gbhiklyyb+ddhGxS9M6HGHxhRIQoKrZYdk6H5yFsiLi2





fexTClZoHNVTyszCEkNUJCUSL6pqy0JSmhyerlb9zladNR/dPWNG44pMevOkm225IaxXm2Sz





s9tfHa9flf3jww998cXfeYYoPeCqcM4Z4BDyg5rim6gMujet4/7Bw2APETgK45jFOR9X42Yx





zp45wDgiU3/9pGmLbnWjMgU7U2zP3BAOLl3mbDWdwO0RaSqNXSfylzRsYBrQN6wPjucjdvqF





ffn67SG0v/3lW5cOZEg4Xp7tU6s7e9ahG7uyqaylK3vWZwPhFTVdWpPTzXqQDfqkzcRMfXn6





hQMACi12R7e1+fCHP/L0lb3fv7v54XdvPrB5FF3u1zfbaRFGatvyxs1bW1tTK0VTtpthRYSr





Vy+tjk6s90Xb6EAx5rauIJJyyjkjdtwWFuN4mtjHiOSMVVUWNcYBDsjWFkACvAECOgE5FB36





va0tBkUEQalI21vVSX86K2pj/NinnNLS355g12J+eVLctOvV2Thta+QkKXsvBlSjPMOCwAmp





EDfkJEas8YTsTHnzjUUxoU0XQxiqWRlyTlA2EoZgrc1jNNYOIapkU9qQ0tgPwoiaQeQt+0KI





KEXa3m4Xi+7u3WXKuZmUIW6mRd37QMt0+cn8yu/Sq7/3yGMfee7wg89f/8zV+8HRnDMzq6aq





keMh0n164EFqfA0dbyHFw+O5PESuytSZqAlEd08Wx6EDoENYXD8qJsVoyFMRdqZ6sI+21t2K





GkcUaUPZcAYx6th3kO5v/Ic/ohRz/trI/Df76wEmYQkxmV//Df35v3/7s8/mpPMosa7L0ldf





fePm7eN1PZvcWh4dLVY5prp2rqKmtNOi1QFeoCo7033T+U/96nEYBFBkfepw6+Pv+87/5Lue





+fi13b/UhtkdzK49BatFYzRtptd2AT28esUQnZ2dCVY7exfnsxKFbQ52ZOcShg1xskWCZhCk





8LZyKJw1GO+sm0nFStabEEKM0bsaVAICWECACrCALzG1cB5eYBlawPehjxg8XIVKTT4Z1jlr





H8YsPMEO4Eqwhy0LW1Ym9D2A2jZzV9RSbdBN4CdoPNoAx5wyAgEKBs5Ob++yPdubXdjb3Vbk





DIUwW1NWJTNDbEcpcw6qfQgh5LIoCl+UZUlMfR+89yCC5TCGwvlpU9WTElDk8uhOZxVnwzJm





XP3QTSK6u/E3//AJsvEbIxeLk9484L482Bj9Fji8RYY8JN7KwwKO7/qeK74sVZOJuZ5Nr777





ksbIVVUXRpIyW756ydV7RD5DxVSQKp4ttCkYiUhjCkyG3OTqk/uqSkzMX/eoUeDxK/gLH5v8





wHfZiwczR5P1Ij777POX7OQd+4dzMcdDd+t0s1qO8+l03tTiuKpN6d32pK1LU5Ru72JLZRy6





UM/o5Hpx3gShzP/kczc+9oPfB5qCq8cvPfred3wHQNjaQeb66ru7V95AOUHfx3Hce/QR72oU





gafb4AQvwDXM36GxR11BRGPKmx5iABXl7WbLqymsm8AX7Kw3nASSAQ/UAAEDMFmtrwN+sTpb





YV2iqFAaGGutgkekHuM4ICcWtsh5v9gBNGEzIB5jE1OPTEXpx1EKoMQWQz2KAQzAIVewBCpR





NCgT2Oh+Gia7F/ub61td7LOAnVhh1pw0DimSSFGcz0KT8+xpyhrGPI4jETGxarbW5hx96Ukk





gpqy8oX1wTetL52fbG2rItGGaTz90oW9p65zdoQHM7JERN5VD778JvaWlfaQiI6HBRxnQ5+h





R3eW7JxrPUHy6QZj5t02VBync80xxzVSx9YIGSCJCEyZtQdUBJhO+iS5mD32+EyVCfbr3ZX0





xRfwd37+9j/87bs3794NNnzwA+OPfdvjwthq6+l0em1ra9TFyep21LAaN50OUbJhXJhXY9Dl





qnvm0rV24ooKL9++DhRvipqcHz9sAeSjN4APLG+vZBOQA3AB06cAUz7yIZSPYHu7efQRaI/t





CayHBHANlMCXMbxE8wYhwQoR2FuwwRAx9qgKMFkoQCmkUosh9EPqgFNgAAjogaGpHwX6SdPU





qBSaMN7BnQr+bDhT6HLsWM04pLCJju3xsPjq4o0vvvzaK7ePw0hbclg623dDUVDAGHFSoCow





a7G7gTr4CeoWNQMFSo52fbxdz+Arb4ypTCVsRdmxIzJMpjDFbDYHkHO2Yu63t6qqZWGFWElJ





maWuazYwloyxMaXN2G/ixheGFN1mOTWtKfL88Vd1Mx1zBH9daJSZmev9x48fpAbwx5Li3B4S





Xty3hwUcnuRocxwE2kq5NyfysrOfUh83vZ0fes0kjL7XMRBxNKRg1E1OG0oRqmO/SdmZ17/U





HZ38tb/254VMTil9LbECgpDtTrtJklY4ve/9+tEPPT5tyndeuyxGNt3pI5cuHk7n1w4vNKbu





+/HVL26s26t4sry9qeAmxXZ/d3lp6xBrPdy5ElUBBVSc/tD3vRfY5f2ngNcmjzyBySH4EtAA





ATCAAPkXP/WZO6+++sXT1S/8yqeBfeAy0ABTwMBPkTJCRAywDCsIA0QBRgxgYVPEkJui9aYE





4MVrCoACCdgBKmAA1GArI59icYZFP6azvDaQu6szdBhXsUJ5ZXurVNOI267b7d2mqkS4jzhj





pKZyBHkj9RE0YAVYIE/QKBRgAlrMR6xqs3d2xhcef76wIooKBYOZTcGls4VmPpwfnm5OrfVF





ZYzJhvi8Wc5aKyLGGDFaNc5YoqxtPbGWbUXd0Nd14eayXg/G82Ran23WacPk2Bhdv/hoffkE





YHB/TgoAGd3y7jzha0FT+jpX5cFTX/f2+UFdfeD/s4X9/5c9FOD43u+/BqgmM59UUlbgRpFU





15zVQK4/91WA6HTxxos3b7xwg8al0REYEwYOIzKUorHSn93E9tzxXUb/0//uU0+8f6gq6APB





jpzKTKMhKczqw9d2n9za/o4PPXVzcXSlnb73qSdO7xxPfVPkhpJM7PbhpWtbk7q07n1Pvefi





ztRp6jQ4Q4u07ta948JYEUtDZ379078PTIDLQEZYAgIk4Ay4BDwCtD/9n/4XP/rRn/zU5vWn





93d+7Ee+GwAwAAboAQvUsFOkCGZUExiG9ygNygmMhVjEVLDLIZP4wlWAkDTACLSABQLgAR9w





nBCGHO/eibWrkMgb77moXF06M2urStyFdmtmdubSHtRPXmp2dsyFBo/PTNHyBIgTqQzEYw9Q





IAN+jZiw8miAuMGwWKfDd7wUM7bMhd3iUgI2Q/BospI1slXNHExlCsrZm2JST6vamTcdFgLA





zM5ZIKsm48SJ8d6CclVVGnkcsy3orOsLlmLCMasIJWWBSq7nj7/GOj+XGyKi1mVpLL/9YI77





d/yP0x8Pg/p4KMARQ2q9m82qyaSGGMASiEAppZEKEnvy+p3lYji8tH+B4nq5IJRI4HWvY1Yp





oFEJzYWLnIKURdysr125BIN3fls9GpsZAJSS1ZgyPvKd8u99/MPve+zwyuUL1c7e93/f9y45





XDk8eOqJfURerU8UkpEPJ2Uz6uJsUVn0y7WdlLN2vl1WNAnb8xkRcs5xzMLxhVdXSK8BN4AA





NkAD7ACPAhXAf/cT/8u/9a/+K//Vq3/9sXdcAXZBT7/w5V8GKmADRGAGXAE61AXqGjAIBI1g





ATawBsKot1CVfjpBCmSKcwkDnKdUGDAAAwcWFwx8y6XxqYvLNEZW9iKL04VlxL4DLFAB+8A4





4PVTnAICGIMrC10a8Ba2DRzAQAYM0DS4uEJcYThBd/3LT3G9bDE9MDsDjjucHo2n3nuFhhCg





MpVtg1lrmraZWLKVr0RgDTMjaSYjxovYwpQuE5QAI867pqmHsc8pU8Sk9U1VsYFzZtiomCwM





IrO5ubt9WQ7fsRIuz+OgXvNwsrrnufzJ0Y23nPlTcHyL2KLrnS1qU6UYAQGgalWJ2smt105C





1PWmv3vnJCzv8CM79fY+sIhHt9PRXUymoAjKnIMmjpY5kZCpNOcwhs1d1miSirJq2W7zv/3x





3R/90NX3fuTpJ//MR8at2XLZZcjefDuvNpe2L/z49394Pq2b0h2dLl+9+/pm6OLsycWyO9m4





D1265oZR+/Cxpz/y/oMPj0NIMQPQFP7RL/6l3/3SLWAArkLOeXGeRE8AfuJHfpiudZev7Hwn





rvz9v/33gK8+9tRHgQ4wwC4wAgM0wTSABxIcIwSgAArA3csJOajCOiADDMyACIR7gycYGIHM





sBPY0jtEJM5DGlbLTqxcml0SEwECCuBVoPV4coanAQn4CnAUchwQRpwBAAYgv3HSAhE4m8Ir





4uZk58mn+hK5wDahHBFPdalGUkoBAYAnBxRA2bq9yWgvVrtznh1Or25NZ9aSMUJExhhrrbWW





REMIm3Ezxqia23nlC1/PqS6cScgBmZIpnPG52tkQEee4fH5+88UiY8g5MzFMOt8B4xwd3zy0





8TBg4hvtW7+O49F3bTeFiIplrFebYkwoVMkQGepf7+NmHFNl6Il37JhJEV4/NVcrvX2XhXLb





KpIiI0WI5P4NK9Nxc3xjfTqenf2Vn/pYM9/6z3/uE3/wm0NI4WP/YoEuztS+83u/UxPipmva





Jm9WNespZ3ewJyEtzs7edXH/sy/dKK3rx4DSv3f3sPGnlw/b1cnx4wcXkzMXHr/01//mP1Ax





QK2q7Mef+YVXaqff8c6rQAeUD9yyGy/+wT+eUvfMhavP7H8cMH/uY38ZOAYY2AFeAFqgAV4D





7QAjkAALJLgSaJBuQxygAAMCUsACCgggwHlSdgXsAQtgAEQR1xgbP2OE42EBoUd2Huu1B6ZT





fxVYAC0wB6C4CcwJFywuAK/MZM5IBAa2gDOAXZuABTABzibYnswXQLK4CtwFuoysCqSsJGM/





elc0OAfNAqi9P0TqRLLHJJph5WweoQhi5FwhlGW5Tl3XxbHv1qvROunGrmptUJFxSOCcclFx





CGZyeGdz61Iy8ez4YtRTJhbTxVF3Lp/deqUlyvS1Qo630Rr09RVfD778lu+y/9YHhzdGs7rS





QtkyZe8Zaz6f2q21prtO4CRR4fOLx3ZaQ3OEyhjtdkOakGJSy2REOPbHfn7xM7/+P//4D/xA





0iyO/sq/+c//0YfuHL/2lSefvDqrJy/+0Vde+szzjz+578RStt4VOoYdV6QxGOVpWfbN/No0





rvpbPeGTR/2/5p7bbF9shLcvX/vlzz/75KWrP/PffLmMYrlQKEv1gX/uGd0cv/ddHwQ8cD8n





eXMAACAASURBVB6LPV+1K2AmObn9y9h/FNgFMrAEPNACJ/3i+eXmud39fwlogQTU96BzDDhg





CZnei4AOQAl4wDygZey9AMcCADACa0LpERrIK+swq9tN7u+ML2+7bWABGKC9d8kjYf+BO3DV





4ouMXWADnJ5/8rbcATIwAgxsIrLBFlAA8zXuRohkC9UwBkArtiNWDk8ACTBABTm/FEfbMh+a





oV/eNE6GcSgKzyoY1Rq7zgOzEdFxGJnZivHOb5Zj0lHEEkUxIBblgmgNLA8ejSc3XDguTFH2





G61qrMJbemT5Pke+vvDrzbcfqvrRh8BVobEsi9PliYKrthEY1ZTzQmOnxj5y+bLPeuWRC/2N





47hdoXV52IjjsU9QVR2HfglNEApxSDn+6m/8wz//Y39uWJ69eswUbJbZ+973yJVnHttup3vz





tpxduHBxJ3UDGFmBzLEPXJammqG26ovdrflsZ7bo0+7M/dSj+w7N8sbtaERHffko/bf/0+tu





U7F6QIccHruS/uuf+va/+NFvJwJwHkQgAD/67/800PzT3//5kXh15ya6U0ABB8yBFiBgXrR/





dnf/IvAcML93l8+jHg2wBnqgBADkexJDAQM4wAPze2EIAOt7HPFAbTEBqqv1dgWTOyrclDEF





9oDmgSv+1qcR42kgALvALrAFVEBxXjR3fmBggCUAYKyxw6Ccs7O2Lspp3ZSYOMyADiDAAhEw





QA3sADVUbWVIufK1F0cBRCRqq8orwRhDyvNpJWyHIWQvGqML2RLb5MKGVM9BQItb/d4H/+CJ





777+yAdfb594LUVHb8EGP+CVfF0d+v9bK/efZfvWBwcZChpjGrscpfSqSmSghmLmMN65/sal





i20unWucyZrHiPWYV1pc2tFhmfLaWQcRTWSshau+93t+EEY68GMXpD+7Y7pxvdzsFnU5a0df





70+mPgRW0aMlna3j3VMWE5XgGAO/cXJ6ut4g942nvWI696Vrqvl8zpv83/+jf3z985XX1prx





vI65TMXr+4LKPd6uf+yDOwABZ8AxsPnF/+yngcW8dMjp4L1Po7wCdMAauAu8CNwF/gh4HbgK





BABABBio7wU+e2ALWN8rD1VgH7gK7AE1MAIdML0HFAuMQATsecR0wAq45tBeqj/iwcfxFnDr





7S78A8W1UOAAKAF3D0979zBn7gVWLHAXmAF53a1rV48pirAHD+gBBvi8nxDw9yizAPIQhnGM





DExs6dXVphARkJIaZ+w49mAlgioBwga2yDyzIGKorieWznsFKG4a6rfguk082tkN/dD9sUHR





t5/o8xYv5v/pdfzPmH2Lg8PZC3U9nVRV4RyFkJzNOqiW4Bri46qfeOadNp+t82bMKStoNHy6





XqrF+myNESnkHMYIjmAyErqzza3bE7P1xT/4fN70i5PbabWuynJ449R5s7VtJZAIw5lREIah





P1lkpbEfY7p7MJ3kpC9dP86JY+ao49j1h4cXjsb4xs0tzQ1zzMoAVNPanfyFn/hu2LDpDW6/





hhywuQMcAycAgNsX3/nte7P293/rc8ASWAF3gCVggRWwBYzAKTC7+cr/itMvAR64CZwAzwOP





AyMw3sPE4+uvLQMLzAELlEB1T2uMwC1gDUTgksfl5eI3gV0ADs9smb170ZC32INL6/7PaAMM





D5z355kXoAB2gB2AFGPtbUKcmbJF67DtUQMWmAAesIABdgAAB4B2/QCVwld78wsT6yZFVYsw





YIWttdvb2wCIKQ6rOHZdTrUvNSPkNITx7MgHjKpZVUWcjgYq2xfouU8+U/CfsHn9177b2+uO





b/FO2W9xcAA4O1stNmdWXB/SF597hfIIhNUixDiinZS7tSp0HLmsMK25Kuy0mVjXbTZVPUlR





0427w2mXMiuMypRyGGK3Xp6WdvflN05uvvri6e1bXkidbPooznXdaTrdKGAjWNhb6yr5d/6D





nz1b86YfdIhWlIX7IZRls93Wt07P1uvV4++/HcrbMZIqcs6SpTyY/c1f+xxAVaVgwWIBVsAD





d4A1MACr6WPvCKxYvgasgQ2g96MGAICItNq/+gHMnkH/h/cCHNvAGwABl4AT4GICXvnK//DA





BSOg+WK+/rvDCeCAFlBgGwgJLwDXAZ60TwL3N20/BC726SXg7p90K/K9YAqAI6ACPDABWqC+





vxQJ9ZynBdi/2SmzC8yB7Xshnvu2AV4HukTJEEhZO4VKYYqJqydSmkjW2mHchDAWzjVu6pxz





CgJiTAAVU0IwTG86VqN2iGUIUeME5xtBPXhR3s43+UZgPDx+y7c+OFIaU9S2nEgly2Wnq04x





iAuf/D9fYE2AatflqsC8Vivd3R6Djim4VcxKyDnVpUfUk1tdF6GR+kj9cJZ17fG//fIfrU7S





wcW9vFqWTEUc/y/23jzGsuw+D/vOdre3v1f1aumu6n16pmeGnCE53ER6KFKSEQpI5CRiYiEr





5ABJENgyHMgKrABCFgcIIscGDDuJ4RiBoygOIktmIlmSSYoiKZKaYXPWnum1urprffu79931





rPmjpnp6VoqBTdpFfyi8uu/WvfXeu+fc7/2W7/x+00ka1FvGo5Y5TRyrhbYVgQX/1S9+4dr1





GyQMrQDhgSzz0PepNWFUgzarjW5HLvtRSZimzDnKGWXM43/+z3wK7RX0ThcyRztCkiAdQBdA





DhlDa1SLT3zyEzAlkABHmeY2QAHzRvCCHfkFJaoCSIF14GgjR/kS0AcUw80rl/4UkB1fMAbg





MVr7uG+PkrJzjMd6DNQZNoEI2ANSgD5kZZCAPfkQlbwrUmBw7L+oY5PhyOOovzXHLAhEiFWA





A6sAO144w4DyoX94FChxQegTxj3OALLaWodPoiDcWF3fXF63WVXzwnanuSgqhTfWszyxdIoG





Lgi5YKFWgYMGQECMYV5zZ7y9vPdKh0I97I+84/E92OEtAZATjpNPHIHv1T1vUs7jKrt0dsVp





B6sCP1xZ46hyXVbIJUmzfG9gxpl1Tjnln1snSzV4PrVERBGYIIz4IjIqG46mjLfWVjr/w6++





YP1mLYx0IePhoJjMh/s7USuolCI1zwmBWqitQ1C3IphOZl6tM5geEKXmqeo2msJRymiWl8S5





tV7jwtJqb9mW0pbSWlnGNqMfXfvCBy8BdX14syyKg5deG4/GUAa5RVbC85Hm0AowaJ890qfD





HAALwAEhIFBlKHPAw/Q6fIJ4CBy1pLPAEoIucAcwwDKwAoyBw+Os7R4wAxbACLhdg//SVgKk





QAPoA11gE6i/NSAKoPtwp6J3QABrR6x0LBKhQOMdhzGgB5wBesDycVzjCBII3npwq0QRBWEQ





+UzwQHCg1Sb9RtBo8JUsz3vNLqeUM8KoN4kLT7C0LAcq96lPYBxzMBLujUZN3dX87gtP6oN+





MoreizXwjnX0bwV5x8aJxcknDquM1Foad2tvZ57P8yp1gLPs6ac2bEi4L8o0o8IPajXGXBAy





ai0h1lJhnXWBp4w2RtPIIy6jjOWmnM7nv/GP/sjS7PUXd3yPVaWBX2e6qgee3/BFpw6PK88D





GAFTuvq13/ydxWIRCrK5dnpvmlBnG2EtXiS39nZLpZQ1l89vNITfj1y9UxFaWe6CKPoP/vQn





j6TZvNa8dX3rzvYW5QzEwQKKIMlBGcyRRqvAkUSCNY+WpQEZTAruw2rM76LdhSMw1c7da5gN





kMcoR8dhhSkwBSpgCW+kNubAdJTfsRgcjHZ2ysUsWXzuUg8IAAkcAAGweLdEvv/WgOg7//oA





ISCOl9K9D7z3/g8ADLAfYLWLXiuod1in0hXggI5BOct3CSGFrIzSMIRSF/pwjjPGDvKpLI11





MIUmjOGYFBh1px6dkkb2hqz43VjjAd6VGE4+WzyEk67joJIJ4fn+YBavr/Z8LyicCgopAkZV





RZiwHvE8q/KEiQ4llMmKN9tgHMInxFmteW5cNwKpmSqRBTm9vLJz7ebBsLI2UrpMSkYTGYSs





4QUEVuWpbYbC1WGNdQxwwhM/92/+1He+/FW/3oCIuO9FUTBPFrVWfTqZPHFp8/bWKA6Kdqd5





aeNi1NrlLVEPwzvz4sLerdeLGzrjr9688ZGPfNhqNxyPWlHAljtQQFahHiAIYQwYYA3okcbB





wWSwBHAoEh1XMzNbtn3I2Hksk9kcpc0W3ZrBgoIJ+AQ6h39EHEcEZOb5aH6Y1Nvdte4GmIfg





KFAKYAI03s1MeAD2/YxNHciP1+l9vzgSqq0CcYi24yZzVUe0gNC5ux75iBfdGM1jR6wlptI6





nldLK8HBzpiHoiqLotBGq53vPkLIG3ETQkh6uLzIDtcvp/euet9bh/He3si/1HH8Cw9P9Lur





NOJ+lmS10EuSZG8ytKVbjA5dFjtl7Dwb3j20AeWtBq3XWL1O6pG11gLOOgcGa+7MZor4hnDH





uCn1ZG9YGFoslGc14yCc7h+MIuJczeec0rIkZWlboXDCWWMbgSnlYn/qsVqt1pgODwlltSgI





qB/HSRjV4oUsi2p3e3DlytnPPHH5iVMXqkF++9adx4X73T/8+v/6D//w137nt8+cuoCkyJO5





VvpwPEAegysQgGqYCo4ACpRCjyf71169+iXMDkAU5pPf+u1v80AEEYE70Pns3uzQ6SZKX6Xk





leuv3Z/u7M/vTea7sipgHdwhYF+49h1lxu3osUvnnw27G6Usb9x+HciAKZACZ/6pfq16QPv/





F2vgWJZGjyQkEZYAm5UZUBFyFtiKqyll1EjFqKiKSiszm5aeFxQxspkXeXWXdohWb0mOsIoT





X7NpbW3v/c2NN47/XpEMQki9dmKXyZ5wi6PR1pqZWq1GmCuc1koVSlPCdVEpI33h1eoNGgU2





9Ai4I4RzQjiDBZhx8wXJcy9oCEqdLGSaT/dGsLTZbb16f+JT39kyiMKte1tPRedMr6Yjw6YJ





zRdsOpXtZSZLH8Ry4rGwt7oMWc3Gs9PrK5OXr4u6aDkOw4RHW836lctnPSG6Z08Xyezysz/Z





bvmPnOrfHsdtr37lkdOgLEuSg739Rru5PzzIyvkjH3wGOgPtY3KIRhdMQOZKFb31c731Tahy





cP3VlZVTP/Mzn8zy7M6tvdpSo8Po1u5U6CqNs1KxlVYrnau4mDeX2otOsv/SK6eWVwHDAyOg





AQkkQBiEjcsXT5fZNKj1oFKIBGgBFVA9pBP9weM+0AIACGAfwLi6V/ODSByJ3yYAafiNHTUS





jEttYF2t7hNIw4kf6OlhuPfqppQJI5A6x0MUUOU0iEhvc5bsrj4c1HDHVQ7eeP3vgzxPrPty





ki0OAJ1u63A8XpSZMcoaXSk5TZJ5kRzEs6IyRrtaJwQjxG9RTqnVThk4Yq1GUQDWNP1Oy3NZ





rA4PXn7uVaptmqV3doeWeY4awtqZnHV6De/Cqmn6UjvTDCzlZhaTLKOekIUEFbzbDMLIMdrt





952VnXqUJum9/azdrk2nc+ExbcTapQ9SlB974uKzTz/VrYfG0StnT42G+3/43Hevvnxzf3BY





q0VUqYiRR848Uu4f5pWE0rAGjQDcg8R8OILNkaYQ0crjj7x86xXIOB1s93o1M40DErVto1dv





EKIKMry7mE3S6XKzbSdVuZe3/F69cWZj/cNMNV6+ubvIRtu3buzduSrLQwsV1PzMxTeyLWBc





xN89Tvr+ELF5TBwA+jEODCM+KNAEciDN5eHucBDPZwqmlGVYC6qioJT4fiCEGG93KTIGZ53E





W80KgUatViMkeKc14dzRjwNATi4d/MlxwoljOk3AmKUglMJRzkRp5GwRV1oSCjjrdIaIG5eb





bGEqCaOdNsQ5MM9yTog/HS6Gg/n/9qUvP/nIIz51itCoVvM0rMPapbZSnLpAbw2QSlkou1CO





cmW0i6eoDA994zQcDRtNafSNrS00ahU1FzfXs3Sa5nmr26txL1mkX/+93+ORZwlLxwcbrc6t





u/PXr+/2OitPnH30/Prp8SyJ2h2fs6XWqet3tjzff/76K0jjl27uYDJHGYOxbqsFKmRcoEgx





TwjYvXsHpaIbl55qrffH2q2fWZqz1G/ypzY3y9kgltl0Nltf6y716kutxtXn1V/9Gwf3RwbC





3hi+hIaZ181r8+G98eH2aGcej11R3Zy+GraesdW9JL4H4DiD80NEDpQtRF0eMbQAAuRxlUXe





Bgsp40xQRkGcc5wzrWiep7qgrvDg3kV8QQjRZp6n8+XV8F2KUb8VPzp6jffCCXdVYB0hqJSS





TDiYaTIPeLDS9UuZ75f2sfU10vSc1azUxGoD6jhMVhBQzSptSVXptu/Fs8UXfuzHUaek5Otr





y//4j647SigXj55a0ZJ5tda1mztXzFqt2wQCm829gFhjLbGuKj3fN1kWRg2jqUr17WuvM1rr





1fxOs1HkxWJR+ETcvnH78cuP3Lw9PrVUa9Zbfq3+9BMtP/QZNQeDWTbJHt08n80Ww+motdxq





1ELa7v3Y08+gkI+eOQXmMJuiv8yCph2OvVMbgMkPx1Xh3R7N5pPiWy/fOts/3W22l88vnSJm





nIwX8D/4wY2v3bz7mUeerp+6dHD3lb35+MlL3o890661ngLY7HDeWX1cql3Lsjtbt1Z7vXaz





UfMTRRXcNeovh1IuZi80OpcAAAkwBc7+MEY3ArYALrABFMAK0G/5Bri3WKQGrqhKCqK1ogxF





VtTrNVdaQt2D4OVDCnFCCRhanueVKbXGdy57jxf9UaeMI5xw4igqaZxxgsMQbaGoFsJKRRyp





It93lUJFHZEuNU74TMBSQdPCNRrzOOstrfh8odO4FwSV5eODOI2TjKVff24siHNKf/gRMpnN





ltqdaUWvvr63uVoy4npLLWY1d87KglpqjaWe74i+eOVKFuc6HjLPCcY9amtBg2hsntuYTMaT





0Sz0Qk1oUmrLdLO1XJZlPSJKsZLQ/cmUO7d57oIxZT0UCAkGGQj8wMNsBmkACkso40AT03vT





eZalRdQQmZpY7Q1V6sTqK7999eyj6/NhpTZGuSlUKhe5zm+/Uq9H5lBauFprGfCRzlS1AJTO





s/3hncf667R1brjzkoh4aTT6p6B2BDeisQpkQB1oAk1gBHDAP86//GBggfPANmCADWAAJICZ





LQZGO21NpnKtNSHMaE0oWSQx48roJQDOubcLNCAUGVUmcnJhSIXj6MbbMiQPy0bf552d+NzK





yXZVHCGEOFYWlbFWK6uJTfLUwljNGKWVVTqp3Ly0kW+dNTwkDqrug/NWu11BV4YRVxcNaojr





rfZps/vq9YKh0s4YyCInsObw8HAWp8oiCgNBsHVny3qeg0VaGkaVx2CUtbpcpJPD/cli3uhH





pSpBudJVwJCpBeVeKk2t3Uli4zwW1r37u3cYF5WiysgzqyuNVjPW8vadu1raLCvM3fvc8+AL





lBKcotsEIWhsol3D7PbsYP/06ulnP/rEE2dPX7l4zpBFZuaTyXV/lRVV+fgHHqGLteRAXWie





ns6KfrvdaPrn+qdPnblkFnMz3kI97J+5BIiodfnipWdpawNo9jc+0+l9aqP/cWT3IZ5E+Biw





Bqw8dKmPhOE/SNYAQIG9QbYDSOAuYKSaDLPdSZlpmID7RjtjnRCcc19r5/sNxoO3NpZ/kzuc





U2sb9eVGAODShxcPSVKceyuFPGyn/GA+5z+HOMnE4UAdnHWGEqato4LqSnmcJkXRjJpcwYEy





7hMJqiyrh1Q5Yqxn3WJvSAT1PE6Jtc7c2y0X6fz+3rC/3L52447VFgTOsccee8xnLAy8WsA8





IeazRWOpc/nRxzVxCk4WCSsUIcSVCoQJQv1mW7dqNArnSWodtJJFWbhCfugDH+msrDqjvKg1





Gad3tg+9sPbajVuz2XC53cqyvN9ufvSRC716MBvP8rRkUc1kCwAQ3nyaItWwFpiDbaJzNlpq





oSFKld+4f3uz1bhybl1UtJioU91HHn38c6U9/cRTH/nQ5Q8/s3quHfn7w1k50jURqL1bTBfM





ryOfAxTFc4AFakAJ3DuOZdRQ+wAwBQ7esXLkBz62kEASu71SlXuTG2l1ME1ujrNZki3idJHk





aS5LZ5w2xBitVEEcydLCJL2j84/NjTctCEfYaIdpWxRF6cQITLzPy78/Y/woEMpJdlUInFaO





MkopqZSkjlCPM+4v8nJ/sH9p/awxVlclF0Rby2FAiBO+jmf1wDe6slWls1zwQJus3zt99+6u





L7TntbIsg4MDPTw89DxPlnm/3QyiRrMdZdJovWhx6pyhIFJQEeeuEZmi/K1Xrv/e7TvCC361





f64dBcKjK/0VayAlxpPD+WD/7JNPXbt987GLj41Gh4yRxx+/uPX6a35TVOksCgmE6CwtU2La





/SWYgtVrMs2pCJrNHrhAWoJ6CMaA9Fd+HOXzlayeevTy1p3Xdg6zbq3vDJuPDpc767s3nut+





5BPtWttxd6oZoNY9XgaSA0fC0/PAaDgc989sA+eBjXdcV++HFNF4gPhojV8lk/3ZAYhL5ql0





TVhIosq8LJVeZBlguRDcqmSRUUI9EZFA3n6ev81JeeBPWDqWsiuPqgEZYyEF51q/hR8fYoT3





NDrIDzti/IPBSbY4QOALJjjRSpeyEoEHuOFs6gjJtdqaDThomatBXBFQKEs8oCo549pk0GC+





H3CfCz+qB9t3d3u95v5uSlzhQEDgIb9zd7sCXVk7vdoMWoHzOIg1TAhpjg5x3FmaFXKROC1v





vbJ9/1Vz4+8f/Owv/k9JoYt0MRxMGHfj+ajRClVVWeuWWsuT8Xipv7K+1JtOpiKoNThp1r2i





kjsHg3rUNMYAzCwqMO51OqU003nsrEVZ6d09xAniDMOvIDjVunheG/XIk8+cWTnNiVwsknla





OVVmVVEmh82OL2htOsxH954/2PpjoA80j29IAMv9M58Hzr/HZa2/x/4fGFpAXFSL1/e2DFHS





VERgmmZpTEptv7OH+7nXa67BIfS4F3ph5Ec1j7B0vM902abOvi36cPSU2eVmf+pU0Awba63V





yx9o58W7KOK/t5/y1v312tP/VD7zP284ycTRXiaEwhhCHAVQydIa7XnCGJtLpTPLGAlCsdRq





QlnneybXepGrrFCE6dKUi9wxoo2kgl04t5IkMaXkcFRySpWR/+rnP+YH0cHu4SyZHSxkqbSy





hhHqhTURBqaoHBeOU9eqWyLgievbE/vS2JGSZCTPyovnzkgqR9MkzyUjor+0lGfpxvpqsx7y





bF4qdW8w7vROTZKkv7JaFWW71faiehC2oDGZy92tASpXr0ew+nAwTLN8Ei+m2zsoFIIQyRZG





AxibjOXK+kqn09k8tfzZT36Q2PHFjV67RkGlv9KbJunymQ+vnf84sAdsA48Bj/2wB+19sAsc





AvtAAnjKSu2KqlSLJHPOKbfIvfmtbZu+7PWCmrIjYlsv/oF381VOqLVWUV41u7aShXXv4oM4





55zVj32sirN5HCedqN7d2CE0Oo5t/EltiB8BH+UNnGRXRXgJsRxUa+kYIZWxBM5j8BlZ6ays





+81KalH3HXHWgpaaGWcJqdI0WlqqKhrWGajVpVlaX5HTzIsi1mZ+0NRaWpArFxt37gwvntvw





PNrpNFv1iFHAOWKks5LyyDnq0tj01lgUOEqG+5OK+CD0yuOb/VawNxr3W63B4WC5fWE8mrQ6





ywBUmc9GhysrK+CiU+9Mk7jebt/cutdpNkeT6fLychgEqpJE0KJyzz/3WrdTX1vrKmN2Dg7j





vFztLBFMw3qUpnG70/n9P3ix0VuDqhjzOWc79+9fuXxhkUZeu4lFarJBYRY42MVaZ3D3+sq5





f+uHPVwP46ieyFGpZAC3AXtcE8ACU8BRnzdrkfOoc1qIAIT8k19rsqhM18Y169+65t+77Yha





zSZpoxvX64TTNg9GH3xWXv1Km77btHdO7+8fMCoIyAvbt2GWGS31UYjUvZmFwcNGx1ufHu/7





UfBUTjRxODgllWYWhFhnYQ2xcII4MG7AGE9l5RVMGVkPPFVKKiillHkRKaQyY9BGo1mTReEG





0/F4Plvkv/7lOzCKsMZ/+5//9N79u77gzUZ4uH+4ubYShJ7ROgh8EG1gKBMWpBwM6601iKaB





jiPjMQpNAq86GNtet2EV+uc34rxsEDsaHTjCah7zgghUlIVO05IxllXwmRjP4/5SbzSZBr5X





VfL1GzdWT610+y1KqVIqaDZ5knVoEEW21Vv++vMv9Psrh/Gg1V1bJOnpzVOHu/vEr9uQf+fa





FqV4PGwukqp/uZW8sMCVK0C1cu5zP+yxehvy4yZyR1gGmsflP0g52586ee9ge2mpobWixLmi





jC23zQq89Mf8jq7tXPfhtDYJMcTzG9bOhKccGkmZnnuC3vru0pHC6w2fhThimaOhR5rWFdqY





oqzdfdkKJt4zxnHMGe//MY4aAJ9InGRXhTLKOSOOHg24NVZrU1VKGg24tMwLbVNjJKGF0do5





TgU11hcMMDWPWWlNVeWlzis1zbOZFLYggKdsarJpWerzp/r1hrh8+XwQMClllhWF1OAeM86F





3FBPeKEpUqNyEHN2vU+IUFBPPv4IaFmklZaSF6oWBR7D5YsXZZkZYwVxt27fTRaZEDwvpLOZ





JpT5dQsyiZPhZAxGOksroySJkwmlkjD26muvlzqrpJxM5LeuXq2Uun7r9t17h1o5B3o4mkzS





Ioln04P9RbpQrPGV51+f5WS0fZsLO9m5CXzgHXUufuhYemg7PwpqACNgBiRlQCbJsLVcX6hc





wwjCv1tk37o1q9qx8yEzNrxbg8mJ1dSCUjlL7yrNosDaTEEyEg5XLu8AzME454hxVDhNda03





yIvaem8JvkkH64HvK/memaN/qTo/ycRBAK2dNUdF66kxTmlqrCy0rHRlKZGVLKSapHNnLCEs





n8+tUrDOWChFKHXE6Ho9iOcZ4/h/v3TdOWOs+emf/LShfL3fWr2wGnBeizytldZSysoPg+3d





fVnBUO4YqPCoq5R1RhZ9znRZMdI4tdJ0IIHvGRi/FmojR9PZaDyczBbDyXRRVPd2RYCeZgAA





IABJREFUdk51Wh7hyqLTXiHgIfPSiq6trZy7tLG0sTmLs9NrG0vdZce8b734umO+LJlSTgTC





9/35bPrZZz86L/Kd0XBvNLz66qvjNI7aTS/yS6VefPkFQskkMbXw0pUrz+wPE2RXj61rA4yP





3YR/FnhnXdI/CY4KFAeAmIwPbuzcvHFva2l1qawKIQQFbUbRh/pLVy6HrG0rybCKucydqzvL





NFHSqp0XL776bb2167c7y60l5vl+1JpqU8Jw55whKKRsrCQrlw7qtWSQbe+9dGZ6kFUxY0K9





+SbeIyZ69Oy9868nll9OMnEYY6Q2FMxaa60lR21ELKOUT/OUMmo5lapyBouysko5xqxCmlfa





ECb1YLawhmijCauGcwNjnHPWuo9+oH739vX+eu/5b39HScA5yixjtN5sZGnSqbeYoBLU1mqm





0pb7lBBp7BOPr4MTh+Qv/pf/8M6wXO5t1FsNSsA5W+33k2wR1bx5vDCVWlrqSC0rnRE1zxaJ





g5kre/rchnOkquhkOGl3e1evvnjt5p2Dw1Gr3TGKMrB2p97tdTzBP/6hi9Ph4M/8Kz9e5CMv





ol4tqKw8GA9fv3ln6+69WuitLvc/+clP0rA2nSbnzj4Jv39w65vl4VcBBgyBu9/rulbA+Psf





DQkwIPv+6SMEHJDE8x1NXJ4nBHY0HVlnszxL4mS+yOugTwf+p56+oPjic09Fl58cnrsyDLsJ





MQGFsGnTjc/cvw3lYkJFkeec+0Q4EOWcc9ZcenrcOX/H5w09CxalV1WM8Qbn1FmON1fHvh3H





ZPF+hTm+z0/6LxJOMnHAEeOcrJSzjnBHCBUepYxWRVEovTsfQBnrnGdpXlRQqrAuNWpaISvK





xJmaH1Sy0haNWuf+/hCEeZxxTvYG+3lZHe5Nm43+ze37BJIBxrqw3U0XZd0XkhNiJbMKTNl6





k1AnRP1rr76gatZq30vpc6+N6m2+urrW7/d933vt+o3pNG42O8L3X7hxg4ve3uHcj/zB4YDC





SkUyYxrLrfaplbDV6K12L59ZefIDT3zms8+uLi93oiCJp5S56Wy0t3//7JnlxtL6169ef33r





5r/zsz/z6acfPXvB85rZjcM7YZO3Ws1+p3Owd/+rX/0dThtLS+civ55PsqXehsfrB3e+hGwB





nAMAbL3bBT2qG+oDS8DsqAflnxgUUMB9YGCzV9/3yJfesWcBVI7ReRyfWzvdb3WMhIaE5xmg





1HI2j5PSlPvbV85HNRJfvhSuPu78XuUssY5Ql0maqDGZJPOylJ7vWSd7a7l2hXOuslaTirnG





nddJXgVMclSUavVOvniniONHGSeZOJwjMMRaShyFZYxDMIA65wgIHSULR0xZqBSqci61Rhu9





sPpgPr4/GTvuEeMM2Hg8HownF873COHWWjjyj3//pW57eZEk8+n0bL+3KGTFgrQy2zdu1zi1





wmoaEAKnqPNC7TWdppWVgp/+yNNrYCUh7stfvbmw2Dsc7+4ddFeWN88/YoH9wYEhyHNFhY0r





DO6PTl+6UGu2Oq3Wpz/+FFyhi1Qv5sgrv1G7dGEzbLIgCLhgveWOserixbNPfvrTzf7KSy9e





b/bqw/nsH/zmF8dx3Ai8ZserNYOt8e724P5Cz7tdnxL5zav/T215ky49GvFSdNesH61d+AlX





cmAXwLGIowIMsH2sFq0DK0ff/0D4kO1Qvuv1B3BcjhjHLVr6yXw/UbkpXn7vU942J1OgAORL





t17q9hrtZjcvCkLg84gRyhyM0VVVzdJypdnY7JaZxbxQX/sWGe0KRwtrtXWEIWJ2bXwgpKyy





LBPcszw2ioLYlXM59/TOjeXVc8ngbiMbday11lnn3JGh+oA+HpKcv+X9vU+9nxO8XOUkZ1VA





nDGGcqqUZT6lhFoLWOssIaBlVQyymBrMyqwVNiLe87g4yJOqVH5UG03GoV/LZdZst/LcbrZ9





Y3ZAHRzG0zAvZ0z0Gs3IE5wENZlXzagmyswLmSQ0jHzGjJ3O6OlVpgsIPj7c+ZU/92e/fXf3





pa//pqNOIPg/f+O7//rnnzJVubV94EMMRvvKelG9/tGPPT4cTBrNqNtfOzzcLZaWiNH3t7cD





6loBL4qKe6pVIwi5KXS9XQuC4MITfZXF4vQGwJ//0h8sKnnm1Eq3WfPggjCq0bO6Fezs37ZU





u4je2b3rOay2uhQmi7drrbNodd3O67wVodpyYg2FIOF95AeIjmqa14FVYAEYYAsQwCrgAwcA





P25n23rvMTiqMyiBIdACJqwmAqfTasLSq/X6OaD7jlNOHW9Uxx1bDOCfXlnjntje315kadAO





tHbGaWed53kO1GpXVZmpAtGWr32t5vYptHTWHhkO1mqDyorQmBljNM3j6dajjAmNtHXq1mvf





WVo+s5NM2qaqje4KY+Mj1njw+LD18WD7nYTyNpxg1sAJtzispQwAhCAMVCtrnbUWhLq8yplj





8zIvYagjozy+ORkphzJX1NG4KmeqyMtM+Iw6u8jn3UZNeAIOBKTgZqnFwGobZ06xWmAqaZXS





umi2myTwKCPOGWuZddqKZWMLa1Um/Xpd/O2/9X8fRfKdc//ot3f+w7/49+zq47V+x3SWnvnU





J3rdthD15ZUVrVwt7C0WmR+EMHowm90vZl/95otVIQ03ZT4zuUSWsVw2mlFnvb+3ffdwMNh/





6eXDl1/1G93pfF7Mp6fX2h958vLGCu/1/A3ROt1nobCtyLfCQfj39wdVqa7+8TdQ3d+/EStX





gRuIBXUjEi4Dm/DWgDbcELh/bFAc5UcjIAZ8YBngx61np+92+eOHtj3gNBACQU1EKWMVldNs





ZrDn1CvvOOuISgpgB9hS6T5QAPDr0f69HaU190QU+EWlYYwjR3EnDWoJg0JysG1HeyBWEsKO





WcNaa5TKL56hoBa8fXD9onKVlJXwJ7e/e96VHQi+/3of/oSSlByvanvAGm9Mp4co47023mUS





nlBZx0kmjlpYByGEOkrYG91zKKWUwBFppAj4Isv3xkNJtXJaGj3JFmlWaeqUkmmWc04peKlU





p1Z3zHOmAAEI2k0uCOcuixdxschCzwY+ET6MgrGWWKulckVGNh6BkZTAGHP+1OqXn7uRhYHz





tLPOAYSnVjX/3L/3V/+jX/6Nv/SLv/Ov/fz/3jvdiZhabrQvXzz/2rW70+lMKbu3s9eoNchM





feyDj+o8tYUhlKVyAWN0IjEZw+fGgXt+vd4MwoBY/NQnn/nAY2dZv781vud59bzIlUSn2WW+





NcYyQ69v3fnQR54JBDu11Lz14lUe8oPhCI5jvECrC3cL7j6YQ7YD0gCi46atR26LAiyQAMmx





LXDUKXIEjIB77zsgHFi2Tn/xD3S/9inqzbcOd18/uDdYfPu4higeMl5SgAOi0NpUBaDTXHbb





nbwoms3mIs2UrirjAGRFxjxhnZmnCxH43/kGJVZao61+8+a31lIW3ZvXMisO7rTl1NOS+TX2





xCfEpacykCFKwUzlVGQsrMVbDQ33bgTxvSnj6LCTKuQ4ycThjHbOEcKU0kppY4zR1hhinIOD





tZYATHBjDAEqJQeLiXLIVJkW1Xya3VocHOQTwsnufBzPE2cpADiXJ7GkHmWsSBLqe/ksKeVC





59aEQMgtJYZTFbRLV4epjDEEQsns9Zs3wsTZuqaBIpYQExJKYKk+rB57gjPjl8bNZPLKzf2V





1c4nPnPZ98VgcNhoNNNkIbxgmKo7w+yFWwdbB/k4dvPJYqEqGS6ls4XMy8PB+ObNm9vb25y5





G7fv1DfXb7368gceuzRNZovUjKeDZlS7sHlqUIwn83gwHWVSZ2lpIbr9hilGFTyZG/Q3J9de





x3yIagqZohhjcht6DGggBPLjFvYBkADxG5NndueoEhdgAQFMgcWxY/IAD4IgISUf+vwno+vb





14jsXFq9EHnBSiM4rmD6cKykAMKt/dfuxHvfvv0KkLh5vLZ6WXCvqCrruDKK+8RaSymlvlDW





OILRPFOyAReBcuOqo9v/KGBBtb33VXfrj9fy3dDogqDtLy/yyjmaNFvtnZdPgVDnKmute2uA





wx1LxY6ZAg/vfPCnB4UF34Ysf2es9yTgJBOHtY4zarQhoMRRTpjTsNYCxFonrVRKWUeMdXCu





IpIybqyUUBraMjuOZ0maJllitfnG8/e0pXAAdSb3B/f3oqjmHNm/t0OdYcynHid+QKiAtnRR





EiEEDh20A7UoicHP/+xnP/OZy91eWPvAkm67FAnl5X/xV37yN//6z+xvHVJi105dDv3WznD0





R9+5s3N/0ltZanTaB6OxMkrlKvAIwnCl16isrbUbzcce73z4o7ky6Sg5HIymcX75sSfPnD4/





GU1KbcF8YyvR2Dh16qylRHidNHdGl6dWwEMZBuTa9dc63dZsvD0dTT1f5Mlwd/fe1S/9fm+9





AyrKOIv3d8p8UaUJuA/kwAHMATAAKrzRZb6NYgdQaHDAAdoevAbYhzTX0+MCv6OH8i8T2O/2





o6cePfvoqZULqkjOtJeBs0AKpMcVzzNgDnAgl9rEsmj49W++/Py5U6en6Y4ftiolsyKngnNm





LUdFdDKLQanng9c8r3nfiAlRDo4fUcDRQ2VKkzc3lgqlKouwlIfJwAx23TzOFmOPGfuAKY4c





Wmvf4aG8YWU8bGg8HO94y/6jU39wc/0HjpMcHFVH9e8dsdZYR622IJQcNXV2REtDQRiIttbj





TBkdl5Uwbq7KQDHDnFWYZbFSihbkua1Rc7lWjCqLjrPqzMXz40nSa0dh6LPlpokXmlZchyYP





qaMQQqcTxpqOOwIBU0G0iOP/2b/xIfJzz/6d3/32du3+F37mxz7+5BW699oXv/KdgyFhEfmF





v/S//LX//j+e7o6MZ7f29juzORFMVVWr082qbKXe6fkJ8drD67ee+27cuT8+s9wP6jaNF/V6





y6aJtdpCBaHv1/27r13rNtqAf+21W9N41mp3puOZH7TaAuV6Xg4xS+bd/kcWE91fbY5Hu5sb





p9Ms/dDj58EtBNcH+1GzIdoNwGK8i2YD3joYgQZ4DnSBBeABPWiD2QTLDSwmdLULzIAZYIEu





EB/3YakADsSAAyJYgFwHoUYmyhgR1YA5EAIV0AEqQBX5Vhh1Ab25uj68cW115XTv4nkG8fKL





z+lIZ1b6jbDth5XMKSHMkcooBlYWiIvdT/9ULwzXv/gPdqNGN92p3nRVHLVscbijKQmUJYHH





ZNGc7I/Xn6gzz1TZgzDqA0PjzQCHc865N9Tjb9357q7KQ1bJieWOk1zgbPNcLa+kcYRSMO6s





hSOUB0ZrQj14hPnCc8rAo8Ij1FEQoqXVHrizhjgisd5vhQh9Gt7fnlV9dvePDQJ+7lL33/3c





pa+9cPPRUxtne/XaSkil9EPBA4+KiAchpYQysDAknAhQwhz3zlg15HoBTVnU4rUaJ47qNLmx





/ZUbo7/79+9Z5CBMmsXf/Bu/cKEXDUYHcVmp0ngh4wZLvR6l6HWaTa6WGh5p94oUcTIyRs3m





848/+4nx3l6SL+qBlxfWUX1neOuzn/qJb3/n+Wa7G8dxURZMkN1kMIqHicpV7vX4ygcffSbM





TLMtHj1/inveeDSK83xjfdXjHO0uBkM0a9AStRB+E6NDLJ8BLJyGLcGOOtTvAwZpCerDCgQ+





rEGeoL2GakeXknOGWh1owAK0Dkxh83uHs2a9BS6TIo/TshU2i3jRbNfXl68AIVAA5oUb31jr





rUZ+4GAFRRR6uayk0lBkJmdJGZdalcwZUzHDyqIUvl9JWejMKcJkV0Z7Z849/c3bbuvX95nj





xhhrrTHGGFDCpFLGZtYQrTU8Vj+zc/Bq31W51lprbYw52jhikKNzrbUAVao8MkaOd77x2xjj





3h0WcGn2wg/7PvhngpNscQCUEWKJAyHEUQIL55w96u9HtLOoJLfMMSMgOKEVrBGWGmrgDJUw





fBHnohF4jF54pP+735gTalxlx/OxNf0Gc2mcBhttqgzh1DHumAfGNTSjgnDinHEOjnDnuHUp





lbGplLYu1BJJXAaWzKqtUbXcrjmirdWUMI82/sIv/J1P/cT6L/2Fn+d7O0xTpeUkSYeTWa/V





psLfGcfD+SIaF9YxuNwYf+PMpvNJEAUhp/vTwb17ozOPLZGQfe3r30pkvjserZ5aE4pqJUtN





/CCsMzvN1WA+/dbVqx9Y27y/nzTDequhl/pnukVZLRZo15HEaDcRL9BuwQ+gMngOiIEGiAP1





gDEqB/9RYAsshylgJByBF8HrID8YplNprCxkr1l4ZGJ9Ti0Lm8sgolMLb9y/JSIWRLzVqs/n





E1aj8zJJtv/40bNPQVdxuiiUSVW1Ozg4e26zkGVSLe4OBi0RSlkRzqxwhDNYQyl3pfNF6AS1





Svq0Zjyd2QPPi5bL/NkOdoOaygr3Zn7EKCOtNc5Sa41zLo+NHa45lziLhy0OvDXq6ZwDDEDc





cS2P9zE3jnFiv4+PcJJjHMYYY51zxFoHchzKIgSA1ZYRCkqtNZxyKZV0yjoLgywrtDTCeL7H





8lRaq4tMbvRX1hqOQlCHWaypk0Gt7nmMBRSMUM41IY4xwymYcFw4IhwoscwSc2SW60WGQmOQ





ybwsTcIlhRHGYe9g1zHlLHPWEUIY59/4yuEX/uz/eLjAYx97xm+0NzfW2826tFVelgxahM1p





quJMGrB7w0GzU//N3/qta3du1QK/3+v9+Gc/urW7pS2d5HkqVbvRns1j54nXxwNmIqfqZV4i





EEWV7Q22bxxue7XaZHqvdXYTlGTpnDgJKZ0sTZZCEAggX6DK0AgAg3KAdAoj4SQIAe5M7+1C





CtTPwCmUc5ipG9421cQn1FbKcm93ntwbz/b3DkbzJD7cLrLFfDHrdJuU8lLaOE8tk7NpPJ8t





au3md29dzcv8Gy89N5jG26P9cZq8fPvG7b2dV25dL0t5Zz7aK2c7s0lcZUpXRBUC0NzB96TU





9VoURl4UBo1mrayqazt3iStVqyKAdm8qMh48HsFjhcwtLDkOcbxFtXHMDA7AAyXYw67Kg8d3





wL1j46ThJBMHoZQyTnA08A4gxjqnYa2jjjoLTgmhTmlFyRupFhgIzp1260srxLlmvQHjSVWk





qf3pz1+xzoLgQ49098eG6FJwL89NZZ1WxnFmQECM1vp4YmpLtLPEOSeLhSmlm2dWVeU4Ncp3





lpbOrPRXmOG//Oc/wbm2zlpnYS0ntMr1L/3lv/v3/q8vblw+zwVtteuO8jjNxqm8N5wNZ/Fi





Mbl9/8CnZjLaHUwPX9y69mt/+DulR7557cU4lR3Ru3Jp8+ee/Ylmo/Oxy09Ent8IQ0IKrQvr





vIh6YIqFLLeq2e74vA0i7t+8FqeL6SLb39+XUsLzlFEwCmWJZIpsgawEAQIGUqCsQCl03N04





CmH0EHCdLtxkqhk/nKfZQvWb/bYIi1kymMVGcMEwzarBZD5bLOJ4XhZlVSnjNOFWCH8ymd98





8VVn3P3bt1ngUw/xtIiTcjqdT+PUsFAZHVIOZQl0q9sWQhDCKquNMIK7Wig4dc7K+XxaVUUd





Qkl9f7D9pz9lNakI2DtZ4w37wtRMlkGCcfrA4sADdngon/KwpuNhb+RtU869GRnFCWYNnGzi





0MoILmDfIA5n4SxgKSHEOmestbCawTqjpJFaV2XFQGFtFHiT8URbK41Oi6JXb+/u78T7t5kL





AXd5zdubz1ZWV1ZXW0o7ZUxlLErlFomNU+QZKmlMaWXljNEwVhbEGVY4o4w1TlBms9SUWeh7





vW6DM7J7++5/85c/ttT1GDisddYSFLDib//qlz73E7/07/+nf/3qdr7Ic8Y8LqIAbKnbrhy0





tGdWl5679vLlCxcf3TgbeuF4d3+zufGFz//shzfPPXbhEZRyqd0gjpLKNEO/Vnc+9zl8j7Fm





vVmvsdliPiylbK98+YtfXV7uEc4tD3gU+aF4+e6OWNuw6RiCoNOBH6BU0Bkm82L/EFkBnoFT





pCUoMPmmW8SGi1hVk3lsjQVw/2C/0jJs1L0gGGfp9nRUWlNJ1QibYViHcc6YbJHmpXS8EGE4





zGWeZHOdFUXmrKk3Xa/vex7Xuqpk7mCEx1Z6rc5yOJtOu90ujH1i83zbj4iTxFZGlsSAE8qt





QK44FUG0tN6U3c3AWPFO1niD2knlKm6cUVI9TChH8+chpsDR+pT3MTcePvhBavbkxg/BfuVX





fuWH/R7+WeFX/9p/p4xkjBrtADAGY2GtI5SAOEJAKSgljDFHHGNwljhtOOfaau4YcdxITSzt





NlrtpW4eT1/fSSytf/bj9Zu3x3UvbEWBF0XC2MD3qNZYpFaXXBNXlYwyC80tsVqzskJW6kWq





GUcUWMGd036ra4IadZbx4DC+Nxqyi5u1F29lzLmCGW4i36eAIyDG+l/98uv/5Fsv+6e6n/nk





hxdlGgb+hY0VR7B5tg+Y5VZ3rdN5+vJjDd/3udGyVDL1ocFrkWNM2dV+f6N/9v7enmUEhGpV





UMEIU3mhd/duzkZJd/lMjZob9/aIqIJANJuNpSC8feveIjeuTNN5VvMcmg2Ixvxw2OitIqTg





p5EuwB3gbDYZpYtZWvEgMA4wjlJeODscDZMij5pNWqgwiqaLuFSy0ioviqPbinEGOKsZM0IZ





U6gqrXLhCWkLrXRVSi6o5wutVVXlUlUmg2h6nscOdvdbUbtGw2boz4vMD3zAVkopY4ui0AYp





sc7NxaKKm/XRrczoyjm8lTWcc9YpbpBagwdB0IeNiwfbAI5OepvL8zbn5SE8oB77V375P/nB





zvofEE5ycLSCDhgDJVLb45GlMNZpCMEIs7CEcALCKHWOOMYY4LTRHmMeWCal5/tMUKWrkPqx





Jk9e2bxx97CsGAgry3L/cP9KJ6R+01JDuEchnLWGh5yRPCsjQq3QeEMXQGirQwAqiE8JHCs0





QmEgi3qj8aELHx/FE+1FcHcVhG8oaKU0ATFUC8ptwJyrar/+N//o//iff7cWLX3m0/3/+pf+





7WBYmEpqhcKUK9GamcWZK1vNRgXU692iKCkrFaecB2WWCz+4sHr69f07ilkrPIJchMF8nvpB





cJjv7byUDDfPPHHmUre3dGq9lg13NaX9Jjq1aFzq6/uHxDP3Xop766TFvGyRbA1uTorFZ37s





IjC3s2kKbywXxhTx/jwI62le1qOoFgQHsgrrtdFgjzK70qg5ayezpFZr1upBtsgoc1QzKmAk





0vnCWKdhvJAwYpiFNppzXkgnZQ7nAo9rg2ExX8oi4ni3vT6dTKeLmYJL4iwMfUJBqJVGV1qV





ytS5J1zwp65cml+XL+tcwVBL3mp0WGutpZnRwtrqbVyA7yUwx0MOyzvm3ZuuihAn9v46sR8M





QKvmLZJKCOKOREuMmspwRiycMW/oQIXHjHVwIIRUlWSgoe87rSS07/m6UqLlB34YJxPnyP/H





3nvFWrZl12FzrrDjyTfWrXpV9VIndpNsNruZpCZNiaDsD8OA4CAYsD/8YTjQX4bAP8Mf9qdh





WIYM2OaHLcEQSJAiKZEUQwem181255df1at847kn77jCnP444Z4b6nU3RZHuqp5169y11w7n





3LPXHnuMseZaO8SMM3k8qBExShLvva1qrevagCrZG7Jl3X2hIZQWkTJlVQ+nMtBSqjAKOYk8





sNKSlAZLurPh6rIsKiGTF3ZaOy3z/pOsEYazzLKYTw4DCIoFMAgHINGHIhLc4az+w3/14D//





pbsvX9vQTQ7z4P2Hh1KHN1pt5+GwPxzN6r32RrPbIltNszwrTO2MJRrMJrvtvTianE4GM1t7





9J1NhTbwpQSe9ut8wvTb/+LX//7P/Wwnmm1v7zy8P2SS3pcv77ZPx/Dhj+1Wo4fDwVglGy92





rr+wYccHb6HjtNd77859obmqKkfi3nGfULSDVrPZuP3Ci+8/uO+J2o1OPs2m0+z63gvHx6fW





V1prFGH/aNBsNmfTwjIYtHHIzSjRqD2T0qooiqrOtVZCYJYViBgG0WiYJXH6ZHzvxt52NiVT





udnEDofTVieUQs9m2Xz8ShqHkuDzj++OZzGQXLc51ikDoAGVsKkujIK9gBQXulqegheLvdcL





F2YefJbiWQaOJ/fivRsIyJWtQAjrGREZ0NYgJWstrXcMKlaSCA07iVIJaZwJhFRKEQuhdVkY





F/tQKwS9tZns7VGVl4iiqirJoqig3VaIeNI/aYRRb3tDoLICNKNIU5XEXJpqOmXBUbwlBOgg





cSx1MyK2XFSNnZvTw8fYagZxaytoOFt6QAkSYZGICABMHhEZhRUzwTpSATn9m7/xlf/6P/oF





L9WT/kErgCKbPCYbS+mc0KJxNBhOTMVCFkVelKVKo8eH+9b7LWq3GpFvNcywlKGu/CiDjIUO





q2A0ffS5rx4pUKOKsPaD0Tu7u1sVgySuTgbdrRdmp8fZzAIoSfbR0ZAlEvg3H2/5+M1f+LGX





H+/fC4NAl+KVvdsHx6ejrBjOcm88e7SVG7lZHCtTwbvvvrux0UHjS+eYHHnwjr3HysziLb3b





7rExniwKIvZKYytI6roOwxAF5nlOVCRhGKdsNfbHJ6EOPJZh6hKRIGL/ZCSErmsbReHDfl8J





0J6H408y9cHDFagBwmJIZbGuTT6AZVymG5da3DnUYGApn1kP8VkGDutOru3FcRQwA3kvQSIS





EWglbElhQAqlcVYpQV5IFDUZgYIZhBK1sUJybe21Tq92VT41hmlX4bQWsZZJ0DwaTK5vbx4N





J2ma1jbXYVDX9dHx0eaNa3GtnCGZBKBD3Q7TRGMQoI6lSBkiqSQDsDKiVdrpQasV2UjuTzv/





zX/1P9XQCgOx5rmtGiICsWCFAkpbkG5WnLz59psHJ4cqTV689aoTGbuocCRAhKkqA31azCbj





zBi7tdMdDIfWORSASGRVfzhCEludrQDCPo6H03HY1FVea5CNsPm1b33tRif8Wz/5o8PjJ9Pp





Q49BGkd+cpK4YDR1ZTXb2saTwfjmS7eDMPh7H/rwl77222+9+3Bnc6vMxrfX/uwEAAAgAElE





QVReeom5YWn6i5/8yHi8/9prfypQ7vWuv3//XXWtF0USIQZwpjYQB3GcDAZDAFQqbSXNpCcU





MUtFbJ0zCpX1wosahc+rmfMAQnW2GijrqS0LU2/IxnBUqCCIY0lcHR9mVeUCrZ0zyCpgXeZl





ifb48SGRuIwa88UoDnOunuJxXl14Ot04G8myOtZ09mwOVIFnGzgAAKXwjqUUyMRMUkprAYC1





0kJAoLTxhjwoIZhEGOg4SusiFx6dI4E4tyiFkJ12p8rz114/nOXpn36z9R//+x9p9SdhUPZP





jgX4XiNJG5GMoqqugSGrib1pxaF1LIVi1kqGROixCLDlyaKbIJCqzZdfP/nYz37ql//hP+rf





dyhTieuNEi/cweYVSgbsq420Y238Qx/55HsPXx/byhr1wq6wNbc7Xa7NOM/H2RSUipP40eF+





u9mZTKeNOLyf7X/o1Vf3NncZ3WwyaTWbw3LWajas92FDmFFWSrW381LafmVY3ui9cLN4/G6V





H7MNDh5NirrGCJIkvr//uNfuPHj8IAzC7WKasBsXh3BaTwbZ0anvbV5vNjf/4HN/1u40fvJn





/u5rX/zC3taO0uJ4eJQ2Q7YqVDizmbfexLXA2NYGmzLdBih8zhAoXdd1oFRdWqV0TdYzWeed





Y+84z7MwBluRMO5JdsoATIVSIq8cCOXBVlWZNuJZlgkhqrrudFv99xn5ovE5fy29FVPzV0Q3





mBfjYFedKc9uhwoAPNsp5wCwvRsqKSuySuvaWCYkAiFZByIIAq3IsEtVlKoIiId2SkgByFAF





jThxjlWommG8EafNIHbOyTD61d97ROwA0DmII/4v/9PPPrj/xo3bN5tE3c0NU1SldTdfeSnP





ZpFOwlhZ4iRuYBLqKJTaS+igttLOuCreOjD//X/3++mWLAcVL0bR4FoIFGfleREAhJAM0LqF





v/xLP73p1N7tGyjMpDpuxR2lVSPpHA/2R1llSBR5PSuyKAwD1JFOWLj90fGLvW0VCtUIZlU2





mUwKa7zCvC6FVpOTMfigqitPkKjOKzc/udnavnXthRSfCCxHw34QdWtDdZ0VZanjJA2T+yf7





G2kzq8h4++nP/GJZns4KenjwRGp95717P/aJH/v5z/5ceXL/S1//f0/H/bipknB3e7d9Ojq4





9+Bkdy9h3tk/Od68HsqoAgeNsDEYT5IoZFs7lNZwZjMUwjuqKhsGcZCyCsAbV5UMENSlYaqT





VuAZppMSWLCDWTaKG0lu8o/s3PjatzZO3q74fI8JLdPFw24wPsnBuXnOuPd+1bfytMJq8XLP





y7y4KgAQM+XF69+xiX6fxjPOOAKpUQpkV1ZWKWGcX0xLDXI8pN4WAHNtTS9oJYGa2twKEcrA





Oe/YVVTrIhpPZtjd6u42dBjcunmdxD0kZAalwBP/L7/ypUDJ1pt3PvPh7mYVSxq/snF9fHoY





hemwom/82Tduv7J5rdeJO2GTY062ArqHzfAf/V9ff/F67/37AyEgP6mFFKsnH6/xjdUUMLiq





F0IyMAKOD9zdh/XurZPjSevV3vbGzp6hYv/w3mjIxqlIt4tZ/fD47lZvG6TKiqzRSAVEPdGx





OYdh1D85KSmXQraicFjkiQ5JicZmh0jeiHaOT/uzYvbtB1/QIvnym+lP/djP//gn/t6gfH1z





k+IkH56I2raqGma53d663hQ6iUcG9Fe+/MXOxvZwXCfNNGnGaSfudrsAjbC3tXvtdtroHg+O





rKzuvX/aaAVRkExPvWqVVoyLKgm8j6Jwlk2kFM45RjnJJ0QEQpEALwgUFraQFAkvAGA2y/IZ





JA3a2WqykHXlirzyniQiohiOqyaUX78TPX6j1OCc9RcUCqOznN7cS0eHJaxd/PABCuVqunGu





vF7gczXPYDzjwOG8B3BKSBIggFAwAiqlokgJRUQm0sp4AKZGnGz61tDm3TQZW1NWxta800lt





GD8Z9bOyfuXG3pP9R1g7FIKFBAAmyQIc+/6x/ZeHT0Aesveb7Uc//7M3f/zV3f/hH/+Whw3+





gwMH+c5G+J/9g7/1P/6v/yfgzq1ddziMv4lvOx8yGIEhMyMiMwACLsGCr0pAJCIhkBDYqNe+





/NYPv3C9UxxQJx483k+6W7aWw3G/ti6vTT87inqhECLWYcUjNkXmaFRMRKdTjkfGeS3CWpac





KG9rU1klu9d6LzLoSEvbkoiDUhamzFDYr37r86NBf//05BOvfPIzH/8UJPcmR+8n2z/xEz/5





0t07b5nRQYS+FTb6ETqhNq633r9zf/po//atH3r73lu71657O6nJzoqqrJzzhthlOVtHadrM





imGnu+Fc4QUYYxCRPFtPzlsppVLKevZEAJDnRRRFg9Nx2ggDhQQUJyLUUBs3nU6MgaqqpZRV





lSdJEoduhrfv/T4EWNbuihQMY2qh+I2vnIYKPEQA5XeUKrC2weKsnF86X3jK5BzPUDzjUgUA





tnbCUAel94zeevKOWy3tyEsUQoBAwex3081unESN4HA2Os1GQqkQI03cCFJQwN7HYbjZaDca





ze3tjf/5//gCyhQQEBEYiB2CAmREQYwgUXgAQCEAEJRMrCuEEMwoZIpUAEkRKCInhATwzGL+





dB9EnEsVAMA1lXJWXugVFIRWwC//w09vbXtTq1daTQ8QBIElMXHW+2oymj6cPt7eupFg5Ewd





SJy5yWZv5/1Hj4MwTMNAS+Fs1ew0azJZUYhQ1Sa/tvNyGu1U5WkSRMPT05PiVIdqY3v76Mkx





epkby4IPD8d/+1N/Z//enZlTkdY7W7uO7EvXP/TZz/zsN+5980/+/F8IETaa8XB62G692kqD





n/r0z73+jT/+yZ/46Xt3DopiPBgcxrHKcufBGMdRKlUknLNlkcctqZWqjMvzvNFIAVhrXda2





dIacQ9YoSEqoTY0AdWWryu9sN0+Gs9m0jKK0KHIiimPNjl2i3/2Da2jYOw9wLu9rfvGjFzIU





dW2ZHXk6kx9rQuZKhbIsrKuScwqFaE5NPDMxU1G++Tfb+P/NxbMPHDs7MSKwABBcWW+tEKS2





ttAxO/LecZIG1+Lu3vb2eDg+zgZWQOlqwQJYNFQIgnc6XSa/kXa2W41uI5kgoHP/+6+/x54Q





ERCYBM6pAiKCAuHnY+kQxdxoR5TzVBFA1CytYFw9DQyX4glXD/fB5USHeP7XosDgQelf+Ldv





/fAP9e5l1Q8nikxxfWdLRNHBeNSNkqNiFLYCl0szm8lAAPheJ/WVG2RT1YiMNa04biUNKVRB





RVnXlTNx2tSBjKI4Tjre2E7SmExGxpVkpipOSu/iAGd5MRhloJNyiuM8bybxbJo1u3u3b3zs





cP+uDKYCkyRqszzubm4OJqdsg632S7euv9o/Od3eaR4c7h/vP97d3rGlGpdHjqDTipJGNB7P





yINIrEBh2MRx5L0zxgnUk2keRsI5n+d5mgZRoKuyIC+ryjgW3U50cDINw7iuSvYkFVWV76Tp





o+G1/h8Lpwwirsa8k2fUgr3xDubjXC+4HhcsjAuV69ixoh7zIbZL8KAlgMwNDp8XH/wUiO/v





eMalCgAIKbSQQiEBWFewhgCBWUgkQxiEQhAMssnHX3ihVkEcRQ0QgQprqgIMlIfdzkYvjid1





IZwPGWxdt7RyUfJL/8mP3n08/N0/ukcEQiCjRPYAzOCQEHCRgwGICGLOIgAAGCx6ZIR5Pvl8





0O4cMpjn2zDwvEMFF0MklolJi0OGUlfs9Z/86f2XbplbUoyLcKPdODo6Ctqtimrj9XazZ4Ec





Vciy8CZUwk2MCoJIBrZ2SRgyysqTqqoac/bUjVMdNy1ZxbLFqjAmDHRbNw3oibXGCqniJGoq





qMkMLDqXZFspS2nTZly64Wj2la3toPItT3XcGklIR6NxlLT6w7HN+llxGAahGL44KyZC82A0





BAAhuBWndVlqHSitXOAYmJGA2RijlPDeF5VRmq2rvcMkDWItjXHGhp5tXjGAHU1tsw15PmKW





nlgwCnACKElJEpQBXP/Qw4OvvwoiZ2ZAttbubG0cHfYRxWXUuOB3XqiEKzpTVudlrmXWl571





u/HzAByBDpxz7EgIDJRiy2GovLdCwOS4mbQz3UGF+mg0q8hGUnrLRBSjRmStw3YS9rNxJ2ym





SlVFhV4qG0pjo27yqZe3X95t39mvf/eLr0cqtIYABCyu/TOk4Lm5iQALZFjAAsxXrxV44Yki





44oKXmypiBV5UODNODp8Ut/YjDHwh4N+QKIlxElxHHW1SkATV+NCBWpLd3JbltZCaVljwqqq





SYfydDKqyceRFAIU+my4H4YRmxgKu9faSsgIlJaDZtIa5DkjVNXETMeaQQuRdNr701leWm+9





FAEFOM3rgvPtre4gq0IFSsah7UrMUE/anY9NsgePDh94M0sC6Q1ZV7Xbjc2trZP+wSyfSUHt





ZjqrKxYi1CEzO2cRUQiYzoowwnzmtdYU19nMSBlY56rKAvhOt1vayltRldZ5aqoIwY7C2+/9





ucMg//jPVbYcYHDTGwKAZCPJTrODgwOlQu/OwcT3iiArT3RRsxgRO69c/jD9NbXvv6F4ZjPb





VvHw4UgI9ESIqKTUAU1nIAARA+8gGzQ9uYkrcuOMrcmhlDpiQY4iVOhIa73VbDtmEQdlXuaT





bDKa1giSoMrytvJ7jeK//Qef3tsINrZaKMOz55PPx2gvk0AX2AFrYyqfWlimBJzTz/PyYvpc





BxbR/P7vDqammswmZV1WkbFUtVRSUe1d7QTGOmCGsq7LsmSUzGhqRh1qocaTSRP1i+0tZUQj





iMmYdtqJdUMYqTFCGeY1odRBECjG2xtbN5u9snaoVCdpmLySIL3nWVHGjVY7bedVPStzkmo6





NlVGVGxp2nn46B6AZ8n3D76tdSioBmJnLahKCI7i8Pi0r4LAg0UmM5t6Y1xt6zJH9tb6uq5m





2URKtIacc2VZ2lopGQCScy4IlI7ULLd1xs6zDFAoXbv8hZc/+u1/OU2OqQzSujwuT7fsYsJq





HJUjTyRQe+fmEuM7IsWq/tJmsASICzMVn53IZ7tLBZ4H4AAA570QghmIPJIwViAigJ8TBOdZ





AI5mk+ubOw0MM1tKxgiUdxwp+c7DB5GWO+3Go/5J7tkaqj1R7aVWQShBhbdf2JQx/js/s/fv





fVqF0hjwiDR34YUQvD5R1IrRfocCrOjuot0uJtBdzaM7F9suz82Xvj4S3gCwGblGEOfGW0v5





LLdFDugTpSyQt46JZBAEQTCeTJxjdLIA289P2s2gyWFTb4JPA9xMwqZWSW15Utqjfl8x3dq9





HaBEDFNO2Ae1B1TK1Db0nIAePZn1TwblLC9dYKaUTYwtVa/VGY7faDYTRteMk8oUJieltbMW





AJkABPRPR7F0vhqlQbso6342qyou8gJYWOOtc8wslRASppMyCIIwVEVVFqWZTY0QKkmFc64s





M4aKgayHROHmzq3P/VEVVlHeddd/Wjf9VEk1T8R1zsmphOUsxCsUuOxofCDX4DOgWD9Xaz22





K95RlG//9bfzv854LoAjiVLrvGfPwArRVsF8SchCC9QqJEtFXTw+Oh6ZIkApAdlCqMJunH7k





xq3aCZTiWm+zAO9RFd6OxjNSWNicqqIuqjSNkjTduf1SbnzAAgAB/H/xH9xctCSAs/657xZE





znY5g54F46D5Hc57r5S6903+lX968sW3o1bUqF32eHY4HWen/fHx4cBZmk6mxWSAilliZa33





rtVqeU8CsMyMnejBgTG5jjg+eP/xk4f3Ytbk+HQ8tQ4kCA16NpqRMdPJsfIVaZ85B8jeVe1G





EqoA9bTVk73thqPcVsX1zb1uEt1/8i1DYeGqvRu3snrUa2+yFXVtru3uCIFax1EcelsQkZJx





nk+FhGYjVUpGUeQ8WeutMY6IEB3LINWeKgKLCJ681trUZjbLvAcGCiLUSjBDGJjRMJzdTRxV





QTN/ZTfPy7SuwNtgHQj4qul8PoBrzL/w5SKscQ0W8zzfMwJytnZFPJ7hePY9DgC4tdN7534O





hIDoGaU2wGCsfel2u/Lee49KduJUANzc3a3zoshzinQranRCbY1HppmzWsoAoEaLAMju8b2D





G69sgZeyIp7ZdpQ+ygOJmtkLkMzwv/2zR5/4+Mbr7wyQgec9txcK8/a1LCDOjdJ1x3SxJTAj





LC1TAABiQkS05JAxhuaDdw5ea1+/2ZtsJz0SThLrIJxMMghUSyVVZUj4RAQqDI9P+rWsAx2j





0PkkS5uNo8GgqerdF3YbzSYTDvqnk2r60o1r3vG0riubVX6mRD0yeQEuhobWUul4/+gw1K20





2TCe62luarvd6uTF9HT4ZGOzG4RJo9kqsxkKsbNx/e67b2nACRpTVTsbjaJyoVZZlgFHURKi





BFMbch4YSXhPvrQGAIQQcUogvPSh99JRhQhlWcZxRMxVDTqAsq4lB3vd7az2X/9jP7eG/CQM





JTHgbNgmLtcVx3rhAul4CoLweXg5g/G6rnGem7dGNwCYgPFZ1ynwnDCOz3/pnVCHlTeC0YMT





TMQsUCBgK1KR0tL7gm2SpAgsJcYqDFVUzvIyKwRg0kg2ex0jjUCUKJpRBDWRgdFhJoLIewBV





50z/+Fd+bz6xNZFjBpL+jXf6yIvHOF12MS4UzpgIP513wLnGiwSIzrBRg/gvPvduq6FbMmwG





URwEseTS2smsyKw1hMWkGJf5MJtmppra+rSazFzGSdQvi8Ppk4cHj44Gxw8e3Mur0hvbTEON





yqLwHqKooVVjkhW1JVt6AF9ZX7FgGVYmE1qWpc0yLzgQlOS56XW3hQNyVFf5dDyoSvPO2+84





a1Gp2po0TSZ5VltXl1ZIbcgaYGMgiNISfI5U2rp2NtAaEYMgABYIQRBJ7yArMgbQiSNh4ziO





tGw1e8yBdU6Gg9d+B/wIAMB7MqPyzS8VBOXJvna+uEA3LqDG07nGGcU7szxhjQLS8pysKRRm





Rn72dQo8J8ABAFKiQklECmS3Z5i01qEATFUkGSMV9GeTN588YhRJmkopSlc6CYPaZnV9Mhq+





fudRO2la8h7Ye9+7tRNGajwuHt19VLMrWf6T3/o664Z3i2eRIYJg4bxgoA9WIk8DkctbXoEd





y3KJk26jTeVJgGJ4clpXdWErpw2jzaZFkVfTqpyU+dF4gIHwxgZhaMjnZIyyY1+jDjvtnU50





XVF4/dpur/HibExHp6ejyf77D94s6xFL5QyhV3lhxpP8yZN9InCWvffE5LyRsgxkJ5DueDpI





kkQaU4/7KGSr2dFaR1EklDDeVaYmYERRO2UJZShnsykAD8ZDJSRbZ52zzgVByMx5nhe5jYKw





KrUIXWejGSWBlp2tzY1mgHt74HHgtem2b3zxN7eTOmWQRAQMAqKDd0ROW+BKZ8VlyFjHiKdz





jRXdgLWfBUA4b6VUZyRkmY6zxJFnP54LqQIAdx8MdnZjLZUlBwioak+R9UTIaRQX+Wyv1WaU





0+ms1UisIBFBbQyidFlmvU3C+PHxOE1C8CAYwfjEsFBSBeHDo+zXv/g2cxoIQhXOu1/n3asS





V/2yZz2tAIh8lrtxtYoBZl6Jl6dqlrMyB5NJZco4V5kMQHkBGepQioC9IBQIuZFBJCSWttjd





2qrLOg6iyajc6MV7jVtR2EhUpANVVLNWs9s/vDczlUqjU1trnz0ZTTQnhAQi9p4msypKAmYF





4IUkRpKohBdKOFLBFsQv7m3d2T+eTmpuNdzMps1GXo4CKRA4UNF4MkMhUQXD8eT6zWtlkdVV





td3sNlrpYNof5NBqJ46NDiBwQRQnzpdCZkA+CqMwjMrcWjO72etWzr/51vDwUc/1JThkwchu





fp07cgzqyRsdAJoLwQsY8TS6cV6SLLjGWbY5r0CE0oYsc7/GQXiZvffM55ov4nlhHABAwB4R





UTgfjvpNIrLO1nWtEJSS5Mlact7n01nhLREWVHn0hi1LcVJO6rKy4ImBpCCmoBU3u41TL37t





c3cN95gISTDQ0g2FtTvPKiOD1wZArWmT+dqreMflDS7zDkBirJniBkR15SMVqUg65Zyjsq5q





riflpJa+IOOcDUCWRVmaCozotZrVzLaTZiLDKitHs/7psP/k8Khw1XQ6vr6z8+h4OjYj7QKw





ut3c7LWCWTaqAdgFW63u3nZPomYGLZNW3J2OD7x3H722u5HKNI1vfuhHWmFcVdl0OgCGQGsA





mGaTui69c7UtG410Ns42eu3t3WsgxPHg1IEKwyjQsbUsSCOTKTO2Jgp0ErZtriajMkDXkeFR





mb953z76xoY7RHDIFwIY2LupZuZ5xidf8kQv040175nW16zSNM5sUQyqYv1NPeJZ3/nzoFPg





uQKOTtL01noiwKrZqcg7ZjDez4yRMkxUROxLY1DKQCqJGKA03lZkClcloRIBg3NEZCXk7KdS





ndTq//nNNxhC5UtgYFjl/JzHDj5fw6txr+tdLfBBIPKB2OG9pyogOJ26oi5NPavKrKiqihki





VmxBsyzYebLMvttocWV3Oht1YdjB1sZ2meN4mg8nj5h8o9s9nUw2tje2t3rHhw9cPmw09+7V





0m3qoRm8eTLLccPBZmdnJxPRg1ye5iUAZfWUKjm1+SwrNpL20cP96clk1s+LaRGE3NvoBEGQ





53mSJmkSCQlBIEFJFYUsxLTge48fn8wGufP7J5NG2jo8PM6mdV3WaayNLQRiYcTA9ZVnwY7T





6sBnp4e9r/2eRt9ivCQvFhLDe1cSkRDiMlg8hXHw2s9ZN8oFysHMnquV8UFEYdIw5BYDBuAZ





z/taxfMiVQDgzr3+9k6khbTspXBRECFDWdUhihzqA6o3w0YQaMc0rjIhAZTKqyl7JAmK0Ury





SBzxxJatGh6N/B/96QMpI+D5vJJ4lvwJABdyQ1dKBVbZ5MvyFV0tiHihHi50zZzTLMiAhpyu





PbVkkgTKohEWiyxTocrAzlytPDvham+DME6U7h9PAh2VWdWOw8GowhC2u51qVuTZqLPTPjnc





37nWmUz9J3bad0/pC++mt7O4HXbccXF0Z1oX/tspm8yp7eizv7hVj4/Z+SAIyamddkuhvLFz





az+7Y3gGokCH0gNVZajbs9nYGiQhs7pUcTzJRlube0fHx5/61GceHn3z6HBUGvv48CTLRxvN





Lgp5eDTa3djwNo+87yQRxVVXNd6+oyYDOv5GqEXJkFmXSqyYGRGdc/PEmcuE4srF84yDzv9c





YBlnwczM80e6LfRkXeRJmpi8fB7SN1bxHAEHAAhEZgiU9p6YpQcvmQs2sQr2oq73xNbn1hTO





kKPcGS0kIKZBWDpoEBfStqJABr1/+YXHWS60Eh6ImRCXxG0FF7CCiWUtM+ASL9bLcIXTMR9n





v4YdyMi4mFP5InYgC0QUEHrhJtm0E3XRQhymUlYTVzEQV+ZDt142TFMzjrTUQk+ofDI8iIRO





Qmw0k1E9fDDMsmm10dx4dLx/e+96mblOVOfC9EcdeW/28M2Z4kCBR0REaSqoHCQPo+GdUdiU





gQpns1oI9dLOjhLaeVMZc3AyitoqCRqDwYnSsdCBqRzqEAWb2irQSqv7j+60mq033/p2p3Vd





Y9XaTI2BRtKofQ6iKsr262/al159qdUqIjn8+p3ena8b5QJvZKinzJKJ56jBzFrr1QNfV6Dg





gFA6NuIDUOPpdOPqOKdZBDIRMpiilEo6a/4G2vTfUDxfwNFOGsNsppCZhHem1Wy4ymx2OqOi





LEwVg9ZSjOvxpASCUuiwytLhwNcVoQjvikJw6OEUxTGQAkQPAoCXqLHEDF6MSIFzMAKrVZew





A+BS4sZTsQNgvX6BHQCIhEguL9FjUbtuq/2kf9KIE+WkNbwRtb31Sok0DYRHBCkZNlrdjbS5





1+5459C4/cl4KkoYD1643T05GTTDuIx2P/9N9fpfZKl2kVTElsR8njLhnNOIuZu++U33d/9+





c3w8rvjEuqoZfNxZU4ucgXd3dkfViXWVJyryMoh6jVZnMhszM6Ot65HzKkmEkA4RiWcIFSK2





22o4mJUuuvsXe5NHWwqyJ/3q3/35/oPT5qM/RyI0YJnNfFT3euR5PqdrvMz0Z5YaiQjcGVhc





gR5X0Y2rFcoVICKVBPbeI6A1BvEcN3m24/kCjnfv9a9dawBR5ZySKssKz7YaVO2o4RFR6VEx





G3tjKKyLaDqItULPjAIQHbBktogIrM5GoABcVCWwjh2r0oVV6/pl+bJY971jx3x4TIgQNbgU





TwYnNblbG73jUSEA0ihg5EDIRpyMK1c5P3TTKImhMq4yj/ygF6W1QVJ2M4gtuQd35Re+cMKu





LcQdIWWq0KNy3ikhBCOQICREFEJoEeYjkm5L6iKJtqbZUV4XcSOVgOhc4Uo2XKEVOgoDVVan





09mk02mHOphOZyhlFIa5GSMFTK4ajSrrqoHd6MJLmx/9Z792UBxrKY4Ud0fk3zjpSYhz75j8





4ltd85fnse5lMDN5YHGa3vSedX4/WaLF1ZxjdZB5/VUK5aI/CsCA4G0lUDAwMKPAonjr33gL





/v9NPF/AAQAS0RLHSjADAQRBGAlFDMOqmHKlIxj2Q1cEJFBKTzSfJ0MsngCIOO9wm1/gK0fj





oiqBD8SO9Zq5e7EwOxDwCuyAuUh5KnYsDmwmriAKa7Hb3qDKHlEmVNCMAqlEFMahFIigVUjO





ogons6kM9IgmRR+Pg1L42Y0XE672/vkXjsYntXMtHRAAMpFDRLYCkYkJAZGEQABBRFpX5P3v





//rkp39xr6L7zpixLVqM33x8h1EW1iipSmPIlVIKm1fI2GluVBbzuo+IQkqJ7aLMpLTTMdWm





DJWWgr78u8PqeEuKmoCHVfbibiUqH8Tvbu7dPHq0ysnkM7f57LJfG5Pm1as/ZTwOW634K++G





iIKhnnsTK7C4nPx1fkjhOW5xbmGe7kt+5VItzurzFM8dcARaSSGNM8RMztUOZOS9E1IgK+oP





hSklskD282l4LqiNeYvFy9Xr5sYHY8cFo3SBBXgeR1bHWTM4rsaO+WvIXNsAACAASURBVFoG





EU1y11Z+lBfdtAHWdLsdobHXjQZlHsetnmo/cqczxS5zWinryoPJGFx+M70WJPGv/d/ZaPpY





CEB0QjrEcP3iBAAAAkAAJEJEBkAiQkQ3LL/wR/gzvyhViG/d35cf3c49oYSU0qyuldJSR54q





FDoQpn96n1SLEIqisERaSSlllrPSMi+ot1W/9aW90UlKfspe6CT42A+rTu+xsWTHnf3781Tu





9U91BfVYAANOe9t48MSEOmp2y2K8ZX19GTUuwsPZGbz8s15/lt9LS4ZSlM8R3YDnEDjuPRy9





ertnPSilPHvBYJwofB1oYYndrIXCoFg8tgvP4OEMGS7Qh0uF9biKZVw0Ry9ix9wCOVd5sf/l





InYQWonJ0X45bB5/cuNjAHDr+m7NVFpbl0Ev6h6f9B9WpwQeE+U8N5vcz3kj0k/63X/+Wx5q





hzIlmSHFAICo5qCAKyxc615GXs5lhoiIEzLSQFbGjUBPsuHJaTCZZM1m48njgzSNPDIY48nG





iXYOgD3LIk2COFLOOfI1ipSZimkeafSz6+Pj1LoSiAE8CZ3svF1W7PzkzpduMHk++yTnu7jP





xaIn1RrXbXSdrKbDgHm0DhZLW/Qc4zjPLy4ZHGsVAHCWh77MNP+raJvfT/Ec5XGswnkg8p58





oFQc6NpboURtDLJv7TD7YKVvlzR0ubQe525QFzbmq/a4kNzBa7usl1e7rWV8LO+sa4X5qkUz





FhBaHh0+Ji20CmykZO1dfzLJrW+EzaNHx6PKyCDqNbuVcdNi9K1HD99+kj3q67vvOVOlhL52





Bn0Cly4ZOmcNLId80dn4/hBZDOSf/GpWVC5K1WhSO8LKWAd12ogQuarLdrs9nU6nRZ2XNZIo





S+McG+uth2w0JgNCyjAK+4+gzmdIQETe0zSbRXEPZPDOn32mmK6MT09XTBJKy/rFqrreAihY





Te99G1cfd61wNeO4CBXnTdL5lwNLnFo/vXj5lvGsx3PHOACAJQQ6sWCMsa0kQZLWTmMdGm+o





8kLAOndY8y/gIq1YEyPfhdlxrvYpHbRP4Rrrk4YxX+YdDF5CaGeqm7qayydT12mkt3s7w8lo





MBrk3kzLsttKGPy4El/6BgvcPNmv2KsIm0KWzKgEADs+/5cy45xbMCCeS5JffmBGixKwCjjJ





T8l0Jx9tb76dl3EPoKSTw0HSiJphVJdlo9ksa5/EDZtb56kmW1WVlLIb6Uk9jOMNZ4LutQeN





g+5onxAJQWEd/OE/tUr0kDNY3NbXz8UZ+7gQxPzyZx869LORPn7jVaaZ94R4kXGs+MaSL1yS





J/P0cu9xmRm6lkUK63TjedMp8HwyjgcPhlrLSAQSRVlX/VMVS62kCoOYogqALlKFD+IdF0tX





8I4rCquKM8YBZ/ewNQJyjmKc22DtrRgAiEglMxJ4OMhjKYfZTAHtbW0xcRhG3XabkAtJv/ar





j0+f+PE+CtJKmUoZBl7Ls55zCrqKaKzdrc/yqWhFPqwVL/Q2OQyYRCDiKNFBwIFmZLTWFHWp





tTwdjRwg+6LXCF6+vvHqtY1r127sdG/42jmX6XDrlU80vGP2ifM1Ui0I2J9NUz4fNX+JZVwK





i3u7ZrfRGZ8qox/XLvfeERGvPdJtHT6uUCNLToG4HDh0nowt2cdzp1BW8TwyDgBAEEKQZyi8





876yROwdAkpvPMcSV21ilZHxNN5xpbUB59aec1Ov6KBd3MvXucZ6r+1TuIZAsawDYEDEKEy9





m1V1FSc3ATBzTgEopZtBOO4fiQRdUWqtgNh6C4LZowSYD86ff8bF7yuIxvLNERcPgVn0+syn





YUYQ2G6Pe/HOrMg7zZ4vQMlGUeXkzSAbbW2lWromSq80ko917GrvLHlQAr1ibjdTS5yXdU2z





lz7avfd2JSSQN3MLdn4uLLIUUgGSp8U3eYXHQcTsOdluJ6f9Mk694G7Uye2Ya3JADlmtb8yX





xqGc/1mMeV1iB58bw7YE9ueQbsDzyTgA4O79Ey21AqWFunHDMhETE7KQgOeMrnVG8RTe8UFm





x/oavrTD5YOfe/0grgGAy7nTV0ebzXIVxYxJYWvD7mQ0PDw9LY2R3nvrrYSdds/VwnsPsJyJ





cDkb4XciGut3aVr5BfPLqa7rV35E97q8Fd7QnhIbffWL9TtfSa9vdxphDNIWQjL72XTKRHEc





Tqf5dJgFQb21kxTlKM+zNIxaUVdKmfTUxovH1z48mRaV54CZVpxCoWDn/eKJjXRGddYMDk9E





npQof/ufhKdD2Lk5auw9vP7xk1uffSJSIyC9SFUuSpyL4Zy9RE9gIZuWrx/QzJ7heE4ZBwBo





IXPiSKmsLKWUxKxZKqEFIlzEiO/EO74rswPOy/OndLKs0YpLPOUcJXHkpZBEtJiHCsFWjdm0





QIU60NOyiKRKG62K4fHxMYS6f3xQNTcQHbNyZHFBWJbEYkEunuJoLIgGrsjG/FUIOb8r711L





G3Hr0fjRH/5+8fA9JlFo0i/9cCWDUTdoumlNYL0SMoDpdOSNbYRNacLx8cRW0zRtl7Nx1Nws





srEB5wqi+OBHfurVt/8CiS2gnH9lVDkAIEQA8P7se1wnHPOrW/hyUtmT/frLn9ttJe3j98pP





/p1hs4Wn/doFmbbIi87mhUC5KkWU+MKh1+TJ+usPgOO5C88sEQXKRqORlzWAC4RCIh16ay4Q





sXPYsWaif4/YcQE8LhmlK22yBhNwHkTmkoIBcZ6ziKtDkwfEh3d994VsOG1rqQtbjbKi0+ve





PXm80emkEb5/WDE7hvljomiBFMu0s8XvD9ApCxUz7wbG1f1XCPH53x4Qg5DgahcGQ0m7Tk20





3hlPSq+c1kgA0pEGL6VuJY3dThssQGhbza3heLbT2XxwcuitKYwRHFgr6/hBunGzGCREbu0b





Y6bz9tPa1b1iRbkzm9fFziv7B++/5KZkK7r71byoxKf/w+M3Pr9tBi1Hbq50LmEHMbNBr2g9





pX0JIks3FJa7AHBRvvOv0wi/f+M5lSoA8NbdQx0oJaQzNtDIBLV3jn2zm53va5vHOQ7CV9Z/





xw7a84Ll4sH5TJus6ZTlnW3ZVFc3PPbeWbfqGxTSI9H4yLOv7x4+OekP7p0c1KHoj4etZiOQ





tqb0m39Rk4dVBsOaDFlKj3ln63fQKbymVlYuqUNwQF5pRRBbMfEmmJUnhqvS2rp2QgQClbM+





RZWGOi8mYzt6MDqYzGahxCofGGuVDjRqZGo1krwwza6hs0erfUAvLNFKpZAnIiHwZF8ZG+ST





JgcjFZfhZpMIB8eaytC4c4iw+HNXg2I9A9nl2tVI2cW5OJMnfIUefa7i+WUcAOCZnbMSlQUn





UXrvUEggoSL0NV8yPs95ogxwiVB8F7zjklEK53nH3Oo8S+9ayhOAxfOoF4dj4MVgX54fwTut





JNk6aqdtLPVHb7wysVnuqklV7G7sGjv+gz84qgotBBJ5FHjubYHX2MecccDKJl0XMkudMp9i





+ZxLurYWQYBHykuQwABiM21a6eoy6zab3lkRJOzIoJcQlgWYuozCwNQmq2si5ZxvtRNjoNWm





J29VDHINiM/wdt4jPL+M170KACbh2Itvfb7b3pjUVdTZmp2+237pM/sP3ourSghZkpcXdciS





f1jtlJXrfsfibdcSvZgBZcBiUkwf/JW1xe+3eK6B49790w+/tMtMVV2zACARKOUIrPfi6nzQ





ebO9bHn85bFjbeGC3zHf8yocgcW0gqtDM7NAQWQBsKoZqZz62W7UcS6ZYOK4dEZX1hEzMsP8





DWD5p/BCCJ37fGeCBc48kAU6AArwTCQEuiDRzhCLs7WMiGyAtEcl0auGSm5u7Hz57rc3e0lW





DPoFh+OwlTSVMzXYBy6TyPnRbKvbVVqRsbKhOkFI3RJcKXHH+PG5b/8ScMz1y/wVEZ3z1z80





ffhOz45ibOJH/61H/UdV3Mje+NLuxs6soApILrXGCjrOZvuKOLBs1lFjxf/WCCE7moEJ//Vb





4PdvPL9SZR7v3jsSSoJc3HM9eQAIA7+2yWUD7HJvyLnKv4Rm4XNrl3dSWCYuwlJa8yKXERZZ





Bks1c/YWqp5px8GdRwffPLx3d3YopK1t5WNrTHmGbby66M6umUuLNKfxa70tc23ivfOe7Cf+





diCxKusCzq0lImJwRHQ6bYhA5vXs/tEDRBAizHLoqSBSwpucTO2JslnmnNM6NOAZKEwjQJ5l





UyT23k2q4yvlCa8prPmnm79674NAP3xnQ0DpWd54yY0eb9ajnUlu0tZwchzozgz9qkf2TIws





K8iY+jxqLAXi0hBFZgDe+8i0Nu9+D+3smYvnHTgAwBsXYuCcM2SscwCqlcIyR3AR8x6GtZ2+





I3ZcfJcrsIOfjh3LtztLA1tLP58PUFm4D7Ck0AussUJXVVUQ2M2kuSkbVge/+YC+WsUaI2aC





MwS6HB+Qgr26wMgDMUmUND516NtSq0t9tATCS1RHpZgRo7cUya/8WeOPfqvutFu5J1YqN2Yy





yyMd3Wi1Ao9kXVVa5+vRdDiYDEpTuMofHU8Fi5XPsrJdltf4+WAijJBVsneAWCBoLd2738Du





1qFmaabNchhilG/d7s8n6Lj0HazsjEV5iRqLU7VuiyLy0Vu9762RPXPxA+CAOw/7gQykAKGU





Ya8FBGoNF5aX+dmQr+Wa+a/5tH4XKteA4gMh5rvHjjN/lIkBmJTW823O2AgRMD2+G+og+Pjt





F1kGRxx8fb8mF9X9/Mc+2zBkEM+OxMuHwi1fnkY9LkANMNjmLk1PwcPMXsy/nN//g/QGbPVm





wrhihoMx2SztbSTTKoqbrcE0n+XlRz/8Q6ao2EIjansAgewsaBElGE3K+uGTk3qyJxBWjGc5





bmYJFKtPM7/Uidjm0Dk6eRAxCfbeeQdssiEcHsritLN9u9x89V4QyAW2XZzCh2EJwXzGNRbn





YPkuLObfhovL6r3vsZU9a/FcexyrePf+0Ydubb9/4HTgg07tyAE2Vo8hQBArwnp+PNPcAVhO





CfrUzldes0U+aPQ9AKzld8BV42gBEJl8GIWiEZqhlbB4yvX8EXBSqGzgq42ofkV97RtHdx6n





qKdO041P4mH/3Y3Nl2cDCWgXF8U85XRFgRCXi7xwftcXV38vOyU7zY4/fsMje7+cQGQViCTC





ht5ptbDfiHRWzd77OgRK7T+uDx9IiuzHfqrDePo7r72mlbq5e+1WytYp45yMA2trNoRWvnDt





2mv/Cldz//LafwBAlESLVSsQcVTWo5bywEwOgIkA6zdfe/XFT90bjeT2C9WwwLIs1vp0+Tz6





rKPGGmozADARIQIBt64fT/e3/pLt7BmKhS3/g/jEq3uzqtw/QmJqdf1spFYgIaSgc+kDF6jH





uheKF7fBSzVnVWvHOYdH6wu4qpiXAFEweMEqjlxh5LlJQ5ABlIicrwWLIEmrupDCNzdk78ZR





I2l4w+98LRWolvQJl/9wmTq+6FJZVq5W4GoXKaUDBwQS5dl+y90BwaMNON38SBxcd9vqYLTf





evJOMjcwhQBLPmpG114ey/BEIv74yzcHpwe9bm9qbM6umPGsmr249dO/8xsPpFTMayi6Fkop





65xAXLIPrGx9/VXTvxfDnIEQAaAH9+HPPi4K1YiAZV0VSVEOjr754bPukRWFOocgsESpFelY





SBWMKiqDsrpz8QM9f/EDqbKI1+8cOOc7G6UAkY00Iq+uXvJ0vu1esEvPboUMq/GlV4uR81VX





+qlw3l/hVcW8NEcwDBRZJ6WAtWwkAEYAzwWh5KCu65EUREydnu40ErKlxACQVk96WrM7aE2P





0JpGWS8RMyupjDFaK7GY9ZPOb0XMrFizdj5W3WgaquDuu9r5mtgxOE9eINisLmairsmW+W4Y





5LZ678lh/3Qs69KjePLmjd/9jSfAMZPkM1/jXFKJ9x6BiTwzCwHOi5c/OevfbwtYZHOwIM/R





xkcfHt9L7/35bSEdCvvOn/UGr//oUpicabHzqLHULCtfY3kSHAmuoitg7LmMHzCOc3HrVu9g





/4p6FMCEF6nGIgtj3nO6rJrX8dkGq9pzNd8F71jff1WBQkgCJ+epFLgcjocrQgLL3/OxpKjk





tZdnna7PpiZO4nrSfXJPgjAwf6jtVdRj+bKqWtYBAqCUwnijhV5nGovtl/sG7SC6rW9de/fR





V3ezaYOdWXsvkKhr5L0fkXWYddWwG3odacHyvbvtk3eB2UoIAWg1Y+OFWJEFxLmEDK6/Wj54





0whUfEYlqH07O73bUdKJJPv4z8yqMnvnSy+AI4SICJndGmrAwjZe5nqtpMq8TqVgcy8EWVeb





+vHlj/Qcxg88jvPhubUJ09OL7ZXPPdrk/Kq1p6XAstF9wBSk69lfeK5mfTO4wvKYZ3OQdwKl





VCiErY1EsXxjALgwkxgDOLIRk7cGwQfGZNYjcReIF5SKz2WU4TInbeV8XAgU4JwLdTi//6+s





kEWiPCAAk/ezLNxrV2Gl/r/2zq3JjiM5zJlZ1ZdznRsuA4ALEgTJpRfaNb1erdZ2OPzmsMP2





k0P6YX7xP3DoyQ6FFJIjFA6tbMmSdy2K9O6K5JIECSxAYGYwl3Pv7qrK9MO5zLnPaWBIgsv8





GEF096mubsygv5OZVd397JGN7vT5KVhEYRoWYnzsK/388O9jj6b5e7hd33N40G7devYhEQDA





8EFKOJycAVNl6cnIEowucmbmtOk//9AiBBaeVImNjbeq9VNyIlJ0au/91CTmujgBMYJuUvId





K4JnM5TRb5GAvITKlV6680XMydEnt9QaEzRVmeHBo9M4CjObEHk0mjGa2zn5oU0u6ck31WSn





6SRk/Of6nGW62XzaMr3jcOITCxMhAsr5VQ/jK+t8mUzM0O+1qnnR3r2SAvOzJzVEJkjPh3hl





nPLD+Vfw8PKbylZ49H9mAAkcmGU8HWTyTDAR4YILImvswGatAoof/EdX293hUGMJw6EMFpYs





5GQCB5JGLcrP2Pzd/7r28/9RTB75cf7H7EgKh8mIqQwTFhbuZy0HhYiABGYPKGiwCIPPfuGG





LQlZsijrMYjUmoEDTmcoPPqbjH68hHb4KwiCWOnv3zuxjaf7O1cP73+HE43Nz9FUZQnWXsGp





en4gMrbAwg6//skU7OPxhM/zWGPMYjIytXFdzgIz/awolwoAGYOWDJEvgvDokRiwkKdM7Yos





ghAMxSKIxqEYARynN7PZyvmmSU10/j9ATLfi/ukgoliGs1snRVzDxsXJ9dYbb3sfF+zh8NnV





gw/TGhQBaCbzIt7Z3dp//ejv36vGOQMR8qyyx1ae3Kg7PAQhTu7oFxF6veMeJJKniADGC4DH





nnWVsQzGBU5BAEIqONCMHicFIgDBgiBuvnEi0IutB7E+cBpMu3DtR7eyntZEz1FxLCGK9qa/





6Rn4jVeiPrsnB4IISZ13KvTFEyQw48tzqTjgS3IHEMFwcvd5XrHUHZMNOLuMkxxo3h0w75Hp





ysXkv5hiF+dSIA0LJVOHQxGwSbo7iLfD63dz9M5L5b3/uRdjLmHWaDMDrLJs48yPRMbp4ihv





YytGXr13FKKDs95bvUeFdJJg0CZt36tOBYIiAgy5CKPEiATgoq1+floXESHAwAIAZBkHZKB5





M280uj1X1IzNhfPjgU2rR59dzQZqjRm0xrEE544jez41kICCd404ttccWnAeYyAinHrB8HTQ





cdHyunrH7C6zJY/JCgtHNuLAIDx5xcv8jA8YT8yYeV3LcHnhIWMi59Wb2arHpJtJJQMERcQ5





16xv9VsDnEwpG/ZAIC70j6R7WE1Nvr3PN69WvHQrkDrxoxttljFdYVjYOFk9z+NMYurXnnZ9





O5Zou/qbre+2P/vgx0mWuU4D0U97AwAIYxFgU4hL4x3sPNuKjQeQ4DxSRHbgg4uIG7ef1KoA





PubMsw1goiS988X9UOj46wIacaxk2h3Nq7JlxZIBFAbpZnnmol7LwMw36PK4A5duXxJ3rGp5





vjx5LAcjIAshnScqM3HHpDNcWF4RX5wvw1zach68jNswIBIDBgwRIiGM75Ib9yCIVkKw9tV/





PriSFl88unn6YFBkjKPy0OguvoVoA8YbZOn2yU8GgWuvnDavdIpMWIIbRFn/jf4XjoyH6UgD





ACUEIpQgYoQ5uXbKjHJ8y3FHDIaAW6984pzvffFm3HjavDbAnkQRXbl1pbpbffTZo0e/ugbA





Wf7ZqlP51qIRx0b0WrK3H0kIEVpgSbDoR32AxtQXNKyKNaZbrI07VrWE6VfGwbDGMRwLGEUM





w6GQJU9Ln4s1AGA2vlhchvNBGRCYCT2G3hIB5OCtsSwkJliLwXsSmu6BxQQogOK8U2nJSaX5





fuOH+OTDN7onJOwNkTDPPlF9nlVOMYg+GdT387TS7ncLgDhJK8cf7jMP0EwbY/w/qlB6xP0q





CAcwAQbGcMsfR0kAllv3TgsXucMmcGZqPSvp6z+66bPQ7/cOnxw//ofrgJxlao0laMSxjvOg





A/HGFW9R4ihCooPOWT+rhGyi3fm4Y3wDy9JoYmkVo2TcgQDn74taqHHMxhrThdIVscZU++Wr





s+c/tAsKYHr9VapUW5/+YidKXBRSBx1AFB9JlKGvEPo3/1Wv/eyQIWk2ff/ZzWcPqiEExI3+





4S3qY+eNU8/PUCocPCPvbdcOHuzlpyQzuhinOOijrYHrxVBEIuIoNG4ctn9zwxqglGvNgUSD





pAZ5t+MH/tU3t3ca19vtbm9w5gM/fHdXDGbZpxee5LcTFccFTNyxf91f395u93oocpYPMnC9





4+bU+8ln3DG81mbdsVEFdLGvRXcQ4nAwYF5Gs3nKqlxjpv/pXWA2W1lcHZ/DqF8kILKVfuXa





QWx5q5kWEuUnW88epDCad4vpddy99XmREXufJHGe55HZffLBVUthUQpIOP9kQEAgYBDDoeDo





O+889S4rikCBPAeKQhrbpx/c5eBocjMRgIgQiReqXDnyrdTnNQDxzPF2K7H17hHlcZGGClq3





f/dZ+yRikfbBNeTsxp3CVo+LwnXo7f5HXbXGGjRV2ZSnB7ZRKSKyjNJru8ApymxuMpWznN87





VrICOklJzi9mmKQYw5wFeDJ5A6dnmk32HucaKDC+A03GZQicVA+mZryOHhcE46xkbhWm1CLT





J8TsedC2yc616vZx67QfmYZpPIHmTWhHwzZnZ+7mjUpSx0G/yAYZi4lsC3H3PCgAAAACKUJU





udIuTo3kVWODjKZgefCxMBaxv/POw7MzHxgILYAYaw1A4ai+U826LmRdFkivmSh+FCVS34L2





MZ8+3GWfADCIIEnop31OCSFygEji4sPHr/hTAcwM5IIYNzoSatWaO3xPrXEBGnFczHSV9I1X





zMC5Vuj2T5sccOYbc+qbG2aXyuQs59vnupvdsDa1mU1Vptq8yCrMbpwcBl3w2zcqJn4c2Yxt





2N7Z/uLXW0W7TuRcGLz+TjcIe1+YSIKnxFQ//+V1gw5GoQEFZhQMAII9hER8sCYeHsw0MWk+





294D77PghhO+MHhJ09j5QgTFMsFrh5/k1tutW31J2/2Dnew0hXiAkoJMbjIyYnuADEVqts8q





1ajzuIoQGwzmbpp93GWCG/cePHn/ztbNtNtx/eNfgbIWnTl6Mc6fTJY/eRSQEAHCdHVvGlmy





NLtVljQYT68eb5fxHwudLJn4MDe79PyeztHspskNGWtXYbIK06syaby4EREkYoPYP/Kto23v





k2oSWZFrdx6+cq/rbc9g9emzayCZy/uJSZhD64QA/KgTIxSTB88IRoBCrV7fAoqjGJkDA+y+





+uneDe51WsLAIsCIyJWqKVwfkeu7dms7kYGjEKW7veOH+ycf7SMFiOXq7R4we/aUclTvgzj0





Kfra1p3HeWu3/ThFoqieNV87yu73SPjevf7T9+8Q+b36r9Uam6ARx6ZM4g4Bubo/OHpSR+TZ





JvMFgYWViyqg8EKhx9zO44PMVTGmm2GZ1bluRpsLyq9drZ0dSQi+cvOsudNOi22xBUQ+qWx/





+kmdk+5br1LWPagmjYFz/ZO9ToHhuCpYGBg/MWk0K1QKhtp1V/SsFXfzDd8pnuQDk0QkDMOJ





IlGEIuwd7uzHXFQ+/ZubSB0G37ze7x5Wku3B1s2j3sHdrMiyXj8MdiILSAENmspZJa2ePhPD





MUuo7MnguOIbhW0ziwMAQkvIQTjLPgflIlQcJTjPWYQEA86HHMurn5fjjvm76+Yrpiu6hTX6





mN2ytko6br4oEQLrxZOhQBRxTLWTnf1WHBkQiqq5TeqHh65/vHfjFlWqp3m/1fbQO3rNH1fE





ZsQzp8oir9zjPBwCORAIobC2kg08C1gkayCKoxDYS7G9W/WQnz3Y6x5UhJPma08s9DFy5LcO





7u8CQXCO0IiArXXqt06PPrwdG2AJIADIIgJIhiUMi0AiAODJmpDrlI0N0VSlBOc5C/JkXsRy





FiZALi4vSUNGi3Pbx0OMM0/akqnMRRYyl9keZNinyGTHqS0zCcjU6soMRWY2sngCAObXfvCb





IAPfjvNuswhFo5GQr0mfrzSLrSuHGQ0+O46Qdt68vp9uZQ4KDGHU2ej8hWHg4YExZK0gYJIk





zH54YRsiG6HLvHhpNkzeKxKS08fsCsG05bmX57kxmHUqIl64j4AiARDzLDr99S1LgSUM/7qj





t0CwDyAIIhyGad07b56qNTZHxVGO6XrH0hLHOZu5Q5a2EVnQx2R6wrKqh8DC8WTeMjLsYrk+





YLk+YKqiMV37gDmJCMGTD3bTihDZQZYwZEfHJ06ygjMyqcEsCmevNFtXt7JfPjGVeiwkAjh+





EdQIgNgVUX/QlxAiK0hcbySEGJOkKSCFuBaYXOuU00r60V/dREkF7d5rR7WEO0e7eV5w2gtc





lVAfn5+nEOFoPEgAgIMxIQGR0eMgIxjOl0vvJD9//2D971OZRsVRmml3zLI8BBk/SmxJHDFc





ni9twrJPVsYUU6GHCGyiD1iij+kAZlof57eHiKzZjg5CVun1rdWJQwAADJtJREFUpVKnXqtD





oQaALg/es3NZFCVFLpFvdnPT/dSdtjOBLAgykAgOe0MmiorcOQEffNzt5FKYos9btTitGSd+





UDgvErw3SfLx324TkaDs3H7ILhTt3cZOkFDZ3h2k9TPmweTVlRPBAQgZRAsOBsOfRnq3AU4E





xNfk7INv4xvnXwQVx/Ow2h0AAPNJDM9eludMu+PitAVnV5fvcq6P1aKRpfqYzVzOP5/euNIg





DCEEZ6202iHi+pNPt1EaQYpupwgOCUjEOdNqdRqVW6lLaklzx2zV2RIiDyOCArpMqcsbganI





fGIT9r6aWrCc5+IcuUIIY7IcfBNEOARgOHu81WzGh1/Yk6eGXe3kKAshMcaMf04CIIBiY59W





bfAB2KEIETR/J2T3ewIClvzx/XW/TWUZKo7nZI07knSmjDh+t+JwbZU7Li55DKdxXRx6nC+W





1Mdi4WM6W1llkOHuKATAWXTzdz4LnFhfaR+RFHG1gsag90WcREXGttqvps9CUdjkjOvtKKqE





SsRpJCGPo6rvF1m7ulXH5jYjZUkFe0X/6KSTuWAjV7FogmAcn3xsx8UXZwiACYCsrR1+Xmv/





5juce5YgEgQ8YmwiZHYup0HPBQLZiugWvv3dg8GDCNhjM826n6z6PSpr0FGVF2J6btgYFGBA





BsEoQu+XqnnlcMzKoZb50ZKLRlLmF9e0XxiFhblhl7ntACvGaAGrAVuRjdl5B/7WaxnYjAzE





sWUQRHSFp6Rydmxq9YGJbz1+JDCwjWZ/cFoJeWHRNPakceOs16kUxUm9gf3TyqC9jfFhpWpr





u6dZa79wUe9g9LRUYbzzztMnn+5lXSIkAeEgiJFABoBxmsdR2uugiDBIzNGVn7jbWZtM472H





oXdi8PZ2/tG7oDwXKo4XZdYd8wVTgYCIyyK71Zf6qrta5j5Z6GGhWLs4r2S1PiY3oEy3XK+J





BQGJ4ShQbvzWDvae1bZeP5VwGsfWWCLLAMAcJdgNceqqic+bg0d1H/sYQvsp2cwzYRAxADkO





IjFWUkA/usEXTQiBiIZBEhKyFCCG0KApQNLCOQRvsGbiri/Su/cOP/nVdcRc2AiIvbm7/+Ob





1+6/m5r0r0/SyOCP7lZ++sf/Z/4HpmyMiuMSWOeOUXKxdARmjTvKzfJYsddsm4v1gctNMZ11





zXU72+dwTQBM4oM3wUevvN1yec8k3loLgCRMhtC7TvP1w6e7Nmc5OgPJCawww8qn/IxPQ6aG





lszwYSRCBBwAkLavnZwcVF99q3h8P/XswI9eCm1vwdt77YiTerXuPGNw15rxH/75t/rNry+O





iuNyWMhZll3CS3hxd6zfa6FNCX0szWiWnsGMT0IIhlIyAQSctze/eyrYJSJmria2mlZcPrjP





3+cDH05ODYf1//4QAJFZSBhGTxsDIEM+5AbTEIqkGpJap3u8b5Jnb/3A/cPP9qZ3J8B/8k5R





MRGARWAASSz88V9qXeNFUXFcGuXccR6FbJ59bO6Oxa5gmTsWV5boY+Vp4LI/pnYTEEICsNVr





eVI7qiaE7Heb9achOnj2FrXb3OoIMAoIEGAwZJg5BE6S2OWO0KBIiBBDjNgJiBDARCy+hpTZ





SCrXTlsP9wPnCEmtOSgGFe8LRBofHQDgRz8IiY2KIidjdyuV//ZTHXa9HFQcl8llxB0bu2Nt





uRRK6OPCfGfxMUUr9p3XmSRJXLgBcFUgr91uEVumXed8cNyoZkXPFkXU7/vENQJlaHp3v98F





v9cdQG0L01hODuDxb2zkWQBFEIR2bp5GRg4eNl6/ax4/DnlOxror1+LjQ8/BwNgXIoAov/t9





t1VJ+x6bsdzebf6n/6ql0EtDxXH5LCl5LNcHjv+dl0o91l+rm4QeSz/ZxDvLY42VjefOlI1Y





L1tyY/8L4yIGMCYqeJBG1TzPEU1sIkCHCGTQh8i5PhK3TnZru+2Dj+tbu9Ggz64fCzD72EQ5





QjR68OH4TmVBYCEkB2T/7e8ai3DSc9d3GpTLf/mLXy77OSjPiYrjS6F86HGJ7oCFS3qx/6Wf





rNLb6qPPxR9rdDN8kRSAIUsVhygiRVqBXmbzPhqgOGGL6Nj7LCayIg4wIAH7BuIAJQYTJIzf





UC8e0Qy7juMkL3IQCFZuvxW+14iPi3w3qlQTcI44hD/6K61oXD4qji+Lr9QdSzrfZMAFNtfH





BeewxCBL+hExRA6YhM6n4q84KyQcPkl05i1NsmzFiL31/fg7UZeidDeNCnBFxtebjafHZ3/6





tzor9EtBxfElssQdsObCnGqzZPk53LHkYOvGa2Ejfawtu1yQOq3teuURZW5NhscZvckxENz7





ntumdK9ZB0SRIsvkta2t//ynOk3jS0TF8aWzQeix/rt3zZ4LO65NW1Y3WrXhghNbWSVdfiaT





saSNKsbzO84dXABTCdvRP7vB29VaTOA5CAJ49M79yf/+delDKGVQcXwVfKXu2LTzDfQBF17n





i4JZeXpENL5RdQ3rnSIAgICBOLkC927HVeZmtYIAsbUuy6s2edzq/sX/1ecMf+moOL46Nhht





WXXZrHHHisrFRinCohXWOmL4oZxvWNxxdS32wpPZFDb4k+9JvW4kEBmq2diCEe9auevl/i/f





/ey5e1Y2R8XxVTOvj+dxxybBwnO7Y805wPBl14aIl/+zWVojKeWINY1FAAzR7/0w2UviGLAa





J61+L4niXp67IH/yNx+WOZDyQqg4vgYuwx2b5CybljxW7bziNKY+XTWAezGb/p1HE12Gb35D





+P1/uZdng91q1bkQGZOFcJLlvjP4s1882OCgyqWh4vjamNLHhtHB/MaN3LFxyWPFB1+qPi7e





BcfjKG//0xv/OOkHQBAoQrAMsbVng/4f/UynaXwNqDi+Ztbq4zncsXSvEmnLiqNuoI8LW23W





YrH1H/yLnYLZeVdPU3HhqJc3IkOEf/jXH5TqTblEVBxfP2szl4vdcaklD7gEfWy+x9pGFIV/





88M9lw3SJMHAkcGjXlbkLk0TBPjvf6eBxteJiuNloaQ+FoOT5ws9LqinXI4+NoYJ/sNPtjq9





PHO+FidV8X1XJFFa+MAkPnBM9s/e1dHWrx8Vx8tFGX0s8cJzhR4bTSF53nBiI4+I9//+xw2O





qpy73Oc2il3eZyCMKGIxNgIQRizY//nP1BovBfq2+pcL50+m3CHjGU/jVYCpS3HxzloZb5pr





M7uXwGyr5RMz5g4x/fWCC5+uZt3XEgH+6x81GnEExuaFiwgPB11CJCIGIiIj0OVAbPqu/bP3





9L0nLxEacby8vFj0AUsHYy8lf1nWqGSGwvDv3ql1nK+lcRLZHgef5bn3Amht7L0HCRSbjg8/





f/9huZ6VrwQVxzeA1SMvF+vjgptiL2j7HAZZtW30vgYAsBG883Y1DWyNjQ0wSO6KzsDXUugN





gEPoevfh58crDq28FGiq8g1g+A6XyO6WT16GOyyGHkuzjM2zmJndV2cxSxIVQuOKooseEsAi





P+oVtcS0ezmRzTN4/6MnC3soLyMqjm8Mk1dARXb3uWofS/WxNKaQ0cMycLolLHa7uPHCO9gY





giHb7wwquxVvfWM7KXL//z45Wruf8tKhqco3lWH+gkSzv8H1ycvSqsdis6ntm87+2Bgk9uHN





f2TyfHD/47Vv0lReYlQc33hWVECmL+8X0cflTiqHotBk5LcBFcdvG2OPrCqjni+UjD4W+9wM





4cIdlt9NealRcfw2c5FEZrZsNvd0avuScsfk5QTgVBa/1ag4vkVEdrfUZPbNVCIAoDHFtw0V





hzJPFF2ZWpvTDKojFFBxKIryHNDXfQKKonzzUHEoilIaFYeiKKVRcSiKUhoVh6IopVFxKIpS





GhWHoiilUXEoilIaFYeiKKVRcSiKUhoVh6IopVFxKIpSGhWHoiilUXEoilIaFYeiKKVRcSiK





UhoVh6IopVFxKIpSGhWHoiilUXEoilIaFYeiKKVRcSiKUhoVh6IopVFxKIpSGhWHoiilUXEo





ilIaFYeiKKVRcSiKUhoVh6IopVFxKIpSGhWHoiilUXEoilIaFYeiKKVRcSiKUhoVh6IopVFx





KIpSGhWHoiilUXEoilIaFYeiKKVRcSiKUhoVh6IopVFxKIpSGhWHoiilUXEoilIaFYeiKKVR





cSiKUhoVh6IopVFxKIpSGhWHoiilUXEoilIaFYeiKKVRcSiKUhoVh6IopVFxKIpSGhWHoiil





UXEoilIaFYeiKKVRcSiKUhoVh6IopVFxKIpSGhWHoiilUXEoilIaFYeiKKVRcSiKUhoVh6Io





pVFxKIpSGhWHoiilUXEoilIaFYeiKKVRcSiKUpr/D9V5SgojIdNXAAAAAElFTkSuQmCC





", extent = {{-107.023, -99.075}, {107.023, 99.075}}), Text(visible = true, origin = {-1.855, 67.166}, textColor = {255, 0, 0}, extent = {{-61.855, 37.628}, {61.855, 68.04}}, textString = "%name", fontName = "Arial")}), Diagram(coordinateSystem(extent = {{-100.0, -60.0}, {100.0, 60.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
    end Earth;

    model SatelliteMass "Models the mass of the satellite"
      Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(visible = true, transformation(origin = {0.0, 24.0}, extent = {{-16.0, -16.0}, {16.0, 16.0}}, rotation = -90), iconTransformation(origin = {0.0, 100.0}, extent = {{-16.0, -16.0}, {16.0, 16.0}}, rotation = -90)));
      Modelica.Mechanics.MultiBody.Parts.Body satellite(frame_a.r_0.start = {12742000, 12742000, 12742000}, m = 450000, r_0.start = {-2462757, -770950, 5824955}, each r_0.stateSelect = StateSelect.always, sphereDiameter = 1100000.0, v_0.start = {1, 1, 1}, each r_0.fixed = true) annotation(Placement(visible = true, transformation(origin = {-0.0, -10.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -450)));
    equation
      connect(frame_a, satellite.frame_a) annotation(Line(visible = true, origin = {0, 12}, points = {{0, 12}, {-0, -12}}, color = {95, 95, 95}));
      annotation(__Wolfram(itemFlippingEnabled = true), Documentation(info = "<html>
<h3>Satellite Mass</h3>
    <h4>Description</h4>
    <p>
This component models the mass of the satellite.
</p><p>
View the <a href=\"modelica://SatelliteControllerDeploy.Components.SatelliteMass#diagram\">model diagram</a> for this model.
</p></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Rectangle(visible = true, lineColor = {0, 0, 128}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 2, extent = {{-100, -100}, {100, 100}}), Rectangle(visible = true, origin = {17.53, -72.731}, rotation = -30, fillColor = {128, 128, 128}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-85.306, 50}, {-15.306, 54.625}}), Ellipse(visible = true, origin = {57.631, -44.462}, rotation = -30, fillColor = {192, 192, 192}, pattern = LinePattern.None, fillPattern = FillPattern.VerticalCylinder, extent = {{-90, -45}, {-50, -25}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20.655, 30}, {40.986, 70}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-4.003, 60.578}, {9.395, 68.789}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-4.003, 50.844}, {9.395, 59.055}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-4.003, 40.867}, {9.395, 49.078}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-4.003, 31.422}, {9.395, 39.633}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.935, 60.672}, {24.333, 68.883}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.935, 50.938}, {24.333, 59.148}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.935, 40.961}, {24.333, 49.172}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.935, 31.516}, {24.333, 39.727}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{25.872, 60.665}, {39.27, 68.876}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{25.872, 50.931}, {39.27, 59.142}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{25.872, 40.954}, {39.27, 49.165}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{25.872, 31.509}, {39.27, 39.72}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-19.128, 60.609}, {-5.73, 68.82}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-19.128, 50.875}, {-5.73, 59.086}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-19.128, 40.898}, {-5.73, 49.109}}), Rectangle(visible = true, origin = {17.035, -70.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-19.128, 31.453}, {-5.73, 39.664}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {160, 160, 164}, fillColor = {192, 192, 192}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20.655, 30}, {40.986, 70}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-4.003, 60.578}, {9.395, 68.789}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-4.003, 50.844}, {9.395, 59.055}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-4.003, 40.867}, {9.395, 49.078}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-4.003, 31.422}, {9.395, 39.633}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.935, 60.672}, {24.333, 68.883}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.935, 50.938}, {24.333, 59.148}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.935, 40.961}, {24.333, 49.172}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{10.935, 31.516}, {24.333, 39.727}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{25.872, 60.665}, {39.27, 68.876}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{25.872, 50.931}, {39.27, 59.142}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{25.872, 40.954}, {39.27, 49.165}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{25.872, 31.509}, {39.27, 39.72}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-19.128, 60.609}, {-5.73, 68.82}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-19.128, 50.875}, {-5.73, 59.086}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-19.128, 40.898}, {-5.73, 49.109}}), Rectangle(visible = true, origin = {-84.641, -10.95}, rotation = -30, lineColor = {5, 134, 255}, fillColor = {0, 157, 236}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, extent = {{-19.128, 31.453}, {-5.73, 39.664}}), Polygon(visible = true, origin = {57.631, -44.462}, rotation = -30, fillColor = {192, 192, 192}, pattern = LinePattern.None, fillPattern = FillPattern.VerticalCylinder, points = {{-70, 55}, {-85, 55}, {-85, 55}, {-85, 55}, {-85, 55}, {-85, 55}, {-90, -35}, {-90, -35}, {-90, -35}, {-50, -35}, {-50, -35}, {-55, 55}, {-55, 55}, {-55, 55}, {-55, 55}, {-55, 55}}, smooth = Smooth.Bezier), Ellipse(visible = true, origin = {57.631, -44.462}, rotation = -30, fillColor = {160, 160, 164}, pattern = LinePattern.None, fillPattern = FillPattern.Sphere, extent = {{-85, 50}, {-55, 60}}), Text(visible = true, origin = {0, -186.897}, textColor = {255, 0, 0}, extent = {{-81.356, 49.492}, {81.356, 89.492}}, textString = "%name", fontName = "Arial")}), Diagram(coordinateSystem(extent = {{-50.0, -50.0}, {50.0, 50.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics));
    end SatelliteMass;
    annotation(Documentation, Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end Components;

  model SatelliteControllerDeploy "Model scenario of the launch and flight of a satellite."
    extends Modelica.Icons.Example;
    inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.PointGravity, nominalLength = 12742000, g = 9.81, enableAnimation = false) annotation(Placement(visible = true, transformation(origin = {-110.0, -50.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Forces.WorldForce force(animation = false) annotation(Placement(visible = true, transformation(origin = {-45.0, 5.0}, extent = {{-10.0, 10.0}, {10.0, -10.0}}, rotation = 180)));
    Modelica.Mechanics.MultiBody.Sensors.Distance distance(animation = false) annotation(Placement(visible = true, transformation(origin = {-80.0, -10.0}, extent = {{10.0, 10.0}, {-10.0, -10.0}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Sensors.AbsolutePosition satellite_position_absolute(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) annotation(Placement(visible = true, transformation(origin = {-45, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Sensors.AbsolutePosition satellite_position_over_earth(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve) annotation(Placement(visible = true, transformation(origin = {10.0, -10.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -720)));
    Components.Earth earth annotation(Placement(visible = true, transformation(origin = {-22.5, -50.0}, extent = {{-22.5, -22.5}, {22.5, 22.5}}, rotation = 0)));
    Components.SatelliteMass satellite(satellite.m = launchVehicleMass) annotation(Placement(visible = true, transformation(origin = {-65.0, -35.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Sensors.AbsoluteVelocity satellite_velocity(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) annotation(Placement(visible = true, transformation(origin = {-45, 66.518}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    parameter Modelica.Units.SI.Mass launchVehicleMass = 450000 "Mass of rigid body (satellite.satellite.m)";
    parameter Real altitudeController_k = 10 "Gain of controller (controlLogic.controller.k)";
    parameter Modelica.Units.SI.Time altitudeController_Td = 2000 "Time constant of Derivative block (controlLogic.controller.Td)";
    parameter Real forwardController_k = 0.1 "Gain of controller (controlLogic.forward_controller.k)";
    parameter Modelica.Units.SI.Time forwardController_Td = 1000 "Time constant of Derivative block (controlLogic.forward_controller.Td)";
    Modelica.Units.SI.Distance satelliteHeight = distance.distance "Altitude of the satellite";
    Modelica.Units.SI.Distance targetAltitude = controlLogic_fmu1.target_height.k "Connector of Real output signal (controlLogic.target_height.y)";
    parameter Real forwardController_yMax = 1000000 "Upper limit of output (controlLogic.forward_controller.yMax)";
    parameter Real altitudeController_yMax = 1000000 "Upper limit of output (controlLogic.controller.yMax)";
    ControlLogic_fmu controlLogic_fmu1(useTriggerInput_ = false, step_ = 10) annotation(Placement(visible = true, transformation(origin = {3.21, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  equation
    connect(earth.frame_world, world.frame_b) annotation(Line(visible = true, origin = {-72.263, -50}, points = {{27.737, 0}, {-27.737, 0}}, color = {95, 95, 95}));
    connect(distance.frame_b, world.frame_b) annotation(Line(visible = true, origin = {-94.006, -30}, points = {{4.006, 20}, {-0.994, 20}, {-0.994, -20}, {-5.994, -20}}, color = {95, 95, 95}));
    connect(satellite_position_over_earth.frame_resolve, earth.frame_earth) annotation(Line(visible = true, origin = {6.667, -40.033}, points = {{3.333, 19.933}, {3.333, -9.967}, {-6.667, -9.967}}, color = {95, 95, 95}));
    connect(distance.frame_a, satellite.frame_a) annotation(Line(visible = true, origin = {-66.667, -15.133}, points = {{-3.333, 5.133}, {1.667, 5.133}, {1.667, -9.867}}, color = {95, 95, 95}));
    connect(satellite.frame_a, satellite_position_over_earth.frame_a) annotation(Line(visible = true, origin = {-43.333, -15.133}, points = {{-21.667, -9.867}, {-21.667, 5.133}, {43.333, 5.133}}, color = {95, 95, 95}));
    connect(satellite_velocity.frame_a, satellite.frame_a) annotation(Line(visible = true, origin = {-61.667, 36.012}, points = {{6.667, 30.506}, {-3.333, 30.506}, {-3.333, -61.012}}, color = {95, 95, 95}));
    connect(satellite_position_absolute.frame_a, satellite.frame_a) annotation(Line(visible = true, origin = {-61.667, 18.333}, points = {{6.667, 21.667}, {-3.333, 21.667}, {-3.333, -43.333}}, color = {95, 95, 95}));
    connect(satellite.frame_a, force.frame_b) annotation(Line(visible = true, origin = {-61.667, -5.133}, points = {{-3.333, -19.867}, {-3.333, 10.133}, {6.667, 10.133}}, color = {95, 95, 95}));
    connect(satellite_position_absolute.r, controlLogic_fmu1.position) annotation(Line(visible = true, origin = {-22.395, 40}, points = {{-11.605, 0}, {11.605, 0}}, color = {1, 37, 163}));
    connect(force.force, controlLogic_fmu1.y) annotation(Line(visible = true, origin = {7.039, 22.5}, points = {{-40.039, -17.5}, {15.934, -17.5}, {15.934, 17.5}, {8.171, 17.5}}, color = {1, 37, 163}));
    connect(satellite_velocity.v, controlLogic_fmu1.direction) annotation(Line(visible = true, origin = {-23.697, 35}, points = {{-10.303, 31.518}, {-1.303, 31.518}, {-1.303, 10}, {12.907, 10}}, color = {1, 37, 163}));
    connect(distance.distance, controlLogic_fmu1.height) annotation(Line(visible = true, origin = {-44.158, 24.2}, points = {{-35.842, -23.2}, {-35.842, 0.8}, {19.158, 0.8}, {19.158, 10.8}, {33.368, 10.8}}, color = {1, 37, 163}));
    annotation(Documentation(info = "<html>
<p>
Model a satellite with a combination of 3D mechanics and block-based control logic. The satellite is launched from the Earth's surface with a launch sequence. Then a control logic subsystem adjusts altitude and velocity via thrusters. The Wolfram Language is used to design the parameters of the control system.</p>

<h4>Design Controller</h4>

<p>Compare different controller options by the altitude and energy expenditure.</p>
<p style=\"text-align: center;\"><img src=\"modelica://Satellite/DocumentationFiles/controlCompare.png\" style=\"width:600px\" alt=\"\" /></p>


<h4>Visualize Trajectory</h4>

<p>Visualize the satellite using Wolfram Language functionality for 3D graphics. The images show the satellite path with and without controller.</p>

<p style=\"text-align: center;\"><img src=\"modelica://Satellite/DocumentationFiles/satelliteVis.png\" style=\"width:600px\" alt=\"\" /></p>

<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at <a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>
<p>For the full example, open the accompanying notebook <a href=\"modelica://SatelliteControllerDeploy.SatelliteControllerDeploy/SatelliteControllerDeploy.nb\">SatelliteControllerDeploy.nb</a>.</p>
</html>"), __Wolfram(PlotSet(plots = {Plot(name = "Distance, Velocity and Force", preferred = true, subPlots = {SubPlot(curves = {Curve(x = time, y = distance.distance)}, label = Label(x = auto, y = "Distance [m]")), SubPlot(curves = {Curve(x = time, y = satellite_velocity.v[1]), Curve(x = time, y = satellite_velocity.v[2]), Curve(x = time, y = satellite_velocity.v[3])}, label = Label(x = auto, y = "Velocity [m/s]")), SubPlot(curves = {Curve(x = time, y = force.force[1]), Curve(x = time, y = force.force[2]), Curve(x = time, y = force.force[3])}, label = Label(x = auto, y = "Force [N]"))}), Plot(name = "Parametric Plots of Position", subPlots = {SubPlot(curves = {Curve(x = satellite_position_over_earth.r[1], y = satellite_position_over_earth.r[2], legend = "Y position to X position")}, label = Label(x = "X Position", y = "Y Position")), SubPlot(curves = {Curve(x = satellite_position_over_earth.r[1], y = satellite_position_over_earth.r[3], legend = "Z position to X position")}, label = Label(x = "X Position", y = "Z Position")), SubPlot(curves = {Curve(x = satellite_position_over_earth.r[2], y = satellite_position_over_earth.r[3], legend = "Z position to Y position")}, label = Label(x = "Y Position", y = "Z Position"))}), Plot(name = "Target and Actual Altitude", subPlots = {SubPlot(curves = {Curve(x = time, y = satelliteHeight, legend = "Satellite Altitude"), Curve(x = time, y = targetAltitude, legend = "Target Altitude")})}), Plot(name = "Altitude", subPlots = {SubPlot(curves = {Curve(x = time, y = satelliteHeight, legend = "Altitude")})})})), preferredView = "info", Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), Diagram(coordinateSystem(extent = {{-140.0, -80.0}, {40.0, 80.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})), experiment(StartTime = 0.0, StopTime = 50000.0, NumberOfIntervals = -1, Algorithm = "dassl", Tolerance = 1e-06));
  end SatelliteControllerDeploy;

  model SatelliteControllerDeployWithoutFMU "Model scenario of the launch and flight of a satellite."
    extends Modelica.Icons.Example;
    inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.PointGravity, nominalLength = 12742000, g = 9.81, enableAnimation = false) annotation(Placement(visible = true, transformation(origin = {-115, -5}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Sensors.Distance distance(animation = false) annotation(Placement(visible = true, transformation(origin = {-85, 35}, extent = {{10, 10}, {-10, -10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Sensors.AbsolutePosition satellite_position_over_earth(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve) annotation(Placement(visible = true, transformation(origin = {5, 35}, extent = {{-10, -10}, {10, 10}}, rotation = -720)));
    Components.Earth earth annotation(Placement(visible = true, transformation(origin = {-27.5, -5}, extent = {{-22.5, -22.5}, {22.5, 22.5}}, rotation = 0)));
    Components.SatelliteMass satellite(satellite.m = launchVehicleMass) annotation(Placement(visible = true, transformation(origin = {-70, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    parameter Modelica.Units.SI.Mass launchVehicleMass = 450000 "Mass of rigid body (satellite.satellite.m)";
    parameter Real altitudeController_k = 10 "Gain of controller (controlLogic.controller.k)";
    parameter Modelica.Units.SI.Time altitudeController_Td = 2000 "Time constant of Derivative block (controlLogic.controller.Td)";
    parameter Real forwardController_k = 0.1 "Gain of controller (controlLogic.forward_controller.k)";
    parameter Modelica.Units.SI.Time forwardController_Td = 1000 "Time constant of Derivative block (controlLogic.forward_controller.Td)";
    Modelica.Units.SI.Distance satelliteHeight = distance.distance "Altitude of the satellite";
    Modelica.Units.SI.Distance targetAltitude = 7500000;
    parameter Real forwardController_yMax = 1000000 "Upper limit of output (controlLogic.forward_controller.yMax)";
    parameter Real altitudeController_yMax = 1000000 "Upper limit of output (controlLogic.controller.yMax)";
  equation
    connect(earth.frame_world, world.frame_b) annotation(Line(visible = true, origin = {-77.263, -5}, points = {{27.737, 0}, {-27.737, -0}}, color = {95, 95, 95}));
    connect(distance.frame_b, world.frame_b) annotation(Line(visible = true, origin = {-99.006, 15}, points = {{4.006, 20}, {-0.994, 20}, {-0.994, -20}, {-5.994, -20}}, color = {95, 95, 95}));
    connect(satellite_position_over_earth.frame_resolve, earth.frame_earth) annotation(Line(visible = true, origin = {1.667, 4.967}, points = {{3.333, 19.933}, {3.333, -9.967}, {-6.667, -9.967}}, color = {95, 95, 95}));
    connect(distance.frame_a, satellite.frame_a) annotation(Line(visible = true, origin = {-71.667, 29.867}, points = {{-3.333, 5.133}, {1.667, 5.133}, {1.667, -9.867}}, color = {95, 95, 95}));
    connect(satellite.frame_a, satellite_position_over_earth.frame_a) annotation(Line(visible = true, origin = {-48.333, 29.867}, points = {{-21.667, -9.867}, {-21.667, 5.133}, {43.333, 5.133}}, color = {95, 95, 95}));
    annotation(Documentation(info = "<html>

    <p>
Model of Earth and Satellite subsystems.
</p>

<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at <a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>
<p>For the full example, open the accompanying notebook <a href=\"modelica://SatelliteControllerDeploy.SatelliteControllerDeploy/SatelliteControllerDeploy.nb\">SatelliteControllerDeploy.nb</a>.</p>
</html>"), __Wolfram(PlotSet(plots = {Plot(name = "Distance, Velocity and Force", preferred = true, subPlots = {SubPlot(curves = {Curve(x = time, y = distance.distance, legend = "Distance")}, range = Range(xmin = -3.885003885003885, xmax = 780.1087801087801, xunit = "s"), label = Label(y = "Distance [m]")), SubPlot(curves = {Curve(x = time, y = satellite.satellite.v_0[1], legend = "Velocity_X"), Curve(x = time, y = satellite.satellite.v_0[2], legend = "Velocity_Y"), Curve(x = time, y = satellite.satellite.v_0[3], legend = "Velocity_Z")}, range = Range(xmin = -3.885003885003885, xmax = 780.1087801087801, xunit = "s"), label = Label(y = "Velocity [m/s]"))}), Plot(name = "Parametric Plots of Position", subPlots = {SubPlot(curves = {Curve(x = satellite_position_over_earth.r[1], y = satellite_position_over_earth.r[2], legend = "Y position to X position")}, label = Label(x = "X Position", y = "Y Position")), SubPlot(curves = {Curve(x = satellite_position_over_earth.r[1], y = satellite_position_over_earth.r[3], legend = "Z position to X position")}, label = Label(x = "X Position", y = "Z Position")), SubPlot(curves = {Curve(x = satellite_position_over_earth.r[2], y = satellite_position_over_earth.r[3], legend = "Z position to Y position")}, label = Label(x = "Y Position", y = "Z Position"))}), Plot(name = "Target and Actual Altitude", subPlots = {SubPlot(curves = {Curve(x = time, y = satelliteHeight, legend = "Satellite Altitude"), Curve(x = time, y = targetAltitude, legend = "Target Altitude")})}), Plot(name = "Altitude", subPlots = {SubPlot(curves = {Curve(x = time, y = satelliteHeight, legend = "Altitude")})})})), preferredView = "info", Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), Diagram(coordinateSystem(extent = {{-140, -45}, {32.317, 60}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})), experiment(StartTime = 0.0, StopTime = 50000.0, NumberOfIntervals = -1, Algorithm = "dassl", Tolerance = 1e-06));
  end SatelliteControllerDeployWithoutFMU;
  annotation(Documentation(info = "<html><div>
<div>
<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at <a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>
<p>For the full example, open the accompanying notebook <a href=\"modelica://SatelliteControllerDeploy/SatelliteControllerDeploy.nb\">SatelliteControllerDeploy.nb</a>.</p>
</div>
</div></html>"), preferredView = "info", uses(Modelica(version = "4.0.0")), __Wolfram(openClass = SatelliteControllerDeploy.SatelliteControllerDeploy), Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90.0, -90.0}, {90.0, 90.0}}, radius = 25), Text(visible = true, origin = {12.0, 20.8333}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-102.0, -70.8333}, {78.0, 29.1667}}, textString = "Ex", fontName = "Arial")}));
end SatelliteControllerDeploy;
